@@ -1,7 +1,7 @@
 /*
   FUSE: Filesystem in Userspace
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
-  Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
 
   This program can be distributed under the terms of the GNU LGPLv2.
   See the file COPYING.LIB.
@@ -351,18 +351,19 @@ fuse_mount_fusermount (const char *mountpoint, const char *opts)
 static
 #endif
 int
-fuse_mnt_umount (const char *progname, const char *mnt, int lazy)
+fuse_mnt_umount (const char *progname, const char *abs_mnt,
+                 const char *rel_mnt, int lazy)
 {
         int res;
         int status;
         sigset_t blockmask;
         sigset_t oldmask;
 
-        if (!mtab_needs_update (mnt)) {
-                res = umount2 (mnt, lazy ? 2 : 0);
+        if (!mtab_needs_update (abs_mnt)) {
+                res = umount2 (rel_mnt, lazy ? 2 : 0);
                 if (res == -1)
                         GFFUSE_LOGERR ("%s: failed to unmount %s: %s",
-                                       progname, mnt, strerror (errno));
+                                       progname, abs_mnt, strerror (errno));
                 return res;
         }
 
@@ -383,7 +384,7 @@ fuse_mnt_umount (const char *progname, const char *mnt, int lazy)
         if (res == 0) {
                 sigprocmask (SIG_SETMASK, &oldmask, NULL);
                 setuid (geteuid ());
-                execl ("/bin/umount", "/bin/umount", "-i", mnt,
+                execl ("/bin/umount", "/bin/umount", "-i", rel_mnt,
                       lazy ? "-l" : NULL, NULL);
                 GFFUSE_LOGERR ("%s: failed to execute /bin/umount: %s",
                                progname, strerror (errno));
@@ -483,7 +484,7 @@ gf_fuse_unmount (const char *mountpoint, int fd)
         }
 
         if (geteuid () == 0) {
-                fuse_mnt_umount ("fuse", mountpoint, 1);
+                fuse_mnt_umount ("fuse", mountpoint, mountpoint, 1);
                 return;
         }
 
@@ -588,11 +589,41 @@ fuse_mount_sys (const char *mountpoint, char *fsname, char *mnt_param)
         return fd;
 }
 
+static char *
+escape (char *s)
+{
+        size_t len = 0;
+        char *p = NULL;
+        char *q = NULL;
+        char *e = NULL;
+
+        for (p = s; *p; p++) {
+                if (*p == ',')
+                       len++;
+                len++;
+        }
+
+        e = CALLOC (1, len + 1);
+        if (!e)
+                return NULL;
+
+        for (p = s, q = e; *p; p++, q++) {
+                if (*p == ',') {
+                        *q = '\\';
+                        q++;
+                }
+                *q = *p;
+        }
+
+        return e;
+}
+
 int
 gf_fuse_mount (const char *mountpoint, char *fsname, char *mnt_param)
 {
         int fd = -1, rv = -1;
         char *fm_mnt_params = NULL, *p = NULL;
+        char *efsname = NULL;
 
         fd = fuse_mount_sys (mountpoint, fsname, mnt_param);
         if (fd == -1) {
@@ -601,10 +632,16 @@ gf_fuse_mount (const char *mountpoint, char *fsname, char *mnt_param)
                         "retry to mount via fusermount",
                         strerror (errno));
 
+                efsname = escape (fsname);
+                if (!efsname) {
+                        GFFUSE_LOGERR ("Out of memory");
+
+                        return -1;
+                }
                 rv = asprintf (&fm_mnt_params,
                                "%s,fsname=%s,nonempty,subtype=glusterfs",
-                               mnt_param, fsname);
-
+                               mnt_param, efsname);
+                FREE (efsname);
                 if (rv == -1) {
                         GFFUSE_LOGERR ("Out of memory");
 
