@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -34,6 +34,7 @@ typedef struct {
 	char wind;
 	call_frame_t *frame;
 	glusterfs_fop_t fop;
+       struct mem_pool *stub_mem_pool;    /* pointer to stub mempool in glusterfs ctx */
 
 	union {
 		/* lookup */
@@ -46,9 +47,9 @@ typedef struct {
 			fop_lookup_cbk_t fn;
 			int32_t op_ret, op_errno;
 			inode_t *inode;
-			struct stat buf;
+			struct iatt buf;
 			dict_t *dict;
-                        struct stat postparent;
+                        struct iatt postparent;
 		} lookup_cbk;
 
 		/* stat */
@@ -59,7 +60,7 @@ typedef struct {
 		struct {
 			fop_stat_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat buf;
+			struct iatt buf;
 		} stat_cbk;
 
 		/* fstat */
@@ -70,7 +71,7 @@ typedef struct {
 		struct {
 			fop_fstat_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat buf;
+			struct iatt buf;
 		} fstat_cbk;
 
 		/* truncate */
@@ -82,8 +83,8 @@ typedef struct {
 		struct {
 			fop_truncate_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat prebuf;
-                        struct stat postbuf;
+			struct iatt prebuf;
+                        struct iatt postbuf;
 		} truncate_cbk;
 
 		/* ftruncate */
@@ -95,8 +96,8 @@ typedef struct {
 		struct {
 			fop_ftruncate_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat prebuf;
-                        struct stat postbuf;
+			struct iatt prebuf;
+                        struct iatt postbuf;
 		} ftruncate_cbk;
 
 		/* access */
@@ -120,7 +121,7 @@ typedef struct {
 			fop_readlink_cbk_t fn;
 			int32_t op_ret, op_errno;
 			const char *buf;
-                        struct stat sbuf;
+                        struct iatt sbuf;
 		} readlink_cbk;
 
 		/* mknod */
@@ -129,14 +130,15 @@ typedef struct {
 			loc_t loc;
 			mode_t mode;
 			dev_t rdev;
+                        dict_t *params;
 		} mknod;
 		struct {
 			fop_mknod_cbk_t fn;
 			int32_t op_ret, op_errno;
 			inode_t *inode;
-			struct stat buf;
-                        struct stat preparent;
-                        struct stat postparent;
+			struct iatt buf;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} mknod_cbk;
 
 		/* mkdir */
@@ -144,14 +146,15 @@ typedef struct {
 			fop_mkdir_t fn;
 			loc_t loc;
 			mode_t mode;
+                        dict_t *params;
 		} mkdir;
 		struct {
 			fop_mkdir_cbk_t fn;
 			int32_t op_ret, op_errno;
 			inode_t *inode;
-			struct stat buf;
-                        struct stat preparent;
-                        struct stat postparent;
+			struct iatt buf;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} mkdir_cbk;
 
 		/* unlink */
@@ -162,20 +165,21 @@ typedef struct {
 		struct {
 			fop_unlink_cbk_t fn;
 			int32_t op_ret, op_errno;
-                        struct stat preparent;
-                        struct stat postparent;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} unlink_cbk;
 
 		/* rmdir */
 		struct {
 			fop_rmdir_t fn;
 			loc_t loc;
+                        int   flags;
 		} rmdir;
 		struct {
 			fop_rmdir_cbk_t fn;
 			int32_t op_ret, op_errno;
-                        struct stat preparent;
-                        struct stat postparent;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} rmdir_cbk;
 
 		/* symlink */
@@ -183,14 +187,15 @@ typedef struct {
 			fop_symlink_t fn;
 			const char *linkname;
 			loc_t loc;
+                        dict_t *params;
 		} symlink;
 		struct {
 			fop_symlink_cbk_t fn;
 			int32_t op_ret, op_errno;
 			inode_t *inode;
-			struct stat buf;
-                        struct stat preparent;
-                        struct stat postparent;
+			struct iatt buf;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} symlink_cbk;
 
 		/* rename */
@@ -202,11 +207,11 @@ typedef struct {
 		struct {
 			fop_rename_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat buf;
-                        struct stat preoldparent;
-                        struct stat postoldparent;
-                        struct stat prenewparent;
-                        struct stat postnewparent;
+			struct iatt buf;
+                        struct iatt preoldparent;
+                        struct iatt postoldparent;
+                        struct iatt prenewparent;
+                        struct iatt postnewparent;
 		} rename_cbk;
 
 		/* link */
@@ -219,9 +224,9 @@ typedef struct {
 			fop_link_cbk_t fn;
 			int32_t op_ret, op_errno;
 			inode_t *inode;
-			struct stat buf;
-                        struct stat preparent;
-                        struct stat postparent;
+			struct iatt buf;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} link_cbk;
 
 		/* create */
@@ -231,15 +236,16 @@ typedef struct {
 			int32_t flags;
 			mode_t mode;
 			fd_t *fd;
+                        dict_t *params;
 		} create;
 		struct {
 			fop_create_cbk_t fn;
 			int32_t op_ret, op_errno;
 			fd_t *fd;
 			inode_t *inode;
-			struct stat buf;
-                        struct stat preparent;
-                        struct stat postparent;
+			struct iatt buf;
+                        struct iatt preparent;
+                        struct iatt postparent;
 		} create_cbk;
 
 		/* open */
@@ -269,7 +275,7 @@ typedef struct {
 			int32_t op_errno;
 			struct iovec *vector;
 			int32_t count;
-			struct stat stbuf;
+			struct iatt stbuf;
 			struct iobref *iobref;
 		} readv_cbk;
 
@@ -285,8 +291,8 @@ typedef struct {
 		struct {
 			fop_writev_cbk_t fn;
 			int32_t op_ret, op_errno;
-                        struct stat prebuf;
-			struct stat postbuf;
+                        struct iatt prebuf;
+			struct iatt postbuf;
 		} writev_cbk;
 
 		/* flush */
@@ -308,8 +314,8 @@ typedef struct {
 		struct {
 			fop_fsync_cbk_t fn;
 			int32_t op_ret, op_errno;
-                        struct stat prebuf;
-                        struct stat postbuf;
+                        struct iatt prebuf;
+                        struct iatt postbuf;
 		} fsync_cbk;
 
 		/* opendir */
@@ -324,35 +330,6 @@ typedef struct {
 			fd_t *fd;
 		} opendir_cbk;
 
-		/* getdents */
-		struct {
-			fop_getdents_t fn;
-			fd_t *fd;
-			size_t size;
-			off_t off;
-			int32_t flag;
-		} getdents;
-		struct {
-			fop_getdents_cbk_t fn;
-			int32_t op_ret;
-			int32_t op_errno;
-			dir_entry_t entries;
-			int32_t count;
-		} getdents_cbk;
-
-		/* setdents */
-		struct {
-			fop_setdents_t fn;
-			fd_t *fd;
-			int32_t flags;
-			dir_entry_t entries;
-			int32_t count;
-		} setdents;
-		struct {
-			fop_setdents_cbk_t fn;
-			int32_t op_ret;
-			int32_t op_errno;
-		} setdents_cbk;
 
 		/* fsyncdir */
 		struct {
@@ -440,12 +417,12 @@ typedef struct {
 			fop_lk_t fn;
 			fd_t *fd;
 			int32_t cmd;
-			struct flock lock;
+			struct gf_flock lock;
 		} lk;
 		struct {
 			fop_lk_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct flock lock;
+			struct gf_flock lock;
 		} lk_cbk;
 
 		/* inodelk */
@@ -454,7 +431,7 @@ typedef struct {
                         const char *volume;
 			loc_t loc;
 			int32_t cmd;
-			struct flock lock;
+			struct gf_flock lock;
 		} inodelk;
 
 		struct {
@@ -468,7 +445,7 @@ typedef struct {
                         const char *volume;
 			fd_t *fd;
 			int32_t cmd;
-			struct flock lock;
+			struct gf_flock lock;
 		} finodelk;
 
 		struct {
@@ -532,19 +509,6 @@ typedef struct {
 			gf_dirent_t entries;
 		} readdirp_cbk;
 
-		/* checksum */
-		struct {
-			fop_checksum_t fn;
-			loc_t loc;
-			int32_t flags;
-		} checksum;
-		struct {
-			fop_checksum_cbk_t fn;
-			int32_t op_ret, op_errno;
-			uint8_t *file_checksum;
-			uint8_t *dir_checksum;
-		} checksum_cbk;
-
 		/* rchecksum */
 		struct {
 			fop_rchecksum_t fn;
@@ -587,56 +551,34 @@ typedef struct {
 			dict_t *xattr;
 		} fxattrop_cbk;
 
-		struct {
-			fop_lock_notify_t fn;
-			loc_t loc;
-			int32_t timeout;
-		} lock_notify;
-		struct {
-			fop_lock_notify_cbk_t fn;
-			int32_t op_ret;
-			int32_t op_errno;
-		} lock_notify_cbk;
-
-		struct {
-			fop_lock_fnotify_t fn;
-			fd_t *fd;
-			int32_t timeout;
-		} lock_fnotify;
-		struct {
-			fop_lock_fnotify_cbk_t fn;
-			int32_t op_ret;
-			int32_t op_errno;
-		} lock_fnotify_cbk;
-                
                 /* setattr */
                 struct {
                         fop_setattr_t fn;
                         loc_t loc;
-                        struct stat stbuf;
+                        struct iatt stbuf;
                         int32_t valid;
                 } setattr;
                 struct {
                         fop_setattr_cbk_t fn;
                         int32_t op_ret;
                         int32_t op_errno;
-                        struct stat statpre;
-                        struct stat statpost;
+                        struct iatt statpre;
+                        struct iatt statpost;
                 } setattr_cbk;
 
                 /* fsetattr */
                 struct {
                         fop_fsetattr_t fn;
                         fd_t *fd;
-                        struct stat stbuf;
+                        struct iatt stbuf;
                         int32_t valid;
                 } fsetattr;
                 struct {
                         fop_fsetattr_cbk_t fn;
                         int32_t op_ret;
                         int32_t op_errno;
-                        struct stat statpre;
-                        struct stat statpost;
+                        struct iatt statpre;
+                        struct iatt statpost;
                 } fsetattr_cbk;
 
 	} args;
@@ -654,9 +596,9 @@ fop_lookup_cbk_stub (call_frame_t *frame,
 		     int32_t op_ret,
 		     int32_t op_errno,
 		     inode_t *inode,
-		     struct stat *buf,
+		     struct iatt *buf,
                      dict_t *dict,
-                     struct stat *postparent);
+                     struct iatt *postparent);
 call_stub_t *
 fop_stat_stub (call_frame_t *frame,
 	       fop_stat_t fn,
@@ -666,7 +608,7 @@ fop_stat_cbk_stub (call_frame_t *frame,
 		   fop_stat_cbk_t fn,
 		   int32_t op_ret,
 		   int32_t op_errno,
-		   struct stat *buf);
+		   struct iatt *buf);
 call_stub_t *
 fop_fstat_stub (call_frame_t *frame,
 		fop_fstat_t fn,
@@ -676,7 +618,7 @@ fop_fstat_cbk_stub (call_frame_t *frame,
 		    fop_fstat_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-		    struct stat *buf);
+		    struct iatt *buf);
 
 call_stub_t *
 fop_truncate_stub (call_frame_t *frame,
@@ -689,8 +631,8 @@ fop_truncate_cbk_stub (call_frame_t *frame,
 		       fop_truncate_cbk_t fn,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       struct stat *prebuf,
-                       struct stat *postbuf);
+		       struct iatt *prebuf,
+                       struct iatt *postbuf);
 
 call_stub_t *
 fop_ftruncate_stub (call_frame_t *frame,
@@ -703,8 +645,8 @@ fop_ftruncate_cbk_stub (call_frame_t *frame,
 			fop_ftruncate_cbk_t fn,
 			int32_t op_ret,
 			int32_t op_errno,
-			struct stat *prebuf,
-                        struct stat *postbuf);
+			struct iatt *prebuf,
+                        struct iatt *postbuf);
 
 call_stub_t *
 fop_access_stub (call_frame_t *frame,
@@ -730,14 +672,11 @@ fop_readlink_cbk_stub (call_frame_t *frame,
 		       int32_t op_ret,
 		       int32_t op_errno,
 		       const char *path,
-                       struct stat *buf);
+                       struct iatt *buf);
 
 call_stub_t *
-fop_mknod_stub (call_frame_t *frame,
-		fop_mknod_t fn,
-		loc_t *loc,
-		mode_t mode,
-		dev_t rdev);
+fop_mknod_stub (call_frame_t *frame, fop_mknod_t fn,
+		loc_t *loc, mode_t mode, dev_t rdev, dict_t *params);
 
 call_stub_t *
 fop_mknod_cbk_stub (call_frame_t *frame,
@@ -745,15 +684,13 @@ fop_mknod_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-                    struct stat *buf,
-                    struct stat *preparent,
-                    struct stat *postparent);
+                    struct iatt *buf,
+                    struct iatt *preparent,
+                    struct iatt *postparent);
 
 call_stub_t *
-fop_mkdir_stub (call_frame_t *frame,
-		fop_mkdir_t fn,
-		loc_t *loc,
-		mode_t mode);
+fop_mkdir_stub (call_frame_t *frame, fop_mkdir_t fn,
+		loc_t *loc, mode_t mode, dict_t *params);
 
 call_stub_t *
 fop_mkdir_cbk_stub (call_frame_t *frame,
@@ -761,9 +698,9 @@ fop_mkdir_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-                    struct stat *buf,
-                    struct stat *preparent,
-                    struct stat *postparent);
+                    struct iatt *buf,
+                    struct iatt *preparent,
+                    struct iatt *postparent);
 
 call_stub_t *
 fop_unlink_stub (call_frame_t *frame,
@@ -775,27 +712,24 @@ fop_unlink_cbk_stub (call_frame_t *frame,
 		     fop_unlink_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-                     struct stat *preparent,
-                     struct stat *postparent);
+                     struct iatt *preparent,
+                     struct iatt *postparent);
 
 call_stub_t *
-fop_rmdir_stub (call_frame_t *frame,
-		fop_rmdir_t fn,
-		loc_t *loc);
+fop_rmdir_stub (call_frame_t *frame, fop_rmdir_t fn,
+		loc_t *loc, int flags);
 
 call_stub_t *
 fop_rmdir_cbk_stub (call_frame_t *frame,
 		    fop_rmdir_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-                    struct stat *preparent,
-                    struct stat *postparent);
+                    struct iatt *preparent,
+                    struct iatt *postparent);
 
 call_stub_t *
-fop_symlink_stub (call_frame_t *frame,
-		  fop_symlink_t fn,
-		  const char *linkname,
-		  loc_t *loc);
+fop_symlink_stub (call_frame_t *frame, fop_symlink_t fn,
+		  const char *linkname, loc_t *loc, dict_t *params);
 
 call_stub_t *
 fop_symlink_cbk_stub (call_frame_t *frame,
@@ -803,9 +737,9 @@ fop_symlink_cbk_stub (call_frame_t *frame,
 		      int32_t op_ret,
 		      int32_t op_errno,
 		      inode_t *inode,
-                      struct stat *buf,
-                      struct stat *preparent,
-                      struct stat *postparent);
+                      struct iatt *buf,
+                      struct iatt *preparent,
+                      struct iatt *postparent);
 
 call_stub_t *
 fop_rename_stub (call_frame_t *frame,
@@ -818,11 +752,11 @@ fop_rename_cbk_stub (call_frame_t *frame,
 		     fop_rename_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-		     struct stat *buf,
-                     struct stat *preoldparent,
-                     struct stat *postoldparent,
-                     struct stat *prenewparent,
-                     struct stat *postnewparent);
+		     struct iatt *buf,
+                     struct iatt *preoldparent,
+                     struct iatt *postoldparent,
+                     struct iatt *prenewparent,
+                     struct iatt *postnewparent);
 
 call_stub_t *
 fop_link_stub (call_frame_t *frame,
@@ -836,16 +770,14 @@ fop_link_cbk_stub (call_frame_t *frame,
 		   int32_t op_ret,
 		   int32_t op_errno,
 		   inode_t *inode,
-                   struct stat *buf,
-                   struct stat *preparent,
-                   struct stat *postparent);
+                   struct iatt *buf,
+                   struct iatt *preparent,
+                   struct iatt *postparent);
 
 call_stub_t *
-fop_create_stub (call_frame_t *frame,
-		 fop_create_t fn,
-		 loc_t *loc,
-		 int32_t flags,
-		 mode_t mode, fd_t *fd);
+fop_create_stub (call_frame_t *frame, fop_create_t fn,
+		 loc_t *loc, int32_t flags, mode_t mode,
+                 fd_t *fd, dict_t *params);
 
 call_stub_t *
 fop_create_cbk_stub (call_frame_t *frame,
@@ -854,9 +786,9 @@ fop_create_cbk_stub (call_frame_t *frame,
 		     int32_t op_errno,
 		     fd_t *fd,
 		     inode_t *inode,
-		     struct stat *buf,
-                     struct stat *preparent,
-                     struct stat *postparent);
+		     struct iatt *buf,
+                     struct iatt *preparent,
+                     struct iatt *postparent);
 
 call_stub_t *
 fop_open_stub (call_frame_t *frame,
@@ -887,7 +819,7 @@ fop_readv_cbk_stub (call_frame_t *frame,
 		    int32_t op_errno,
 		    struct iovec *vector,
 		    int32_t count,
-		    struct stat *stbuf,
+		    struct iatt *stbuf,
                     struct iobref *iobref);
 
 call_stub_t *
@@ -904,8 +836,8 @@ fop_writev_cbk_stub (call_frame_t *frame,
 		     fop_writev_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-                     struct stat *prebuf,
-                     struct stat *postbuf);
+                     struct iatt *prebuf,
+                     struct iatt *postbuf);
 
 call_stub_t *
 fop_flush_stub (call_frame_t *frame,
@@ -929,8 +861,8 @@ fop_fsync_cbk_stub (call_frame_t *frame,
 		    fop_fsync_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-                    struct stat *prebuf,
-                    struct stat *postbuf);
+                    struct iatt *prebuf,
+                    struct iatt *postbuf);
 
 call_stub_t *
 fop_opendir_stub (call_frame_t *frame,
@@ -944,36 +876,6 @@ fop_opendir_cbk_stub (call_frame_t *frame,
 		      int32_t op_errno,
 		      fd_t *fd);
 
-call_stub_t *
-fop_getdents_stub (call_frame_t *frame,
-		   fop_getdents_t fn,
-		   fd_t *fd,
-		   size_t size,
-		   off_t off,
-		   int32_t flag);
-
-call_stub_t *
-fop_getdents_cbk_stub (call_frame_t *frame,
-		       fop_getdents_cbk_t fn,
-		       int32_t op_ret,
-		       int32_t op_errno,
-		       dir_entry_t *entries,
-		       int32_t count);
-
-call_stub_t *
-fop_setdents_stub (call_frame_t *frame,
-		   fop_setdents_t fn,
-		   fd_t *fd,
-		   int32_t flags,
-		   dir_entry_t *entries,
-		   int32_t count);
-
-call_stub_t *
-fop_setdents_cbk_stub (call_frame_t *frame,
-		       fop_setdents_cbk_t fn,
-		       int32_t op_ret,
-		       int32_t op_errno);
-
 call_stub_t *
 fop_fsyncdir_stub (call_frame_t *frame,
 		   fop_fsyncdir_t fn,
@@ -1066,24 +968,24 @@ fop_lk_stub (call_frame_t *frame,
 	     fop_lk_t fn,
 	     fd_t *fd,
 	     int32_t cmd,
-	     struct flock *lock);
+	     struct gf_flock *lock);
 
 call_stub_t *
 fop_lk_cbk_stub (call_frame_t *frame,
 		 fop_lk_cbk_t fn,
 		 int32_t op_ret,
 		 int32_t op_errno,
-		 struct flock *lock);
+		 struct gf_flock *lock);
 
 call_stub_t *
 fop_inodelk_stub (call_frame_t *frame, fop_inodelk_t fn,
 		  const char *volume, loc_t *loc, int32_t cmd, 
-                  struct flock *lock);
+                  struct gf_flock *lock);
 
 call_stub_t *
 fop_finodelk_stub (call_frame_t *frame, fop_finodelk_t fn,
 		   const char *volume, fd_t *fd, int32_t cmd, 
-                   struct flock *lock);
+                   struct gf_flock *lock);
 
 call_stub_t *
 fop_entrylk_stub (call_frame_t *frame, fop_entrylk_t fn,
@@ -1139,20 +1041,6 @@ fop_readdir_cbk_stub (call_frame_t *frame,
 		      int32_t op_errno,
 		      gf_dirent_t *entries);
 
-call_stub_t *
-fop_checksum_stub (call_frame_t *frame,
-		   fop_checksum_t fn,
-		   loc_t *loc,
-		   int32_t flags);
-
-call_stub_t *
-fop_checksum_cbk_stub (call_frame_t *frame,
-		       fop_checksum_cbk_t fn,
-		       int32_t op_ret,
-		       int32_t op_errno,
-		       uint8_t *file_checksum,
-		       uint8_t *dir_checksum);
-
 call_stub_t *
 fop_rchecksum_stub (call_frame_t *frame,
                     fop_rchecksum_t fn,
@@ -1193,35 +1081,11 @@ fop_fxattrop_stub_cbk_stub (call_frame_t *frame,
 			    int32_t op_ret,
 			    int32_t op_errno);
 
-call_stub_t *
-fop_lock_notify_stub_cbk_stub (call_frame_t *frame,
-			       fop_lock_notify_cbk_t fn,
-			       int32_t op_ret,
-			       int32_t op_errno);
-
-call_stub_t *
-fop_lock_notify_stub (call_frame_t *frame,
-		      fop_lock_notify_t fn,
-		      loc_t *loc,
-		      int32_t timeout);
-
-call_stub_t *
-fop_lock_fnotify_stub_cbk_stub (call_frame_t *frame,
-				fop_lock_fnotify_cbk_t fn,
-				int32_t op_ret,
-				int32_t op_errno);
-
-call_stub_t *
-fop_lock_fnotify_stub (call_frame_t *frame,
-		       fop_lock_fnotify_t fn,
-		       fd_t *fd,
-		       int32_t timeout);
-
 call_stub_t *
 fop_setattr_stub (call_frame_t *frame,
                   fop_setattr_t fn,
                   loc_t *loc,
-                  struct stat *stbuf,
+                  struct iatt *stbuf,
                   int32_t valid);
 
 call_stub_t *
@@ -1229,14 +1093,14 @@ fop_setattr_cbk_stub (call_frame_t *frame,
                       fop_setattr_cbk_t fn,
                       int32_t op_ret,
                       int32_t op_errno,
-                      struct stat *statpre,
-                      struct stat *statpost);
+                      struct iatt *statpre,
+                      struct iatt *statpost);
 
 call_stub_t *
 fop_fsetattr_stub (call_frame_t *frame,
                    fop_fsetattr_t fn,
                    fd_t *fd,
-                   struct stat *stbuf,
+                   struct iatt *stbuf,
                    int32_t valid);
 
 call_stub_t *
@@ -1244,8 +1108,8 @@ fop_fsetattr_cbk_stub (call_frame_t *frame,
                        fop_setattr_cbk_t fn,
                        int32_t op_ret,
                        int32_t op_errno,
-                       struct stat *statpre,
-                       struct stat *statpost);
+                       struct iatt *statpre,
+                       struct iatt *statpost);
 
 void call_resume (call_stub_t *stub);
 void call_stub_destroy (call_stub_t *stub);
