@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -40,10 +40,12 @@ data_pair_t *
 get_new_data_pair ()
 {
 	data_pair_t *data_pair_ptr = NULL;
-  
-	data_pair_ptr = (data_pair_t *) CALLOC (1, sizeof (data_pair_t));
-	ERR_ABORT (data_pair_ptr);
-  
+
+	data_pair_ptr = (data_pair_t *) GF_CALLOC (1, sizeof (data_pair_t),
+                                                   gf_common_mt_data_pair_t);
+        if (!data_pair_ptr)
+                gf_log ("dict", GF_LOG_ERROR, "memory alloc failed");
+
 	return data_pair_ptr;
 }
 
@@ -52,7 +54,7 @@ get_new_data ()
 {
 	data_t *data = NULL;
 
-	data = (data_t *) CALLOC (1, sizeof (data_t));
+	data = (data_t *) GF_CALLOC (1, sizeof (data_t), gf_common_mt_data_t);
 	if (!data) {
 		gf_log ("dict", GF_LOG_CRITICAL,
 			"calloc () returned NULL");
@@ -66,7 +68,7 @@ get_new_data ()
 dict_t *
 get_new_dict_full (int size_hint)
 {
-	dict_t *dict = CALLOC (1, sizeof (dict_t));
+	dict_t *dict = GF_CALLOC (1, sizeof (dict_t), gf_common_mt_dict_t);
 
 	if (!dict) {
 		gf_log ("dict", GF_LOG_CRITICAL,
@@ -75,7 +77,8 @@ get_new_dict_full (int size_hint)
 	}
 
 	dict->hash_size = size_hint;
-	dict->members = CALLOC (size_hint, sizeof (data_pair_t *));
+	dict->members = GF_CALLOC (size_hint, sizeof (data_pair_t *),
+                                   gf_common_mt_data_pair_t);
 
 	if (!dict->members) {
 		gf_log ("dict", GF_LOG_CRITICAL,
@@ -98,17 +101,17 @@ dict_t *
 dict_new (void)
 {
 	dict_t *dict = NULL;
-	
+
 	dict = get_new_dict_full(1);
-	
+
 	if (dict)
 		dict_ref (dict);
-	
+
 	return dict;
 }
 
 
-int32_t 
+int32_t
 is_data_equal (data_t *one,
 	       data_t *two)
 {
@@ -137,15 +140,19 @@ data_destroy (data_t *data)
 		LOCK_DESTROY (&data->lock);
 
 		if (!data->is_static) {
-			if (data->data)
-				FREE (data->data);
+			if (data->data) {
+                                if (data->is_stdalloc)
+                                        free (data->data);
+				else
+                                        GF_FREE (data->data);
+                        }
 			if (data->vec)
-				FREE (data->vec);
+				GF_FREE (data->vec);
 		}
 
 		data->len = 0xbabababa;
 		if (!data->is_const)
-			FREE (data);
+			GF_FREE (data);
 	}
 }
 
@@ -158,7 +165,8 @@ data_copy (data_t *old)
 		return NULL;
 	}
 
-	data_t *newdata = (data_t *) CALLOC (1, sizeof (*newdata));
+	data_t *newdata = (data_t *) GF_CALLOC (1, sizeof (*newdata),
+                                                gf_common_mt_data_t);
 
 	if (!newdata) {
 		gf_log ("dict", GF_LOG_CRITICAL,
@@ -168,19 +176,33 @@ data_copy (data_t *old)
 
 	if (old) {
 		newdata->len = old->len;
-		if (old->data)
+		if (old->data) {
 			newdata->data = memdup (old->data, old->len);
-		if (old->vec)
+			if (!newdata->data)
+				goto err_out;
+		}
+		if (old->vec) {
 			newdata->vec = memdup (old->vec, old->len * (sizeof (void *) +
 								     sizeof (size_t)));
-		if (!old->data && !old->vec) {
-			gf_log ("dict", GF_LOG_CRITICAL,
-				"@newdata->data || @newdata->vec got NULL from CALLOC()");
-			return NULL;
+			if (!newdata->vec)
+				goto err_out;
 		}
 	}
 
+        LOCK_INIT (&newdata->lock);
 	return newdata;
+
+ err_out:
+
+	if (newdata->data)
+		FREE (newdata->data);
+	if (newdata->vec)
+		FREE (newdata->vec);
+	GF_FREE (newdata);
+
+	gf_log ("dict", GF_LOG_CRITICAL,
+		"@newdata->data || @newdata->vec got NULL from CALLOC()");
+	return NULL;
 }
 
 static data_pair_t *
@@ -194,19 +216,19 @@ _dict_lookup (dict_t *this, char *key)
 
 	int hashval = SuperFastHash (key, strlen (key)) % this->hash_size;
 	data_pair_t *pair;
-   
+
 	for (pair = this->members[hashval]; pair != NULL; pair = pair->hash_next) {
 		if (pair->key && !strcmp (pair->key, key))
 			return pair;
 	}
-  
+
 	return NULL;
 }
 
 
 static int32_t
-_dict_set (dict_t *this, 
-	   char *key, 
+_dict_set (dict_t *this,
+	   char *key,
 	   data_t *value)
 {
 	int hashval;
@@ -216,7 +238,7 @@ _dict_set (dict_t *this,
         int ret = 0;
 
 	if (!key) {
-		ret = asprintf (&key, "ref:%p", value);
+		ret = gf_asprintf (&key, "ref:%p", value);
                 if (-1 == ret) {
                         gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                         return -1;
@@ -233,39 +255,45 @@ _dict_set (dict_t *this,
 		pair->value = data_ref (value);
 		data_unref (unref_data);
 		if (key_free)
-			FREE (key);
+			GF_FREE (key);
 		/* Indicates duplicate key */
 		return 0;
 	}
-	pair = (data_pair_t *) CALLOC (1, sizeof (*pair));
+	pair = (data_pair_t *) GF_CALLOC (1, sizeof (*pair),
+                                        gf_common_mt_data_pair_t);
 	if (!pair) {
 		gf_log ("dict", GF_LOG_CRITICAL,
 			"@pair - NULL returned by CALLOC");
 		return -1;
 	}
 
-	pair->key = (char *) CALLOC (1, strlen (key) + 1);
+	pair->key = (char *) GF_CALLOC (1, strlen (key) + 1,
+                                        gf_common_mt_char);
 	if (!pair->key) {
 		gf_log ("dict", GF_LOG_CRITICAL,
 			"@pair->key - NULL returned by CALLOC");
+		GF_FREE (pair);
+
+                if (key_free)
+                        GF_FREE (key);
 		return -1;
 	}
 
 	strcpy (pair->key, key);
 	pair->value = data_ref (value);
- 
+
 	pair->hash_next = this->members[hashval];
 	this->members[hashval] = pair;
- 
+
 	pair->next = this->members_list;
 	pair->prev = NULL;
 	if (this->members_list)
 		this->members_list->prev = pair;
 	this->members_list = pair;
 	this->count++;
-  
+
 	if (key_free)
-		FREE (key);
+		GF_FREE (key);
 	return 0;
 }
 
@@ -277,8 +305,8 @@ dict_set (dict_t *this,
 	int32_t ret;
 
 	if (!this || !value) {
-		gf_log ("dict", GF_LOG_CRITICAL,
-			"@this=%p @value=%p", this, value);
+		gf_log ("dict", GF_LOG_ERROR,
+			"@this=%p @value=%p, key=%s", this, value, key);
 		return -1;
 	}
 
@@ -299,8 +327,8 @@ dict_get (dict_t *this,
 	data_pair_t *pair;
 
 	if (!this || !key) {
-		gf_log ("dict", GF_LOG_DEBUG,
-			"@this=%p @key=%p", this, key);
+		gf_log_callingfn ("dict", GF_LOG_DEBUG,
+                                  "@this=%p key=%s", this, (key) ? key : "()");
 		return NULL;
 	}
 
@@ -312,7 +340,7 @@ dict_get (dict_t *this,
 
 	if (pair)
 		return pair->value;
-  
+
 	return NULL;
 }
 
@@ -331,16 +359,16 @@ dict_del (dict_t *this,
 	int hashval = SuperFastHash (key, strlen (key)) % this->hash_size;
 	data_pair_t *pair = this->members[hashval];
 	data_pair_t *prev = NULL;
-  
+
 	while (pair) {
 		if (strcmp (pair->key, key) == 0) {
 			if (prev)
 				prev->hash_next = pair->hash_next;
 			else
 				this->members[hashval] = pair->hash_next;
-  
+
 			data_unref (pair->value);
-  
+
 			if (pair->prev)
 				pair->prev->next = pair->next;
 			else
@@ -348,13 +376,13 @@ dict_del (dict_t *this,
 
 			if (pair->next)
 				pair->next->prev = pair->prev;
-  
-			FREE (pair->key);
-			FREE (pair);
+
+			GF_FREE (pair->key);
+			GF_FREE (pair);
 			this->count--;
 			break;
 		}
- 
+
 		prev = pair;
 		pair = pair->hash_next;
 	}
@@ -381,18 +409,20 @@ dict_destroy (dict_t *this)
 	while (prev) {
 		pair = pair->next;
 		data_unref (prev->value);
-		FREE (prev->key);
-		FREE (prev);
+		GF_FREE (prev->key);
+		GF_FREE (prev);
 		prev = pair;
 	}
 
-	FREE (this->members);
+	GF_FREE (this->members);
 
 	if (this->extra_free)
-		FREE (this->extra_free);
+		GF_FREE (this->extra_free);
+        if (this->extra_stdfree)
+                free (this->extra_stdfree);
 
 	if (!this->is_static)
-		FREE (this);
+		GF_FREE (this);
 
 	return;
 }
@@ -489,7 +519,7 @@ data_ref (data_t *this)
   .
 */
 
-int32_t 
+int32_t
 dict_serialized_length_old (dict_t *this)
 {
 
@@ -521,7 +551,7 @@ dict_serialized_length_old (dict_t *this)
 	return len;
 }
 
-int32_t 
+int32_t
 dict_serialize_old (dict_t *this, char *buf)
 {
 	if (!this || !buf) {
@@ -561,9 +591,10 @@ dict_unserialize_old (char *buf, int32_t size, dict_t **fill)
 	int32_t ret = 0;
 	int32_t cnt = 0;
 
-	if (!buf || fill == NULL || !*fill) {
+	if (!buf || !fill || !(*fill)) {
 		gf_log ("dict", GF_LOG_ERROR,
-			"@buf=%p @fill=%p @*fill=%p", buf, fill, *fill);
+			"@buf=%p @fill=%p @*fill=%p",
+                        buf, fill, (fill) ? (*fill) : NULL);
 		return NULL;
 	}
 
@@ -578,7 +609,7 @@ dict_unserialize_old (char *buf, int32_t size, dict_t **fill)
 		goto err;
 	}
 	buf += 9;
-  
+
 	if (count == 0) {
 		gf_log ("dict",
 			GF_LOG_ERROR,
@@ -590,7 +621,7 @@ dict_unserialize_old (char *buf, int32_t size, dict_t **fill)
 		data_t *value = NULL;
 		char *key = NULL;
 		uint64_t key_len, value_len;
-    
+
 		ret = sscanf (buf, "%"SCNx64":%"SCNx64"\n", &key_len, &value_len);
 		if (ret != 2) {
 			gf_log ("dict",
@@ -602,7 +633,7 @@ dict_unserialize_old (char *buf, int32_t size, dict_t **fill)
 
 		key = buf;
 		buf += key_len;
-    
+
 		value = get_new_data ();
 		value->len = value_len;
 		value->data = buf;
@@ -615,8 +646,8 @@ dict_unserialize_old (char *buf, int32_t size, dict_t **fill)
 	goto ret;
 
 err:
-	FREE (*fill);
-	*fill = NULL; 
+	GF_FREE (*fill);
+	*fill = NULL;
 
 ret:
 	return *fill;
@@ -729,7 +760,7 @@ int_to_data (int64_t value)
 		return NULL;
 	}
 
-	ret = asprintf (&data->data, "%"PRId64, value);
+	ret = gf_asprintf (&data->data, "%"PRId64, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -750,7 +781,7 @@ data_from_int64 (int64_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%"PRId64, value);
+	ret = gf_asprintf (&data->data, "%"PRId64, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -771,7 +802,7 @@ data_from_int32 (int32_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%"PRId32, value);
+	ret = gf_asprintf (&data->data, "%"PRId32, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -793,7 +824,7 @@ data_from_int16 (int16_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%"PRId16, value);
+	ret = gf_asprintf (&data->data, "%"PRId16, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -815,7 +846,7 @@ data_from_int8 (int8_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%d", value);
+	ret = gf_asprintf (&data->data, "%d", value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -837,7 +868,7 @@ data_from_uint64 (uint64_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%"PRIu64, value);
+	ret = gf_asprintf (&data->data, "%"PRIu64, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -862,7 +893,7 @@ data_from_double (double value)
 		return NULL;
 	}
 
-	ret = asprintf (&data->data, "%f", value);
+	ret = gf_asprintf (&data->data, "%f", value);
 	if (ret == -1) {
 		gf_log ("dict", GF_LOG_CRITICAL,
 			"@data - allocation failed by ASPRINTF");
@@ -885,7 +916,7 @@ data_from_uint32 (uint32_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%"PRIu32, value);
+	ret = gf_asprintf (&data->data, "%"PRIu32, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -908,7 +939,7 @@ data_from_uint16 (uint16_t value)
 			"@data - NULL returned by CALLOC");
 		return NULL;
 	}
-	ret = asprintf (&data->data, "%"PRIu16, value);
+	ret = gf_asprintf (&data->data, "%"PRIu16, value);
         if (-1 == ret) {
                 gf_log ("dict", GF_LOG_ERROR, "asprintf failed");
                 return NULL;
@@ -1007,6 +1038,24 @@ data_from_dynstr (char *value)
 	return data;
 }
 
+data_t *
+data_from_dynmstr (char *value)
+{
+	if (!value) {
+		gf_log ("dict", GF_LOG_CRITICAL,
+			"@value=%p", value);
+		return NULL;
+	}
+
+	data_t *data = get_new_data ();
+
+	data->len = strlen (value) + 1;
+	data->data = value;
+        data->is_stdalloc = 1;
+
+	return data;
+}
+
 data_t *
 data_from_dynptr (void *value, int32_t len)
 {
@@ -1043,7 +1092,9 @@ data_to_int64 (data_t *data)
 		return -1;
 
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 	return (int64_t) strtoull (str, NULL, 0);
@@ -1056,7 +1107,9 @@ data_to_int32 (data_t *data)
 		return -1;
 
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 
@@ -1070,7 +1123,9 @@ data_to_int16 (data_t *data)
 		return -1;
 
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 
@@ -1085,7 +1140,9 @@ data_to_int8 (data_t *data)
 		return -1;
 
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 
@@ -1099,7 +1156,9 @@ data_to_uint64 (data_t *data)
 	if (!data)
 		return -1;
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 
@@ -1113,7 +1172,9 @@ data_to_uint32 (data_t *data)
 		return -1;
 
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 
@@ -1127,7 +1188,9 @@ data_to_uint16 (data_t *data)
 		return -1;
 
 	char *str = alloca (data->len + 1);
-	ERR_ABORT (str);
+        if (!str)
+                return -1;
+
 	memcpy (str, data->data, data->len);
 	str[data->len] = '\0';
 
@@ -1138,7 +1201,7 @@ char *
 data_to_str (data_t *data)
 {
 	if (!data) {
-		gf_log ("dict", GF_LOG_CRITICAL,
+		gf_log ("dict", GF_LOG_WARNING,
 			"@data=%p", data);
 		return NULL;
 	}
@@ -1173,18 +1236,20 @@ dict_foreach (dict_t *dict,
 			 void *data),
 	      void *data)
 {
-	if (!data) {
-		gf_log ("dict", GF_LOG_CRITICAL,
-			"@data=%p", data);
-		return;
-	}
+        if (!data || !dict) {
+                gf_log_callingfn ("dict", GF_LOG_CRITICAL,
+                        "@data=%p, @dict=%p", data, dict);
+                return;
+        }
 
-	data_pair_t *pairs = dict->members_list;
+        data_pair_t *pairs = dict->members_list;
+        data_pair_t *next = NULL;
 
-	while (pairs) {
-		fn (dict, pairs->key, pairs->value, data);
-		pairs = pairs->next;
-	}
+        while (pairs) {
+                next = pairs->next;
+                fn (dict, pairs->key, pairs->value, data);
+                pairs = next;
+        }
 }
 
 
@@ -1241,7 +1306,7 @@ fail:
 
 /**
  * Common cleaned up interface:
- * 
+ *
  * Return value:  0   success
  *               -val error, val = errno
  */
@@ -1269,7 +1334,7 @@ dict_get_with_ref (dict_t *this, char *key, data_t **data)
 		*data = data_ref (pair->value);
 	}
 
-err:  
+err:
 	return ret;
 }
 
@@ -1530,7 +1595,7 @@ dict_get_int8 (dict_t *this, char *key, int8_t *val)
 	}
 
 	ret = _data_to_int8 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1573,7 +1638,7 @@ dict_get_int16 (dict_t *this, char *key, int16_t *val)
 	}
 
 	ret = _data_to_int16 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1616,7 +1681,7 @@ dict_get_int32 (dict_t *this, char *key, int32_t *val)
 	}
 
 	ret = _data_to_int32 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1659,7 +1724,7 @@ dict_get_int64 (dict_t *this, char *key, int64_t *val)
 	}
 
 	ret = _data_to_int64 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1702,7 +1767,7 @@ dict_get_uint16 (dict_t *this, char *key, uint16_t *val)
 	}
 
 	ret = _data_to_uint16 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1745,7 +1810,7 @@ dict_get_uint32 (dict_t *this, char *key, uint32_t *val)
 	}
 
 	ret = _data_to_uint32 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1789,7 +1854,7 @@ dict_get_uint64 (dict_t *this, char *key, uint64_t *val)
 	}
 
 	ret = _data_to_uint64 (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1832,7 +1897,7 @@ dict_get_double (dict_t *this, char *key, double *val)
 	}
 
 	ret = _data_to_double (data, val);
-    
+
 err:
 	if (data)
 		data_unref (data);
@@ -1914,7 +1979,7 @@ dict_get_ptr (dict_t *this, char *key, void **ptr)
 		goto err;
 	}
 
-err: 
+err:
 	if (data)
 		data_unref (data);
 
@@ -1960,7 +2025,7 @@ dict_get_str (dict_t *this, char *key, char **str)
 	}
 	*str = data->data;
 
-err: 
+err:
 	if (data)
 		data_unref (data);
 
@@ -2003,6 +2068,27 @@ err:
 	return ret;
 }
 
+/*
+        for malloced strings we should do a free instead of GF_FREE
+*/
+int
+dict_set_dynmstr (dict_t *this, char *key, char *str)
+{
+	data_t * data = NULL;
+	int      ret  = 0;
+
+	data = data_from_dynmstr (str);
+	if (!data) {
+		ret = -EINVAL;
+		goto err;
+	}
+
+	ret = dict_set (this, key, data);
+
+err:
+	return ret;
+}
+
 
 int
 dict_get_bin (dict_t *this, char *key, void **bin)
@@ -2024,7 +2110,7 @@ dict_get_bin (dict_t *this, char *key, void **bin)
 	}
 	*bin = data->data;
 
-err: 
+err:
 	if (data)
 		data_unref (data);
 
@@ -2087,6 +2173,67 @@ err:
 	return ret;
 }
 
+
+/**
+ * dict_get_str_boolean - get a boolean value based on string representation.
+ *
+ * @this        : dictionary
+ * @key         : dictionary key queried
+ * @default_val : default value if key not found
+ *
+ * @return      : @default_val if key not found
+ *              : boolean interpretation of @this[@key] if it makes sense
+ *                (ie., "on", "true", "enable" ...)
+ *              : -1 if error occurs or @this[@key] doesn't make sens as
+ *                  boolean
+ *
+ *   So if you query a boolean option, then via @default_val you can choose
+ *   between following patterns:
+ *
+ *   - fall back to _gf_false if @key is not set  [@default_val = 0]
+ *   - fall back to _gf_true if @key is not set   [@default_val = 1]
+ *   - regard as failure if @key is not set       [@default_val = -1]
+ *   - handle specially (not as error) if @key is not set
+ *                                                [@default_val = anything else]
+ */
+
+int
+dict_get_str_boolean (dict_t *this, char *key, int default_val)
+{
+        data_t       *data = NULL;
+        gf_boolean_t  boo = _gf_false;
+        int           ret  = 0;
+
+        ret = dict_get_with_ref (this, key, &data);
+        if (ret < 0) {
+                if (ret == -ENOENT)
+                        ret = default_val;
+                else
+                        ret = -1;
+                goto err;
+        }
+
+        GF_ASSERT (data);
+
+        if (!data->data) {
+                ret = -1;
+                goto err;
+        }
+
+        ret = gf_string2boolean (data->data, &boo);
+        if (ret == -1)
+                goto err;
+
+        ret = boo;
+
+err:
+        if (data)
+                data_unref (data);
+
+        return ret;
+}
+
+
 /**
  * Serialization format:
  *  -------- --------  --------  ----------- -------------
@@ -2129,7 +2276,7 @@ _dict_serialized_length (dict_t *this)
 
 	while (count) {
 		if (!pair) {
-			gf_log ("dict", GF_LOG_ERROR, 
+			gf_log ("dict", GF_LOG_ERROR,
 				"less than count data pairs found!");
 			goto out;
 		}
@@ -2174,18 +2321,18 @@ _dict_serialized_length (dict_t *this)
 		pair = pair->next;
 		count--;
 	}
-	
+
 	ret = len;
 out:
 	return ret;
 }
 
 /**
- * _dict_serialize - serialize a dictionary into a buffer. This procedure has 
+ * _dict_serialize - serialize a dictionary into a buffer. This procedure has
  *                   to be called with this->lock held.
  *
  * @this: dict to serialize
- * @buf:  buffer to serialize into. This must be 
+ * @buf:  buffer to serialize into. This must be
  *        atleast dict_serialized_length (this) large
  *
  * @return: success: 0
@@ -2195,78 +2342,80 @@ out:
 int
 _dict_serialize (dict_t *this, char *buf)
 {
-	int           ret     = -1;
-	data_pair_t * pair    = NULL;
-	int32_t       count   = 0;
-	int32_t       keylen  = 0;
-	int32_t       vallen  = 0;
-	int32_t       netword = 0;
-	
-	if (!buf) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"buf is null!");
-		goto out;
-	}
-	
-	count = this->count;
-	if (count < 0) {
-		gf_log ("dict", GF_LOG_ERROR, "count (%d) < 0!", count);
-		goto out;
-	}
+        int           ret     = -1;
+        data_pair_t * pair    = NULL;
+        int32_t       count   = 0;
+        int32_t       keylen  = 0;
+        int32_t       vallen  = 0;
+        int32_t       netword = 0;
 
-	netword = hton32 (count);
-	memcpy (buf, &netword, sizeof(netword));
-	buf += DICT_HDR_LEN;
-	pair = this->members_list;
 
-	while (count) {
-		if (!pair) {
-			gf_log ("dict", GF_LOG_ERROR,
-				"less than count data pairs found!");
-			goto out;
-		}
+        if (!buf) {
+                gf_log ("dict", GF_LOG_ERROR,
+			    "buf is null!");
+		        goto out;
+	    }
 
-		if (!pair->key) {
-			gf_log ("dict", GF_LOG_ERROR,
-				"pair->key is null!");
-			goto out;
-		}
 
-		keylen  = strlen (pair->key);
-		netword = hton32 (keylen);
-		memcpy (buf, &netword, sizeof(netword));
-		buf += DICT_DATA_HDR_KEY_LEN;
+        count = this->count;
+        if (count < 0) {
+                gf_log ("dict", GF_LOG_ERROR, "count (%d) < 0!", count);
+                goto out;
+        }
 
-		if (!pair->value) {
-			gf_log ("dict", GF_LOG_ERROR,
-				"pair->value is null!");
-			goto out;
-		}
+        netword = hton32 (count);
+        memcpy (buf, &netword, sizeof(netword));
+        buf += DICT_HDR_LEN;
+        pair = this->members_list;
 
-		vallen  = pair->value->len;
-		netword = hton32 (vallen);
-		memcpy (buf, &netword, sizeof(netword));
-		buf += DICT_DATA_HDR_VAL_LEN;
+        while (count) {
+                if (!pair) {
+                        gf_log ("dict", GF_LOG_ERROR,
+                                "less than count data pairs found!");
+                        goto out;
+                }
 
-		memcpy (buf, pair->key, keylen);
-		buf += keylen;
-		*buf++ = '\0';
+                if (!pair->key) {
+        			gf_log ("dict", GF_LOG_ERROR,
+                        "pair->key is null!");
+                    goto out;
+                }
 
-		if (!pair->value->data) {
-			gf_log ("dict", GF_LOG_ERROR,
-				"pair->value->data is null!");
-			goto out;
-		}
-		memcpy (buf, pair->value->data, vallen);
-		buf += vallen;
+                keylen  = strlen (pair->key);
+                netword = hton32 (keylen);
+                memcpy (buf, &netword, sizeof(netword));
+                buf += DICT_DATA_HDR_KEY_LEN;
 
-		pair = pair->next;
-		count--;
-	}
+                if (!pair->value) {
+                    gf_log ("dict", GF_LOG_ERROR,
+                        "pair->value is null!");
+                    goto out;
+                }
 
-	ret = 0;
+                vallen  = pair->value->len;
+                netword = hton32 (vallen);
+                memcpy (buf, &netword, sizeof(netword));
+                buf += DICT_DATA_HDR_VAL_LEN;
+
+                memcpy (buf, pair->key, keylen);
+                buf += keylen;
+                *buf++ = '\0';
+
+                if (!pair->value->data) {
+                    gf_log ("dict", GF_LOG_ERROR,
+                        "pair->value->data is null!");
+                    goto out;
+                }
+                memcpy (buf, pair->value->data, vallen);
+                buf += vallen;
+
+                pair = pair->next;
+                count--;
+        }
+
+        ret = 0;
 out:
-	return ret;
+        return ret;
 }
 
 
@@ -2287,7 +2436,7 @@ dict_serialized_length (dict_t *this)
 		gf_log ("dict", GF_LOG_ERROR, "this is null!");
 		goto out;
 	}
-	
+
         LOCK (&this->lock);
         {
                 ret = _dict_serialized_length (this);
@@ -2302,7 +2451,7 @@ out:
  * dict_serialize - serialize a dictionary into a buffer
  *
  * @this: dict to serialize
- * @buf:  buffer to serialize into. This must be 
+ * @buf:  buffer to serialize into. This must be
  *        atleast dict_serialized_length (this) large
  *
  * @return: success: 0
@@ -2313,7 +2462,7 @@ int
 dict_serialize (dict_t *this, char *buf)
 {
 	int           ret    = -1;
-	
+
 	if (!this) {
 		gf_log ("dict", GF_LOG_ERROR,
 			"this is null!");
@@ -2341,7 +2490,7 @@ out:
  * @buf:  buf containing serialized dict
  * @size: size of the @buf
  * @fill: dict to fill in
- * 
+ *
  * @return: success: 0
  *          failure: -errno
  */
@@ -2349,116 +2498,116 @@ out:
 int32_t
 dict_unserialize (char *orig_buf, int32_t size, dict_t **fill)
 {
-	char   *buf = NULL;
-	int     ret   = -1;
-	int32_t count = 0;
-	int     i     = 0;
+        char   *buf = NULL;
+        int     ret   = -1;
+        int32_t count = 0;
+        int     i     = 0;
 
-	data_t * value   = NULL;
-	char   * key     = NULL;
-	int32_t  keylen  = 0;
-	int32_t  vallen  = 0;
-	int32_t  hostord = 0;
+        data_t * value   = NULL;
+        char   * key     = NULL;
+        int32_t  keylen  = 0;
+        int32_t  vallen  = 0;
+        int32_t  hostord = 0;
 
-	buf = orig_buf;
+        buf = orig_buf;
 
-	if (!buf) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"buf is null!");
-		goto out;
-	}
+        if (!buf) {
+                gf_log ("dict", GF_LOG_ERROR,
+                        "buf is null!");
+		        goto out;
+	    }
 
-	if (size == 0) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"size is 0!");
-		goto out;
-	}
+	    if (size == 0) {
+		        gf_log ("dict", GF_LOG_ERROR,
+			    "size is 0!");
+		        goto out;
+	    }
 
-	if (!fill) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"fill is null!");
-		goto out;
-	}
+	    if (!fill) {
+		        gf_log ("dict", GF_LOG_ERROR,
+			    "fill is null!");
+		        goto out;
+	    }
 
-	if (!*fill) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"*fill is null!");
-		goto out;
-	}
+	    if (!*fill) {
+		        gf_log ("dict", GF_LOG_ERROR,
+			    "*fill is null!");
+		        goto out;
+	    }
 
-	if ((buf + DICT_HDR_LEN) > (orig_buf + size)) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"undersized buffer passsed");
-		goto out;
-	}
+	    if ((buf + DICT_HDR_LEN) > (orig_buf + size)) {
+		        gf_log ("dict", GF_LOG_ERROR,
+			    "undersized buffer passed");
+		        goto out;
+	    }
 
-	memcpy (&hostord, buf, sizeof(hostord));
-	count = ntoh32 (hostord);
-	buf += DICT_HDR_LEN;
+	    memcpy (&hostord, buf, sizeof(hostord));
+	    count = ntoh32 (hostord);
+	    buf += DICT_HDR_LEN;
 
-	if (count < 0) {
-		gf_log ("dict", GF_LOG_ERROR,
-			"count (%d) <= 0", count);
-		goto out;
-	}
+	    if (count < 0) {
+		        gf_log ("dict", GF_LOG_ERROR,
+			    "count (%d) <= 0", count);
+		        goto out;
+	    }
 
-	/* count will be set by the dict_set's below */
-	(*fill)->count = 0;
+        	/* count will be set by the dict_set's below */
+      	(*fill)->count = 0;
 
-	for (i = 0; i < count; i++) {
-		if ((buf + DICT_DATA_HDR_KEY_LEN) > (orig_buf + size)) {
-			gf_log ("dict", GF_LOG_DEBUG,
-				"No room for keylen (size %d).",
-				DICT_DATA_HDR_KEY_LEN);
-			gf_log ("dict", GF_LOG_ERROR,
-				"undersized buffer passsed");
-			goto out;
-		}
-		memcpy (&hostord, buf, sizeof(hostord));
-		keylen = ntoh32 (hostord);
-		buf += DICT_DATA_HDR_KEY_LEN;
+        	for (i = 0; i < count; i++) {
+                    if ((buf + DICT_DATA_HDR_KEY_LEN) > (orig_buf + size)) {
+			            gf_log ("dict", GF_LOG_DEBUG,
+                                        "No room for keylen (size %d).",
+				        DICT_DATA_HDR_KEY_LEN);
+			            gf_log ("dict", GF_LOG_ERROR,
+				                "undersized buffer passed");
+			            goto out;
+		        }
+		        memcpy (&hostord, buf, sizeof(hostord));
+		        keylen = ntoh32 (hostord);
+		        buf += DICT_DATA_HDR_KEY_LEN;
 
-		if ((buf + DICT_DATA_HDR_VAL_LEN) > (orig_buf + size)) {
-			gf_log ("dict", GF_LOG_DEBUG,
-				"No room for vallen (size %d).",
-				DICT_DATA_HDR_VAL_LEN);
-			gf_log ("dict", GF_LOG_ERROR,
-				"undersized buffer passsed");
-			goto out;
-		}
-		memcpy (&hostord, buf, sizeof(hostord));
-		vallen = ntoh32 (hostord);
-		buf += DICT_DATA_HDR_VAL_LEN;
+		        if ((buf + DICT_DATA_HDR_VAL_LEN) > (orig_buf + size)) {
+			            gf_log ("dict", GF_LOG_DEBUG,
+				                "No room for vallen (size %d).",
+				        DICT_DATA_HDR_VAL_LEN);
+			            gf_log ("dict", GF_LOG_ERROR,
+				                "undersized buffer passed");
+			            goto out;
+		        }
+		        memcpy (&hostord, buf, sizeof(hostord));
+		        vallen = ntoh32 (hostord);
+		        buf += DICT_DATA_HDR_VAL_LEN;
 
-		if ((buf + keylen) > (orig_buf + size)) {
-			gf_log ("dict", GF_LOG_DEBUG,
-				"No room for key (size %d).", keylen);
-			gf_log ("dict", GF_LOG_ERROR,
-				"undersized buffer passsed");
-			goto out;
-		}
-		key = buf;
-		buf += keylen + 1;  /* for '\0' */
+		        if ((buf + keylen) > (orig_buf + size)) {
+			            gf_log ("dict", GF_LOG_DEBUG,
+				                "No room for key (size %d).", keylen);
+			            gf_log ("dict", GF_LOG_ERROR,
+				                "undersized buffer passed");
+			            goto out;
+		        }
+		        key = buf;
+		        buf += keylen + 1;  /* for '\0' */
 
-		if ((buf + vallen) > (orig_buf + size)) {
-			gf_log ("dict", GF_LOG_DEBUG,
-				"No room for value (size %d).", vallen);
-			gf_log ("dict", GF_LOG_ERROR,
-				"undersized buffer passsed");
-			goto out;
-		}
-		value = get_new_data ();
-		value->len  = vallen;
-		value->data = buf;
-		value->is_static = 1;
-		buf += vallen;
+		        if ((buf + vallen) > (orig_buf + size)) {
+			            gf_log ("dict", GF_LOG_DEBUG,
+				                "No room for value (size %d).", vallen);
+			            gf_log ("dict", GF_LOG_ERROR,
+				                "undersized buffer passed");
+			            goto out;
+		        }
+		        value = get_new_data ();
+		        value->len  = vallen;
+		        value->data = memdup (buf, vallen);
+		        value->is_static = 0;
+		        buf += vallen;
 
-		dict_set (*fill, key, value);
-	}
+		        dict_set (*fill, key, value);
+	    }
 
-	ret = 0;
+	    ret = 0;
 out:
-	return ret;
+	    return ret;
 }
 
 
@@ -2467,7 +2616,7 @@ out:
  *
  * @this: dict to serialize
  * @buf:  pointer to pointer to character. The allocated buffer is stored in
- *        this pointer. The buffer has to be freed by the caller. 
+ *        this pointer. The buffer has to be freed by the caller.
  *
  * @return: success: 0
  *          failure: -errno
@@ -2477,7 +2626,7 @@ int32_t
 dict_allocate_and_serialize (dict_t *this, char **buf, size_t *length)
 {
 	int           ret    = -EINVAL;
-        ssize_t       len = 0; 
+        ssize_t       len = 0;
 
 	if (!this) {
 		gf_log ("dict", GF_LOG_DEBUG,
@@ -2489,16 +2638,16 @@ dict_allocate_and_serialize (dict_t *this, char **buf, size_t *length)
 			"NULL passed as buf");
 		goto out;
 	}
-	
+
         LOCK (&this->lock);
-        { 
+        {
                 len = _dict_serialized_length (this);
                 if (len < 0) {
                         ret = len;
                         goto unlock;
                 }
 
-                *buf = CALLOC (1, len);
+                *buf = GF_CALLOC (1, len, gf_common_mt_char);
                 if (*buf == NULL) {
                         ret = -ENOMEM;
                         gf_log ("dict", GF_LOG_ERROR, "out of memory");
@@ -2507,7 +2656,7 @@ dict_allocate_and_serialize (dict_t *this, char **buf, size_t *length)
 
                 ret = _dict_serialize (this, *buf);
                 if (ret < 0) {
-                        FREE (*buf);
+                        GF_FREE (*buf);
                         *buf = NULL;
                         goto unlock;
                 }
