@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -41,7 +41,8 @@ __iobuf_arena_init_iobufs (struct iobuf_arena *iobuf_arena)
         page_size  = iobuf_arena->iobuf_pool->page_size;
         iobuf_cnt  = arena_size / page_size;
 
-        iobuf_arena->iobufs = CALLOC (sizeof (*iobuf), iobuf_cnt);
+        iobuf_arena->iobufs = GF_CALLOC (sizeof (*iobuf), iobuf_cnt,
+                                        gf_common_mt_iobuf);
         if (!iobuf_arena->iobufs)
                 return;
 
@@ -81,13 +82,13 @@ __iobuf_arena_destroy_iobufs (struct iobuf_arena *iobuf_arena)
 
         iobuf = iobuf_arena->iobufs;
         for (i = 0; i < iobuf_cnt; i++) {
-                assert (iobuf->ref == 0);
+                GF_ASSERT (iobuf->ref == 0);
 
                 list_del_init (&iobuf->list);
                 iobuf++;
         }
 
-        FREE (iobuf_arena->iobufs);
+        GF_FREE (iobuf_arena->iobufs);
 }
 
 
@@ -107,7 +108,7 @@ __iobuf_arena_destroy (struct iobuf_arena *iobuf_arena)
             && iobuf_arena->mem_base != MAP_FAILED)
                 munmap (iobuf_arena->mem_base, iobuf_pool->arena_size);
 
-        FREE (iobuf_arena);
+        GF_FREE (iobuf_arena);
 }
 
 
@@ -117,7 +118,8 @@ __iobuf_arena_alloc (struct iobuf_pool *iobuf_pool)
         struct iobuf_arena *iobuf_arena = NULL;
         size_t              arena_size = 0;
 
-        iobuf_arena = CALLOC (sizeof (*iobuf_arena), 1);
+        iobuf_arena = GF_CALLOC (sizeof (*iobuf_arena), 1,
+                             gf_common_mt_iobuf_arena);
         if (!iobuf_arena)
                 goto err;
 
@@ -224,7 +226,8 @@ iobuf_pool_new (size_t arena_size, size_t page_size)
         if (arena_size < page_size)
                 return NULL;
 
-        iobuf_pool = CALLOC (sizeof (*iobuf_pool), 1);
+        iobuf_pool = GF_CALLOC (sizeof (*iobuf_pool), 1,
+                                gf_common_mt_iobuf_pool);
         if (!iobuf_pool)
                 return NULL;
 
@@ -463,7 +466,8 @@ iobref_new ()
 {
         struct iobref *iobref = NULL;
 
-        iobref = CALLOC (sizeof (*iobref), 1);
+        iobref = GF_CALLOC (sizeof (*iobref), 1,
+                            gf_common_mt_iobref);
         if (!iobref)
                 return NULL;
 
@@ -508,7 +512,7 @@ iobref_destroy (struct iobref *iobref)
                         iobuf_unref (iobuf);
         }
 
-        FREE (iobref);
+        GF_FREE (iobref);
 }
 
 
@@ -702,33 +706,42 @@ iobuf_stats_dump (struct iobuf_pool *iobuf_pool)
         if (!iobuf_pool)
                 return;
 
-        memset (msg, 0, sizeof(msg));
+        memset(msg, 0, sizeof(msg));
 
-        ret = pthread_mutex_trylock (&iobuf_pool->mutex);
+        ret = pthread_mutex_trylock(&iobuf_pool->mutex);
 
         if (ret) {
-                gf_log ("", GF_LOG_WARNING, "Unable to dump iobuf pool"
+                gf_log("", GF_LOG_WARNING, "Unable to dump iobuf pool"
                 " errno: %d", errno);
                 return;
         }
-        gf_proc_dump_add_section ("iobuf.global");
-        gf_proc_dump_write ("iobuf.global.iobuf_pool","%p", iobuf_pool);
-        gf_proc_dump_write ("iobuf.global.iobuf_pool.page_size", "%d",
+        gf_proc_dump_add_section("iobuf.global");
+        gf_proc_dump_write("iobuf.global.iobuf_pool","%p", iobuf_pool);
+        gf_proc_dump_write("iobuf.global.iobuf_pool.page_size", "%d",
 						 iobuf_pool->page_size);
-        gf_proc_dump_write( "iobuf.global.iobuf_pool.arena_size", "%d",
+        gf_proc_dump_write("iobuf.global.iobuf_pool.arena_size", "%d",
 						 iobuf_pool->arena_size);
-        gf_proc_dump_write ("iobuf.global.iobuf_pool.arena_cnt", "%d",
+        gf_proc_dump_write("iobuf.global.iobuf_pool.arena_cnt", "%d",
 						 iobuf_pool->arena_cnt);
 
         list_for_each_entry (trav, &iobuf_pool->arenas.list, list) {
-                snprintf (msg, sizeof(msg), "iobuf.global.iobuf_pool.arena.%d",
+                snprintf(msg, sizeof(msg), "iobuf.global.iobuf_pool.arena.%d",
                                                                             i);
-		gf_proc_dump_add_section (msg);
-                iobuf_arena_info_dump (trav,msg);
+		gf_proc_dump_add_section(msg);
+                iobuf_arena_info_dump(trav,msg);
                 i++;
         }
         
-        pthread_mutex_unlock (&iobuf_pool->mutex);
+        pthread_mutex_unlock(&iobuf_pool->mutex);
 
         return;
 }
+
+
+void
+iobuf_to_iovec(struct iobuf *iob, struct iovec *iov)
+{
+        iov->iov_base = iobuf_ptr (iob);
+        iov->iov_len =  iobuf_pagesize (iob);
+}
+
