@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -26,6 +26,7 @@
 #endif
 
 #include "glusterfs.h"
+#include "xlator.h"
 #include <sys/time.h>
 #include <pthread.h>
 
@@ -34,8 +35,9 @@ typedef void (*gf_timer_cbk_t) (void *);
 struct _gf_timer {
   struct _gf_timer *next, *prev;
   struct timeval at;
-  gf_timer_cbk_t cbk;
+  gf_timer_cbk_t callbk;
   void *data;
+  xlator_t *xl;
 };
 
 struct _gf_timer_registry {
