@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -28,17 +28,11 @@
 #include <fnmatch.h>
 #include "defaults.h"
 
-
 #define SET_DEFAULT_FOP(fn) do {			\
 		if (!xl->fops->fn)			\
 			xl->fops->fn = default_##fn;	\
 	} while (0)
 
-#define SET_DEFAULT_MOP(fn) do {			\
-		if (!xl->mops->fn)			\
-			xl->mops->fn = default_##fn;	\
-	} while (0)
-
 #define SET_DEFAULT_CBK(fn) do {			\
 		if (!xl->cbks->fn)			\
 			xl->cbks->fn = default_##fn;	\
@@ -90,19 +84,13 @@ fill_defaults (xlator_t *xl)
 	SET_DEFAULT_FOP (entrylk);
 	SET_DEFAULT_FOP (fentrylk);
 	SET_DEFAULT_FOP (lookup);
-	SET_DEFAULT_FOP (setdents);
-	SET_DEFAULT_FOP (getdents);
-	SET_DEFAULT_FOP (checksum);
 	SET_DEFAULT_FOP (rchecksum);
 	SET_DEFAULT_FOP (xattrop);
 	SET_DEFAULT_FOP (fxattrop);
-	SET_DEFAULT_FOP (lock_notify);
-	SET_DEFAULT_FOP (lock_fnotify);
         SET_DEFAULT_FOP (setattr);
         SET_DEFAULT_FOP (fsetattr);
 
-        SET_DEFAULT_MOP (log);
-	SET_DEFAULT_MOP (stats);
+        SET_DEFAULT_FOP (getspec);
 
 	SET_DEFAULT_CBK (release);
 	SET_DEFAULT_CBK (releasedir);
@@ -111,138 +99,22 @@ fill_defaults (xlator_t *xl)
 	if (!xl->notify)
 		xl->notify = default_notify;
 
+        if (!xl->mem_acct_init)
+                xl->mem_acct_init = default_mem_acct_init;
+
 	return;
 }
 
-/* RFC 1123 & 952 */
-static char 
-valid_host_name (char *address, int length)
-{
-        int i = 0;
-        char ret = 1;
-
-        if ((length > 75) || (length == 1)) {
-                ret = 0;
-                goto out;
-        }
-
-        if (!isalnum (address[length - 1])) {
-                ret = 0;
-                goto out;
-        }
-
-        for (i = 0; i < length; i++) {
-                if (!isalnum (address[i]) && (address[i] != '.')
-                    && (address[i] != '-')) {
-                        ret = 0;
-                        goto out;
-                }
-        }
-
-out:
-        return ret;
-}
-
-static char
-valid_ipv4_address (char *address, int length)
-{
-        int octets = 0;
-        int value = 0;
-        char *tmp = NULL, *ptr = NULL, *prev = NULL, *endptr = NULL;
-        char ret = 1;
-
-        prev = tmp = strdup (address);
-        prev = strtok_r (tmp, ".", &ptr);
-
-        while (prev != NULL) 
-        {
-                octets++;
-                value = strtol (prev, &endptr, 10);
-                if ((value > 255) || (value < 0) || (endptr != NULL)) {
-                        ret = 0;
-                        goto out;
-                }
-   
-                prev = strtok_r (NULL, ".", &ptr);
-        }
-
-        if (octets != 4) {
-                ret = 0;
-        }
-
-out:
-        FREE (tmp);
-        return ret;
-}
-
-static char
-valid_ipv6_address (char *address, int length)
-{
-        int hex_numbers = 0;
-        int value = 0;
-        char *tmp = NULL, *ptr = NULL, *prev = NULL, *endptr = NULL;
-        char ret = 1;
-
-        tmp = strdup (address);
-        prev = strtok_r (tmp, ":", &ptr);
-
-        while (prev != NULL) 
-        {
-                hex_numbers++;
-                value = strtol (prev, &endptr, 16);
-                if ((value > 0xffff) || (value < 0)
-                    || (endptr != NULL && *endptr != '\0')) {
-                        ret = 0;
-                        goto out;
-                }
-   
-                prev = strtok_r (NULL, ":", &ptr);
-        }
-        
-        if (hex_numbers > 8) {
-                ret = 0;
-        }
-
-out:
-        FREE (tmp);
-        return ret;
-}
-
-static char
-valid_internet_address (char *address)
-{
-        char ret = 0;
-        int length = 0;
-
-        if (address == NULL) {
-                goto out;
-        }
-
-        length = strlen (address);
-        if (length == 0) {
-                goto out;
-        }
-
-        if (valid_ipv4_address (address, length) 
-            || valid_ipv6_address (address, length)
-            || valid_host_name (address, length)) {
-                ret = 1;
-        }
-
-out:        
-        return ret;
-}
-
-int 
-_volume_option_value_validate (xlator_t *xl, 
-			       data_pair_t *pair, 
+int
+_volume_option_value_validate (xlator_t *xl,
+			       data_pair_t *pair,
 			       volume_option_t *opt)
 {
 	int       i = 0;
 	int       ret = -1;
  	uint64_t  input_size = 0;
 	long long inputll = 0;
-	
+
 	/* Key is valid, validate the option */
 	switch (opt->type) {
 	case GF_OPTION_TYPE_PATH:
@@ -260,7 +132,7 @@ _volume_option_value_validate (xlator_t *xl,
 			gf_log (xl->name, GF_LOG_WARNING,
 				"option %s %s: '%s' is not an "
 				"absolute path name",
-				pair->key, pair->value->data, 
+				pair->key, pair->value->data,
 				pair->value->data);
 		}
 		ret = 0;
@@ -269,7 +141,7 @@ _volume_option_value_validate (xlator_t *xl,
 	case GF_OPTION_TYPE_INT:
 	{
 		/* Check the range */
-		if (gf_string2longlong (pair->value->data, 
+		if (gf_string2longlong (pair->value->data,
 					&inputll) != 0) {
 			gf_log (xl->name, GF_LOG_ERROR,
 				"invalid number format \"%s\" in "
@@ -286,12 +158,12 @@ _volume_option_value_validate (xlator_t *xl,
 			ret = 0;
 			break;
 		}
-		if ((inputll < opt->min) || 
+		if ((inputll < opt->min) ||
 		    (inputll > opt->max)) {
 			gf_log (xl->name, GF_LOG_WARNING,
 				"'%lld' in 'option %s %s' is out of "
 				"range [%"PRId64" - %"PRId64"]",
-				inputll, pair->key, 
+				inputll, pair->key,
 				pair->value->data,
 				opt->min, opt->max);
 		}
@@ -301,7 +173,7 @@ _volume_option_value_validate (xlator_t *xl,
 	case GF_OPTION_TYPE_SIZET:
 	{
 		/* Check the range */
-		if (gf_string2bytesize (pair->value->data, 
+		if (gf_string2bytesize (pair->value->data,
 					&input_size) != 0) {
 			gf_log (xl->name, GF_LOG_ERROR,
 				"invalid size format \"%s\" in "
@@ -318,12 +190,12 @@ _volume_option_value_validate (xlator_t *xl,
 			ret = 0;
 			break;
 		}
-		if ((input_size < opt->min) || 
+		if ((input_size < opt->min) ||
 		    (input_size > opt->max)) {
-			gf_log (xl->name, GF_LOG_ERROR,
+			gf_log (xl->name, GF_LOG_WARNING,
 				"'%"PRId64"' in 'option %s %s' is "
 				"out of range [%"PRId64" - %"PRId64"]",
-				input_size, pair->key, 
+				input_size, pair->key,
 				pair->value->data,
 				opt->min, opt->max);
 		}
@@ -332,15 +204,15 @@ _volume_option_value_validate (xlator_t *xl,
 	break;
 	case GF_OPTION_TYPE_BOOL:
 	{
-		/* Check if the value is one of 
+		/* Check if the value is one of
 		   '0|1|on|off|no|yes|true|false|enable|disable' */
 		gf_boolean_t bool_value;
-		if (gf_string2boolean (pair->value->data, 
+		if (gf_string2boolean (pair->value->data,
 				       &bool_value) != 0) {
 			gf_log (xl->name, GF_LOG_ERROR,
 				"option %s %s: '%s' is not a valid "
 				"boolean value",
-				pair->key, pair->value->data, 
+				pair->key, pair->value->data,
 				pair->value->data);
 			goto out;
 		}
@@ -355,7 +227,7 @@ _volume_option_value_validate (xlator_t *xl,
 			xlopt = xlopt->prev;
 
 		while (xlopt) {
-			if (strcmp (pair->value->data, 
+			if (strcmp (pair->value->data,
 				    xlopt->name) == 0) {
 				ret = 0;
 				break;
@@ -366,7 +238,7 @@ _volume_option_value_validate (xlator_t *xl,
 			gf_log (xl->name, GF_LOG_ERROR,
 				"option %s %s: '%s' is not a "
 				"valid volume name",
-				pair->key, pair->value->data, 
+				pair->key, pair->value->data,
 				pair->value->data);
 		}
 		ret = 0;
@@ -382,22 +254,27 @@ _volume_option_value_validate (xlator_t *xl,
 
 		for (i = 0; (i < ZR_OPTION_MAX_ARRAY_SIZE) &&
 			     opt->value[i]; i++) {
-			if (strcasecmp (opt->value[i], 
-					pair->value->data) == 0) {
+ #ifdef  GF_DARWIN_HOST_OS
+                        if (fnmatch (opt->value[i],
+                                     pair->value->data, 0) == 0) {
+ #else
+                        if (fnmatch (opt->value[i],
+                                     pair->value->data, FNM_EXTMATCH) == 0) {
+ #endif
 				ret = 0;
 				break;
 			}
 		}
 
-		if ((i == ZR_OPTION_MAX_ARRAY_SIZE) 
-		    || ((i < ZR_OPTION_MAX_ARRAY_SIZE) 
+		if ((i == ZR_OPTION_MAX_ARRAY_SIZE)
+		    || ((i < ZR_OPTION_MAX_ARRAY_SIZE)
 			&& (!opt->value[i]))) {
 			/* enter here only if
-			 * 1. reached end of opt->value array and haven't 
+			 * 1. reached end of opt->value array and haven't
                          *    validated input
 			 *                      OR
-			 * 2. valid input list is less than 
-                         *    ZR_OPTION_MAX_ARRAY_SIZE and input has not 
+			 * 2. valid input list is less than
+                         *    ZR_OPTION_MAX_ARRAY_SIZE and input has not
                          *    matched all possible input values.
 			 */
 			char given_array[4096] = {0,};
@@ -410,9 +287,9 @@ _volume_option_value_validate (xlator_t *xl,
 			gf_log (xl->name, GF_LOG_ERROR,
 				"option %s %s: '%s' is not valid "
 				"(possible options are %s)",
-				pair->key, pair->value->data, 
+				pair->key, pair->value->data,
 				pair->value->data, given_array);
-			
+
 			goto out;
 		}
 	}
@@ -421,9 +298,9 @@ _volume_option_value_validate (xlator_t *xl,
 	{
 		uint32_t percent = 0;
 
-		
+
 		/* Check if the value is valid percentage */
-		if (gf_string2percent (pair->value->data, 
+		if (gf_string2percent (pair->value->data,
 				       &percent) != 0) {
 			gf_log (xl->name, GF_LOG_ERROR,
 				"invalid percent format \"%s\" "
@@ -436,7 +313,7 @@ _volume_option_value_validate (xlator_t *xl,
 			gf_log (xl->name, GF_LOG_ERROR,
 				"'%d' in 'option %s %s' is out of "
 				"range [0 - 100]",
-				percent, pair->key, 
+				percent, pair->key,
 				pair->value->data);
 		}
 		ret = 0;
@@ -446,7 +323,7 @@ _volume_option_value_validate (xlator_t *xl,
 	{
 		uint32_t percent = 0;
 		uint64_t input_size = 0;
-		
+
 		/* Check if the value is valid percentage */
 		if (gf_string2percent (pair->value->data,
 				       &percent) == 0) {
@@ -457,14 +334,13 @@ _volume_option_value_validate (xlator_t *xl,
         		        if (gf_string2bytesize (pair->value->data,
       	        			                &input_size) == 0) {
 			       	        /* Check the range */
-	        			if ((opt->min == 0) && 
-                                            (opt->max == 0)) {
+                                if ((opt->min == 0) && (opt->max == 0)) {
 	       	        		        gf_log (xl->name, GF_LOG_DEBUG,
         	      	        		        "no range check "
-                                                        "required for "
+                                            "required for "
                 	      		       		"'option %s %s'",
-	               	        	        	pair->key, 
-                                                        pair->value->data);
+                                            pair->key,
+                                            pair->value->data);
 						// It is a size
 			                        ret = 0;
        				                goto out;
@@ -537,7 +413,7 @@ _volume_option_value_validate (xlator_t *xl,
 		uint32_t input_time = 0;
 
 		/* Check if the value is valid percentage */
-		if (gf_string2time (pair->value->data, 
+		if (gf_string2time (pair->value->data,
 				    &input_time) != 0) {
 			gf_log (xl->name,
 				GF_LOG_ERROR,
@@ -555,12 +431,12 @@ _volume_option_value_validate (xlator_t *xl,
 			ret = 0;
 			goto out;
 		}
-		if ((input_time < opt->min) || 
+		if ((input_time < opt->min) ||
 		    (input_time > opt->max)) {
 			gf_log (xl->name, GF_LOG_ERROR,
 				"'%"PRIu32"' in 'option %s %s' is "
 				"out of range [%"PRId64" - %"PRId64"]",
-				input_time, pair->key, 
+				input_time, pair->key,
 				pair->value->data,
 				opt->min, opt->max);
 		}
@@ -572,7 +448,7 @@ _volume_option_value_validate (xlator_t *xl,
 		double input_time = 0.0;
 
 		/* Check if the value is valid double */
-		if (gf_string2double (pair->value->data, 
+		if (gf_string2double (pair->value->data,
 				      &input_time) != 0) {
 			gf_log (xl->name,
 				GF_LOG_ERROR,
@@ -580,7 +456,7 @@ _volume_option_value_validate (xlator_t *xl,
 				pair->value->data, pair->key);
 			goto out;
 		}
-		
+
 		if (input_time < 0.0) {
 			gf_log (xl->name,
 				GF_LOG_ERROR,
@@ -616,7 +492,7 @@ _volume_option_value_validate (xlator_t *xl,
 		ret = 0;
 		break;
 	}
-	
+
 out:
 	return ret;
 }
@@ -639,14 +515,11 @@ validate_xlator_volume_options (xlator_t *xl, volume_option_t *opt)
  	pairs = xl->options->members_list;
  	while (pairs) {
 		ret = -1;
-  		for (index = 0; 
-		     opt[index].key && opt[index].key[0] ; index++) {
+        for (index = 0; opt[index].key && opt[index].key[0] ; index++) {
   			trav = &(opt[index]);
-			for (i = 0 ; 
-			     (i < ZR_VOLUME_MAX_NUM_KEY) && 
-				     trav->key[i]; i++) {
+			for (i = 0 ; (i < ZR_VOLUME_MAX_NUM_KEY) && trav->key[i]; i++) {
 				/* Check if the key is valid */
-				if (fnmatch (trav->key[i], 
+				if (fnmatch (trav->key[i],
 					     pairs->key, FNM_NOESCAPE) == 0) {
 					ret = 0;
 					break;
@@ -660,7 +533,7 @@ validate_xlator_volume_options (xlator_t *xl, volume_option_t *opt)
 						" with correction",
 						trav->key[i], trav->key[0]);
 					/* TODO: some bytes lost */
-					pairs->key = strdup (trav->key[0]);
+                                        pairs->key = gf_strdup (trav->key[0]);
 				}
 				break;
 			}
@@ -674,29 +547,38 @@ validate_xlator_volume_options (xlator_t *xl, volume_option_t *opt)
 
   		pairs = pairs->next;
   	}
-	
+
 	ret = 0;
  out:
   	return ret;
 }
 
 int32_t
-xlator_set_type (xlator_t *xl,
-		 const char *type)
+xlator_set_type_virtual (xlator_t *xl, const char *type)
+{
+	if (xl == NULL || type == NULL)	{
+		gf_log ("xlator", GF_LOG_DEBUG, "invalid argument");
+		return -1;
+	}
+
+        xl->type = gf_strdup (type);
+
+        if (xl->type)
+                return 0;
+        else
+                return -1;
+}
+
+
+int32_t
+xlator_dynload (xlator_t *xl)
 {
         int   ret = 0;
 	char *name = NULL;
 	void *handle = NULL;
 	volume_opt_list_t *vol_opt = NULL;
 
-	if (xl == NULL || type == NULL)	{
-		gf_log ("xlator", GF_LOG_DEBUG, "invalid argument");
-		return -1;
-	}
-
-	xl->type = strdup (type);
-
-	ret = asprintf (&name, "%s/%s.so", XLATORDIR, type);
+	ret = gf_asprintf (&name, "%s/%s.so", XLATORDIR, xl->type);
         if (-1 == ret) {
                 gf_log ("xlator", GF_LOG_ERROR, "asprintf failed");
                 return -1;
@@ -707,36 +589,36 @@ xlator_set_type (xlator_t *xl,
 	handle = dlopen (name, RTLD_NOW|RTLD_GLOBAL);
 	if (!handle) {
 		gf_log ("xlator", GF_LOG_DEBUG, "%s", dlerror ());
+                GF_FREE (name);
 		return -1;
 	}
+        xl->dlhandle = handle;
 
 	if (!(xl->fops = dlsym (handle, "fops"))) {
 		gf_log ("xlator", GF_LOG_DEBUG, "dlsym(fops) on %s",
 			dlerror ());
-		return -1;
-	}
-
-	if (!(xl->mops = dlsym (handle, "mops"))) {
-		gf_log ("xlator", GF_LOG_DEBUG, "dlsym(mops) on %s",
-			dlerror ());
+                GF_FREE (name);
 		return -1;
 	}
 
 	if (!(xl->cbks = dlsym (handle, "cbks"))) {
 		gf_log ("xlator", GF_LOG_DEBUG, "dlsym(cbks) on %s",
 			dlerror ());
+                GF_FREE (name);
 		return -1;
 	}
 
 	if (!(xl->init = dlsym (handle, "init"))) {
 		gf_log ("xlator", GF_LOG_DEBUG, "dlsym(init) on %s",
 			dlerror ());
+                GF_FREE (name);
 		return -1;
 	}
 
 	if (!(xl->fini = dlsym (handle, "fini"))) {
 		gf_log ("xlator", GF_LOG_DEBUG, "dlsym(fini) on %s",
 			dlerror ());
+                GF_FREE (name);
 		return -1;
 	}
 
@@ -750,9 +632,34 @@ xlator_set_type (xlator_t *xl,
 			"dlsym(dumpops) on %s -- neglecting", dlerror ());
 	}
 
+        if (!(xl->mem_acct_init = dlsym (handle, "mem_acct_init"))) {
+                gf_log (xl->name, GF_LOG_DEBUG,
+                        "dlsym(mem_acct_init) on %s -- neglecting",
+                        dlerror ());
+        }
+
+	if (!(xl->reconfigure = dlsym (handle, "reconfigure"))) {
+		gf_log ("xlator", GF_LOG_DEBUG,
+			"dlsym(reconfigure) on %s -- neglecting",
+			dlerror());
+	}
+
+        if (!(xl->validate_options = dlsym (handle, "validate_options"))) {
+                gf_log ("xlator", GF_LOG_DEBUG,
+                        "dlsym(validate_options) on %s -- neglecting",
+                        dlerror());
+        }
+
+
 	INIT_LIST_HEAD (&xl->volume_options);
 
-	vol_opt = CALLOC (1, sizeof (volume_opt_list_t));
+	vol_opt = GF_CALLOC (1, sizeof (volume_opt_list_t),
+                         gf_common_mt_volume_opt_list_t);
+
+        if (!vol_opt) {
+                GF_FREE (name);
+                return -1;
+        }
 
 	if (!(vol_opt->given_opt = dlsym (handle, "options"))) {
 		dlerror ();
@@ -763,11 +670,24 @@ xlator_set_type (xlator_t *xl,
 
 	fill_defaults (xl);
 
-	FREE (name);
+	GF_FREE (name);
 	return 0;
 }
 
 
+int32_t
+xlator_set_type (xlator_t *xl, const char *type)
+{
+        int ret = 0;
+
+        ret = xlator_set_type_virtual (xl, type);
+        if (!ret)
+                ret = xlator_dynload (xl);
+
+        return ret;
+}
+
+
 void
 xlator_foreach (xlator_t *this,
 		void (*fn)(xlator_t *each,
@@ -818,50 +738,26 @@ xlator_search_by_name (xlator_t *any, const char *name)
 }
 
 
-static int32_t
-xlator_init_rec (xlator_t *xl)
+static int
+__xlator_init(xlator_t *xl)
 {
-	xlator_t *trav = NULL;
-	int32_t ret = 0;
+        xlator_t *old_THIS = NULL;
+        int       ret = 0;
 
-	if (xl == NULL)	{
-		gf_log ("xlator", GF_LOG_DEBUG, "invalid argument");
-		return 0;
-	}
+        old_THIS = THIS;
+        THIS = xl;
 
-	trav = xl;
-        while (trav->prev)
-                trav = trav->prev;
+        ret = xl->init (xl);
 
-        while (trav) {
-		ret = -1;
-		if (trav->init && !trav->ready) {
-			ret = xlator_init (trav);
-			if (ret) {
-				gf_log (trav->name, GF_LOG_ERROR,
-					"Initialization of volume '%s' failed,"
-					" review your volfile again",
-					trav->name);
-                                break;
-			} else {
-				trav->init_succeeded = 1;
-			}
-		} else {
-			gf_log (trav->name, GF_LOG_DEBUG, "No init() found");
-		}
-		/* This 'xl' is checked */
-		trav->ready = 1;
-                trav = trav->next;
-	}
+        THIS = old_THIS;
 
-	return ret;
+        return ret;
 }
 
 
-int32_t
-xlator_tree_init (xlator_t *xl)
+int
+xlator_init (xlator_t *xl)
 {
-	xlator_t *top = NULL;
 	int32_t ret = 0;
 
 	if (xl == NULL)	{
@@ -869,17 +765,30 @@ xlator_tree_init (xlator_t *xl)
 		return 0;
 	}
 
-	top = xl;
-/*
-	while (top->parents)
-		top = top->parents->xlator;
-*/
-	ret = xlator_init_rec (top);
+        ret = -1;
 
-	if (ret == 0 && top->notify) {
-		top->notify (top, GF_EVENT_PARENT_UP, NULL);
-	}
+        if (xl->mem_acct_init)
+                xl->mem_acct_init (xl);
 
+        if (!xl->init) {
+                gf_log (xl->name, GF_LOG_DEBUG, "No init() found");
+                goto out;
+        }
+
+        ret = __xlator_init (xl);
+
+        if (ret) {
+                gf_log (xl->name, GF_LOG_ERROR,
+                        "Initialization of volume '%s' failed,"
+                        " review your volfile again",
+                        xl->name);
+                goto out;
+        }
+
+        xl->init_succeeded = 1;
+
+        ret = 0;
+out:
 	return ret;
 }
 
@@ -916,7 +825,97 @@ xlator_fini_rec (xlator_t *xl)
 	}
 }
 
+static int
+xlator_reconfigure_rec (xlator_t *old_xl, xlator_t *new_xl)
+{
+	xlator_list_t *trav1 = NULL;
+        xlator_list_t *trav2 = NULL;
+        int32_t       ret    = 0;
 
+	if (old_xl == NULL || new_xl == NULL)	{
+		gf_log ("xlator", GF_LOG_DEBUG, "invalid argument");
+		return -1;
+	}
+
+	trav1 = old_xl->children;
+        trav2 = new_xl->children;
+
+	while (trav1 && trav2) {
+		ret = xlator_reconfigure_rec (trav1->xlator, trav2->xlator);
+
+                if (ret)
+                        goto out;
+
+		gf_log (trav1->xlator->name, GF_LOG_DEBUG, "reconfigured");
+
+		trav1 = trav1->next;
+                trav2 = trav2->next;
+	}
+
+        if (old_xl->reconfigure) {
+                ret = old_xl->reconfigure (old_xl, new_xl->options);
+                if (ret)
+                        goto out;
+        }
+        else
+                gf_log (old_xl->name, GF_LOG_DEBUG, "No reconfigure() found");
+
+out:
+        return ret;
+}
+
+int
+xlator_validate_rec (xlator_t *xlator, char **op_errstr)
+{
+        xlator_list_t *trav = NULL;
+
+        if (xlator == NULL )    {
+                gf_log ("xlator", GF_LOG_DEBUG, "invalid argument");
+                return -1;
+        }
+
+        trav = xlator->children;
+
+        while (trav) {
+                if (xlator_validate_rec (trav->xlator, op_errstr) )
+                        return -1;
+
+                trav = trav->next;
+        }
+
+        if (xlator_dynload (xlator))
+                gf_log ("", GF_LOG_DEBUG, "Did not load the symbols");
+
+        if (xlator->validate_options) {
+                if (xlator->validate_options (xlator, xlator->options,
+                    op_errstr)) {
+                        gf_log ("", GF_LOG_DEBUG, "%s", *op_errstr);
+                        return -1;
+                }
+                gf_log (xlator->name, GF_LOG_DEBUG, "Validated option");
+
+        }
+
+        gf_log (xlator->name, GF_LOG_DEBUG, "No validate_options() found");
+
+        return 0;
+}
+
+int
+graph_reconf_validateopt (glusterfs_graph_t *graph,
+                          char **op_errstr)
+{
+        xlator_t *xlator = NULL;
+        int ret = -1;
+
+        GF_ASSERT (graph);
+
+        xlator = graph->first;
+
+        ret = xlator_validate_rec (xlator, op_errstr);
+
+        return ret;
+}
 int
 xlator_notify (xlator_t *xl, int event, void *data, ...)
 {
@@ -935,22 +934,39 @@ xlator_notify (xlator_t *xl, int event, void *data, ...)
 
 
 int
-xlator_init (xlator_t *xl)
+xlator_mem_acct_init (xlator_t *xl, int num_types)
 {
-        xlator_t *old_THIS = NULL;
-        int       ret = 0;
+        int             i = 0;
+        int             ret = 0;
 
-        old_THIS = THIS;
-        THIS = xl;
+        if (!gf_mem_acct_is_enabled())
+                return 0;
 
-        ret = xl->init (xl);
+        if (!xl)
+                return -1;
 
-        THIS = old_THIS;
+        xl->mem_acct.num_types = num_types;
 
-        return ret;
+        xl->mem_acct.rec = calloc(num_types, sizeof(struct mem_acct_rec));
+
+        if (!xl->mem_acct.rec) {
+                gf_log("xlator", GF_LOG_ERROR, "Out of Memory");
+                return -1;
+        }
+
+        for (i = 0; i < num_types; i++) {
+                ret = LOCK_INIT(&(xl->mem_acct.rec[i].lock));
+                if (ret) {
+                        fprintf(stderr, "Unable to lock..errno : %d",errno);
+                }
+        }
+
+        gf_log(xl->name, GF_LOG_DEBUG, "Allocated mem_acct_rec for %d types",
+                        num_types);
+
+        return 0;
 }
 
-
 void
 xlator_tree_fini (xlator_t *xl)
 {
@@ -965,6 +981,21 @@ xlator_tree_fini (xlator_t *xl)
 	xlator_fini_rec (top);
 }
 
+int
+xlator_tree_reconfigure (xlator_t *old_xl, xlator_t *new_xl)
+{
+        xlator_t *new_top = NULL;
+        xlator_t *old_top = NULL;
+
+        GF_ASSERT (old_xl);
+        GF_ASSERT (new_xl);
+
+        old_top = old_xl;
+        new_top = new_xl;
+
+	return xlator_reconfigure_rec (old_top, new_top);
+}
+
 
 int
 xlator_tree_free (xlator_t *tree)
@@ -979,12 +1010,12 @@ xlator_tree_free (xlator_t *tree)
   while (prev) {
     trav = prev->next;
     dict_destroy (prev->options);
-    FREE (prev->name);
-    FREE (prev->type);
-    FREE (prev);
+    GF_FREE (prev->name);
+    GF_FREE (prev->type);
+    GF_FREE (prev);
     prev = trav;
   }
-  
+
   return 0;
 }
 
@@ -997,10 +1028,10 @@ loc_wipe (loc_t *loc)
                 loc->inode = NULL;
         }
         if (loc->path) {
-                FREE (loc->path);
+                GF_FREE ((char *)loc->path);
                 loc->path = NULL;
         }
-  
+
         if (loc->parent) {
                 inode_unref (loc->parent);
                 loc->parent = NULL;
@@ -1021,7 +1052,7 @@ loc_copy (loc_t *dst, loc_t *src)
 	if (src->parent)
 		dst->parent = inode_ref (src->parent);
 
-	dst->path = strdup (src->path);
+        dst->path = gf_strdup (src->path);
 
 	if (!dst->path)
 		goto out;
@@ -1032,5 +1063,163 @@ loc_copy (loc_t *dst, loc_t *src)
 
 	ret = 0;
 out:
+        if (ret == -1) {
+                if (dst->inode)
+                        inode_unref (dst->inode);
+
+                if (dst->parent)
+                        inode_unref (dst->parent);
+        }
 	return ret;
 }
+
+
+int
+xlator_list_destroy (xlator_list_t *list)
+{
+        xlator_list_t *next = NULL;
+
+        while (list) {
+                next = list->next;
+                GF_FREE (list);
+                list = next;
+        }
+
+        return 0;
+}
+
+
+int
+xlator_destroy (xlator_t *xl)
+{
+        volume_opt_list_t *vol_opt = NULL;
+        volume_opt_list_t *tmp     = NULL;
+
+        if (!xl)
+                return 0;
+
+        if (xl->name)
+                GF_FREE (xl->name);
+        if (xl->type)
+                GF_FREE (xl->type);
+        if (xl->dlhandle)
+                dlclose (xl->dlhandle);
+        if (xl->options)
+                dict_destroy (xl->options);
+
+        xlator_list_destroy (xl->children);
+
+        xlator_list_destroy (xl->parents);
+
+        list_for_each_entry_safe (vol_opt, tmp, &xl->volume_options, list) {
+                list_del_init (&vol_opt->list);
+                GF_FREE (vol_opt);
+        }
+
+        GF_FREE (xl);
+
+        return 0;
+}
+int
+is_gf_log_command (xlator_t *this, const char *name, char *value)
+{
+        xlator_t       *trav        = NULL;
+        char            key[1024]   = {0,};
+        int             ret         = -1;
+        int             log_level   = -1;
+        gf_boolean_t    syslog_flag = 0;
+        glusterfs_ctx_t *ctx        = NULL;
+
+        if (!strcmp ("trusted.glusterfs.syslog", name)) {
+                ret = gf_string2boolean (value, &syslog_flag);
+                if (ret) {
+                        ret = EOPNOTSUPP;
+                        goto out;
+                }
+                if (syslog_flag)
+                        gf_log_enable_syslog ();
+                else
+                        gf_log_disable_syslog ();
+
+                goto out;
+        }
+
+        if (fnmatch ("trusted.glusterfs*set-log-level", name, FNM_NOESCAPE))
+                goto out;
+
+        log_level = glusterd_check_log_level (value);
+        if (log_level == -1) {
+                ret = EOPNOTSUPP;
+                goto out;
+        }
+
+        /* Some crude way to change the log-level of process */
+        if (!strcmp (name, "trusted.glusterfs.set-log-level")) {
+                /* */
+                gf_log ("glusterfs", gf_log_get_loglevel(),
+                        "setting log level to %d (old-value=%d)",
+                        log_level, gf_log_get_loglevel());
+                gf_log_set_loglevel (log_level);
+                ret = 0;
+                goto out;
+        }
+        if (!strcmp (name, "trusted.glusterfs.fuse.set-log-level")) {
+                /* */
+                gf_log (this->name, gf_log_get_xl_loglevel (this),
+                        "setting log level to %d (old-value=%d)",
+                        log_level, gf_log_get_xl_loglevel (this));
+                gf_log_set_xl_loglevel (this, log_level);
+                ret = 0;
+                goto out;
+        }
+
+        ctx = glusterfs_ctx_get();
+        if (!ctx)
+                goto out;
+        if (!ctx->active)
+                goto out;
+        trav = ctx->active->top;
+
+        while (trav) {
+                snprintf (key, 1024, "trusted.glusterfs.%s.set-log-level",
+                          trav->name);
+                if (fnmatch (name, key, FNM_NOESCAPE) == 0) {
+                        gf_log (trav->name, gf_log_get_xl_loglevel (trav),
+                                "setting log level to %d (old-value=%d)",
+                                log_level, gf_log_get_xl_loglevel (trav));
+                        gf_log_set_xl_loglevel (trav, log_level);
+                        ret = 0;
+                }
+                trav = trav->next;
+        }
+out:
+        return ret;
+}
+
+int
+glusterd_check_log_level (const char *value)
+{
+        int log_level = -1;
+
+        if (!strcasecmp (value, "CRITICAL")) {
+                log_level = GF_LOG_CRITICAL;
+        } else if (!strcasecmp (value, "ERROR")) {
+                log_level = GF_LOG_ERROR;
+        } else if (!strcasecmp (value, "WARNING")) {
+                log_level = GF_LOG_WARNING;
+        } else if (!strcasecmp (value, "INFO")) {
+                log_level = GF_LOG_INFO;
+        } else if (!strcasecmp (value, "DEBUG")) {
+                log_level = GF_LOG_DEBUG;
+        } else if (!strcasecmp (value, "TRACE")) {
+                log_level = GF_LOG_TRACE;
+        } else if (!strcasecmp (value, "NONE")) {
+                log_level = GF_LOG_NONE;
+        }
+
+        if (log_level == -1)
+                gf_log ("", GF_LOG_ERROR, "Invalid log-level. possible values "
+                        "are DEBUG|WARNING|ERROR|CRITICAL|NONE|TRACE");
+
+        return log_level;
+}
