@@ -68,7 +68,7 @@
 /* Copy the first part of user declarations.  */
 
 /* Line 189 of yacc.c  */
-#line 23 "../../../libglusterfs/src/spec.y"
+#line 23 "../../../libglusterfs/src/graph.y"
 
 #include <stdio.h>
 #include <string.h>
@@ -77,28 +77,29 @@
 #include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-	 
+
 #include "xlator.h"
+#include "graph-utils.h"
 #include "logging.h"
 
-static int new_section (char *name);
-static int section_type (char *type);
-static int section_option (char *key, char *value);
-static int section_sub (char *sub);
-static int section_end (void);
+static int new_volume (char *name);
+static int volume_type (char *type);
+static int volume_option (char *key, char *value);
+static int volume_sub (char *sub);
+static int volume_end (void);
 static void sub_error (void);
 static void type_error (void);
 static void option_error (void);
 
 #define YYSTYPE char *
-#define GF_CMD_BUFFER_LEN (8 * GF_UNIT_KB) 
+#define GF_CMD_BUFFER_LEN (8 * GF_UNIT_KB)
 
 int yyerror (const char *);
 int yylex ();
 
 
 /* Line 189 of yacc.c  */
-#line 102 "y.tab.c"
+#line 103 "y.tab.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -125,11 +126,11 @@ int yylex ();
    /* Put the tokens into the symbol table, so that GDB and other debuggers
       know about them.  */
    enum yytokentype {
-     SECTION_BEGIN = 258,
-     SECTION_END = 259,
+     VOLUME_BEGIN = 258,
+     VOLUME_END = 259,
      OPTION = 260,
      NEWLINE = 261,
-     SUBSECTION = 262,
+     SUBVOLUME = 262,
      ID = 263,
      WHITESPACE = 264,
      COMMENT = 265,
@@ -138,11 +139,11 @@ int yylex ();
    };
 #endif
 /* Tokens.  */
-#define SECTION_BEGIN 258
-#define SECTION_END 259
+#define VOLUME_BEGIN 258
+#define VOLUME_END 259
 #define OPTION 260
 #define NEWLINE 261
-#define SUBSECTION 262
+#define SUBVOLUME 262
 #define ID 263
 #define WHITESPACE 264
 #define COMMENT 265
@@ -164,7 +165,7 @@ typedef int YYSTYPE;
 
 
 /* Line 264 of yacc.c  */
-#line 168 "y.tab.c"
+#line 169 "y.tab.c"
 
 #ifdef short
 # undef short
@@ -455,9 +456,9 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    53,    53,    53,    55,    56,    57,    59,    60,    61,
-      62,    63,    64,    65,    67,    67,    69,    69,    71,    71,
-      73,    74,    75,    77,    77,    78,    78
+       0,    54,    54,    54,    56,    57,    58,    60,    61,    62,
+      63,    64,    65,    66,    68,    68,    70,    70,    72,    72,
+      74,    75,    76,    78,    78,    79,    79
 };
 #endif
 
@@ -466,10 +467,10 @@ static const yytype_uint8 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "SECTION_BEGIN", "SECTION_END", "OPTION",
-  "NEWLINE", "SUBSECTION", "ID", "WHITESPACE", "COMMENT", "TYPE",
-  "STRING_TOK", "$accept", "SECTIONS", "SECTION", "SECTION_HEADER",
-  "SECTION_FOOTER", "SECTION_DATA", "TYPE_LINE", "SUBSECTION_LINE",
+  "$end", "error", "$undefined", "VOLUME_BEGIN", "VOLUME_END", "OPTION",
+  "NEWLINE", "SUBVOLUME", "ID", "WHITESPACE", "COMMENT", "TYPE",
+  "STRING_TOK", "$accept", "VOLUMES", "VOLUME", "VOLUME_HEADER",
+  "VOLUME_FOOTER", "VOLUME_DATA", "TYPE_LINE", "SUBVOLUME_LINE",
   "OPTIONS_LINE", "OPTION_LINE", "WORDS", "WORD", 0
 };
 #endif
@@ -1376,77 +1377,77 @@ yyreduce:
         case 5:
 
 /* Line 1455 of yacc.c  */
-#line 56 "../../../libglusterfs/src/spec.y"
-    {if( -1 == new_section ((yyvsp[(2) - (2)]))) { YYABORT; } }
+#line 57 "../../../libglusterfs/src/graph.y"
+    {if (new_volume ((yyvsp[(2) - (2)])) == -1) { YYABORT; }}
     break;
 
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 57 "../../../libglusterfs/src/spec.y"
-    {if( -1 == section_end ()) { YYABORT; } }
+#line 58 "../../../libglusterfs/src/graph.y"
+    {if (volume_end () == -1) { YYABORT; }}
     break;
 
   case 14:
 
 /* Line 1455 of yacc.c  */
-#line 67 "../../../libglusterfs/src/spec.y"
-    {if ( -1 == section_type ((yyvsp[(2) - (2)]))) { YYABORT; }}
+#line 68 "../../../libglusterfs/src/graph.y"
+    {if (volume_type ((yyvsp[(2) - (2)])) == -1) { YYABORT; }}
     break;
 
   case 15:
 
 /* Line 1455 of yacc.c  */
-#line 67 "../../../libglusterfs/src/spec.y"
+#line 68 "../../../libglusterfs/src/graph.y"
     { type_error(); YYABORT; }
     break;
 
   case 17:
 
 /* Line 1455 of yacc.c  */
-#line 69 "../../../libglusterfs/src/spec.y"
+#line 70 "../../../libglusterfs/src/graph.y"
     { sub_error (); YYABORT; }
     break;
 
   case 20:
 
 /* Line 1455 of yacc.c  */
-#line 73 "../../../libglusterfs/src/spec.y"
-    {if(-1 == section_option((yyvsp[(2) - (3)]),(yyvsp[(3) - (3)]))){YYABORT;} }
+#line 74 "../../../libglusterfs/src/graph.y"
+    {if (volume_option ((yyvsp[(2) - (3)]), (yyvsp[(3) - (3)])) == -1) { YYABORT; }}
     break;
 
   case 21:
 
 /* Line 1455 of yacc.c  */
-#line 74 "../../../libglusterfs/src/spec.y"
+#line 75 "../../../libglusterfs/src/graph.y"
     { option_error (); YYABORT; }
     break;
 
   case 22:
 
 /* Line 1455 of yacc.c  */
-#line 75 "../../../libglusterfs/src/spec.y"
+#line 76 "../../../libglusterfs/src/graph.y"
     { option_error (); YYABORT; }
     break;
 
   case 23:
 
 /* Line 1455 of yacc.c  */
-#line 77 "../../../libglusterfs/src/spec.y"
-    {if (-1 == section_sub ((yyvsp[(1) - (1)]))) {YYABORT; } }
+#line 78 "../../../libglusterfs/src/graph.y"
+    {if (volume_sub ((yyvsp[(1) - (1)])) == -1) {YYABORT; }}
     break;
 
   case 24:
 
 /* Line 1455 of yacc.c  */
-#line 77 "../../../libglusterfs/src/spec.y"
-    { if (-1 == section_sub ((yyvsp[(2) - (2)]))) { YYABORT; } }
+#line 78 "../../../libglusterfs/src/graph.y"
+    { if (volume_sub ((yyvsp[(2) - (2)])) == -1) { YYABORT; }}
     break;
 
 
 
 /* Line 1455 of yacc.c  */
-#line 1450 "y.tab.c"
+#line 1451 "y.tab.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1658,361 +1659,329 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 79 "../../../libglusterfs/src/spec.y"
+#line 80 "../../../libglusterfs/src/graph.y"
 
 
-xlator_t *complete_tree = NULL;
-xlator_t *tree = NULL;
-glusterfs_ctx_t *gctx;
+xlator_t *curr;
+glusterfs_graph_t *construct;
+
 
 static void
 type_error (void)
 {
         extern int yylineno;
 
-        fprintf (stderr, "Volume '%s', before line %d: Please specify volume 'type'.",
-                 complete_tree->name, yylineno);
-        gf_log ("parser", GF_LOG_ERROR, 
-                "Volume '%s', before line %d: Please specify volume 'type'.",
-                complete_tree->name, yylineno);
+        gf_log ("parser", GF_LOG_ERROR,
+                "Volume %s, before line %d: Please specify volume type",
+                curr->name, yylineno);
         return;
 }
 
+
 static void
 sub_error (void)
 {
         extern int yylineno;
 
-        fprintf (stderr, "Volume %s, before line %d: Please specify subvolumes for the volume. ",
-                 complete_tree->name, yylineno);
-        gf_log ("parser", GF_LOG_ERROR, 
-                "Volume %s, before line %d: Please specify subvolumes for the volume. ",
-                complete_tree->name, yylineno);
+        gf_log ("parser", GF_LOG_ERROR,
+                "Volume %s, before line %d: Please specify subvolumes",
+                curr->name, yylineno);
         return;
 }
 
+
 static void
 option_error (void)
 {
         extern int yylineno;
 
-        fprintf (stderr, "Volume %s, before line %d: Please specify "
-		 "option <key> <value> \n",
-                 complete_tree->name, yylineno);
-        gf_log ("parser", GF_LOG_ERROR, 
+        gf_log ("parser", GF_LOG_ERROR,
                 "Volume %s, before line %d: Please specify "
                 "option <key> <value>",
-                complete_tree->name, yylineno);
+                curr->name, yylineno);
         return;
 }
 
-static int
-cut_tree (xlator_t *tree)
-{
-        xlator_t *trav = tree, *prev = tree;
-
-        if (!tree) {
-                gf_log ("parser", GF_LOG_DEBUG, "Translator tree not found");
-                return -1;
-        }
-
-        gf_log ("parser", GF_LOG_DEBUG, "Failed to build translator graph");
-
-        while (prev) {
-                trav = prev->next;
-                dict_destroy (prev->options);
-                FREE (prev->name);
-                FREE (prev);
-                prev = trav;
-        }
-  
-        return 0;
-}
-
 
 static int
-new_section (char *name)
+new_volume (char *name)
 {
-        extern int yylineno;
-        xlator_t *trav = complete_tree;
-        xlator_t *node = (void *) calloc (1, sizeof (*node));
+        extern int   yylineno;
+        xlator_t    *trav = NULL;
+        int          ret = 0;
 
         if (!name) {
-                gf_log ("parser", GF_LOG_DEBUG, 
+                gf_log ("parser", GF_LOG_DEBUG,
 			"Invalid argument name: '%s'", name);
-                return -1;
+                ret = -1;
+                goto out;
         }
 
+        if (curr) {
+                gf_log ("parser", GF_LOG_ERROR,
+                        "new volume (%s) defintion in line %d unexpected",
+                        name, yylineno);
+                ret = -1;
+                goto out;
+        }
+
+        curr = (void *) GF_CALLOC (1, sizeof (*curr),
+                                   gf_common_mt_xlator_t);
+
+        if (!curr) {
+                gf_log ("parser", GF_LOG_ERROR, "Out of memory");
+                ret = -1;
+                goto out;
+        }
+
+        trav = construct->first;
+
         while (trav) {
-                if (!strcmp (name,  trav->name)) {
-                        fprintf (stderr, 
-				 "Line %d: volume '%s' defined again\n", 
-                                 yylineno, name);
-                        gf_log ("parser", GF_LOG_ERROR, 
-				"Line %d: volume '%s' defined again", 
+                if (!strcmp (name, trav->name)) {
+                        gf_log ("parser", GF_LOG_ERROR,
+				"Line %d: volume '%s' defined again",
                                 yylineno, name);
-                        return -1;
+                        ret = -1;
+                        goto out;
                 }
                 trav = trav->next;
         }
 
-        node->ctx = gctx;
-        node->name = name;
-        node->next = complete_tree;
-        if (complete_tree)
-                complete_tree->prev = node;
-        node->options = get_new_dict ();
-        complete_tree = node;
+        curr->name = gf_strdup (name);
+        if (!curr->name) {
+                GF_FREE (curr);
+                ret = -1;
+                goto out;
+        }
+
+        curr->options = get_new_dict ();
+
+        if (!curr->options) {
+                GF_FREE (curr->name);
+                GF_FREE (curr);
+                ret = -1;
+                goto out;
+        }
+
+        curr->next = construct->first;
+        if (curr->next)
+                curr->next->prev = curr;
+
+        curr->graph = construct;
+
+        construct->first = curr;
+
+        construct->xl_count++;
 
-        tree = node;
         gf_log ("parser", GF_LOG_TRACE, "New node for '%s'", name);
 
-        return 0;
+out:
+        GF_FREE (name);
+
+        return ret;
 }
 
+
 static int
-section_type (char *type)
+volume_type (char *type)
 {
-        extern int yylineno;
-        int32_t ret = -1;
+        extern int   yylineno;
+        int32_t      ret = 0;
+
         if (!type) {
                 gf_log ("parser", GF_LOG_DEBUG, "Invalid argument type");
-                return -1;
+                ret = -1;
+                goto out;
         }
 
-        ret = xlator_set_type (tree, type);
+        ret = xlator_set_type (curr, type);
         if (ret) {
-                fprintf (stderr, "Volume '%s', line %d: type '%s' is not "
-			 "valid or not found on this machine\n", 
-                         complete_tree->name, yylineno, type);
-                gf_log ("parser", GF_LOG_ERROR, 
+                gf_log ("parser", GF_LOG_ERROR,
                         "Volume '%s', line %d: type '%s' is not valid or "
-			"not found on this machine", 
-                        complete_tree->name, yylineno, type);
-                return -1;
+			"not found on this machine",
+                        curr->name, yylineno, type);
+                ret = -1;
+                goto out;
         }
-        gf_log ("parser", GF_LOG_TRACE, "Type:%s:%s", tree->name, type);
+
+        gf_log ("parser", GF_LOG_TRACE, "Type:%s:%s", curr->name, type);
+
+out:
+        GF_FREE (type);
 
         return 0;
 }
 
 
-static int 
-section_option (char *key, char *value)
+static int
+volume_option (char *key, char *value)
 {
-        extern int yylineno;
-
-        int ret = 0;
+        extern int  yylineno;
+        int         ret = 0;
+        char       *set_value = NULL;
 
         if (!key || !value){
-                fprintf (stderr, "Invalid argument\n");
                 gf_log ("parser", GF_LOG_ERROR, "Invalid argument");
-                return -1;
+                ret = -1;
+                goto out;
         }
 
-	ret = dict_set (tree->options, key, str_to_data (value));
+        set_value = gf_strdup (value);
+	ret = dict_set_dynstr (curr->options, key, set_value);
 
         if (ret == 1) {
-                gf_log ("parser", GF_LOG_ERROR, 
+                gf_log ("parser", GF_LOG_ERROR,
                         "Volume '%s', line %d: duplicate entry "
-			"('option %s') present", 
-                        tree->name, yylineno, key);
-                return -1;
+			"('option %s') present",
+                        curr->name, yylineno, key);
+                ret = -1;
+                goto out;
         }
+
         gf_log ("parser", GF_LOG_TRACE, "Option:%s:%s:%s",
-                tree->name, key, value);
+                curr->name, key, value);
+
+out:
+        GF_FREE (key);
+        GF_FREE (value);
 
         return 0;
 }
 
-static int 
-section_sub (char *sub)
+
+static int
+volume_sub (char *sub)
 {
-        extern int yylineno;
-        xlator_t *trav = complete_tree;
-        xlator_list_t *xlchild, *tmp, *xlparent;
+        extern int       yylineno;
+        xlator_t        *trav = NULL;
+        int              ret = 0;
 
         if (!sub) {
-                fprintf (stderr, "Invalid subvolumes argument\n");
                 gf_log ("parser", GF_LOG_ERROR, "Invalid subvolumes argument");
-                return -1;
+                ret = -1;
+                goto out;
         }
 
+        trav = construct->first;
+
         while (trav) {
                 if (!strcmp (sub,  trav->name))
                         break;
                 trav = trav->next;
         }
+
         if (!trav) {
-                fprintf (stderr, 
-                         "Volume '%s', line %d: subvolume '%s' is not "
-			 "defined prior to usage\n", 
-                         complete_tree->name, yylineno, sub);
-                gf_log ("parser", GF_LOG_ERROR, 
+                gf_log ("parser", GF_LOG_ERROR,
                         "Volume '%s', line %d: subvolume '%s' is not defined "
-			"prior to usage", 
-                        complete_tree->name, yylineno, sub);
-                return -1;
-        }
-  
-        if (trav == tree) {
-                fprintf (stderr, "Volume '%s', line %d: has '%s' itself as "
-			 "subvolume\n", 
-                         complete_tree->name, yylineno, sub);
-                gf_log ("parser", GF_LOG_ERROR, 
-                        "Volume '%s', line %d: has '%s' itself as subvolume", 
-                        complete_tree->name, yylineno, sub);
-                return -1;
-        }
-  
-        xlparent = (void *) calloc (1, sizeof (*xlparent));
-        xlparent->xlator = tree;
-
-        tmp = trav->parents;
-        if (tmp == NULL) {
-                trav->parents = xlparent;
-        } else {
-                while (tmp->next)
-                        tmp = tmp->next;
-                tmp->next = xlparent;
+			"prior to usage",
+                        curr->name, yylineno, sub);
+                ret = -1;
+                goto out;
         }
 
-        xlchild = (void *) calloc (1, sizeof(*xlchild));
-        xlchild->xlator = trav;
-
-        tmp = tree->children;
-        if (tmp == NULL) {
-                tree->children = xlchild;
-        } else {
-                while (tmp->next)
-                        tmp = tmp->next;
-                tmp->next = xlchild;
+        if (trav == curr) {
+                gf_log ("parser", GF_LOG_ERROR,
+                        "Volume '%s', line %d: has '%s' itself as subvolume",
+                        curr->name, yylineno, sub);
+                ret = -1;
+                goto out;
         }
 
-        gf_log ("parser", GF_LOG_TRACE, "child:%s->%s", tree->name, sub);
+	ret = glusterfs_xlator_link (curr, trav);
+	if (ret) {
+                gf_log ("parser", GF_LOG_ERROR, "Out of memory");
+                ret = -1;
+                goto out;
+        }
+
+        gf_log ("parser", GF_LOG_TRACE, "child:%s->%s", curr->name, sub);
+
+out:
+        GF_FREE (sub);
 
         return 0;
 }
 
+
 static int
-section_end (void)
+volume_end (void)
 {
-        if (!tree->fops || !tree->mops) {
-                fprintf (stderr, 
-                         "\"type\" not specified for volume %s\n", tree->name);
-                gf_log ("parser", GF_LOG_ERROR, 
-                        "\"type\" not specified for volume %s", tree->name);
+        if (!curr->fops) {
+                gf_log ("parser", GF_LOG_ERROR,
+                        "\"type\" not specified for volume %s", curr->name);
                 return -1;
         }
-        gf_log ("parser", GF_LOG_TRACE, "end:%s", tree->name);
+        gf_log ("parser", GF_LOG_TRACE, "end:%s", curr->name);
 
-        tree = NULL;
+        curr = NULL;
         return 0;
 }
 
+
 int
 yywrap ()
 {
         return 1;
 }
 
-int 
+
+int
 yyerror (const char *str)
 {
-        extern char *yytext;
-        extern int yylineno;
+        extern char  *yytext;
+        extern int    yylineno;
 
-        if (complete_tree && complete_tree->name) 
-        {
-                if (!strcmp (yytext, "volume")) 
-                {
-                        fprintf (stderr, 
-                                 "'end-volume' not defined for volume '%s'\n",
-				 complete_tree->name);
-                        gf_log ("parser", GF_LOG_ERROR, 
-                                "'end-volume' not defined for volume '%s'", 
-				complete_tree->name);
-                } 
-                else if (!strcmp (yytext, "type")) 
-                {
-                        fprintf (stderr, "line %d: duplicate 'type' defined "
-				 "for volume '%s'", 
-                                 yylineno, complete_tree->name);
-                        gf_log ("parser", GF_LOG_ERROR, 
+        if (curr && curr->name && yytext) {
+                if (!strcmp (yytext, "volume")) {
+                        gf_log ("parser", GF_LOG_ERROR,
+                                "'end-volume' not defined for volume '%s'",
+				curr->name);
+                } else if (!strcmp (yytext, "type")) {
+                        gf_log ("parser", GF_LOG_ERROR,
                                 "line %d: duplicate 'type' defined for "
-				"volume '%s'", 
-                                yylineno, complete_tree->name);
-                } 
-                else if (!strcmp (yytext, "subvolumes")) 
-                {
-                        fprintf (stderr, "line %d: duplicate 'subvolumes' "
-				 "defined for volume '%s'", 
-                                 yylineno, complete_tree->name);
-                        gf_log ("parser", GF_LOG_ERROR, 
+				"volume '%s'",
+                                yylineno, curr->name);
+                } else if (!strcmp (yytext, "subvolumes")) {
+                        gf_log ("parser", GF_LOG_ERROR,
                                 "line %d: duplicate 'subvolumes' defined for "
-				"volume '%s'", 
-                                yylineno, complete_tree->name);
-                } 
-                else if (tree) 
-                {
-                        fprintf (stderr, 
-                                 "syntax error: line %d (volume '%s'): \"%s\""
-				 "\nallowed tokens are 'volume', 'type', "
-				 "'subvolumes', 'option', 'end-volume'",
-                                 yylineno, complete_tree->name, 
-				 yytext);
-                                 
+				"volume '%s'",
+                                yylineno, curr->name);
+                } else if (curr) {
                         gf_log ("parser", GF_LOG_ERROR,
                                 "syntax error: line %d (volume '%s'): \"%s\""
 				"\nallowed tokens are 'volume', 'type', "
-				"'subvolumes', 'option', 'end-volume'()", 
-                                yylineno, complete_tree->name, 
+				"'subvolumes', 'option', 'end-volume'()",
+                                yylineno, curr->name,
 				yytext);
-                } 
-                else 
-                {
-                        fprintf (stderr, 
-                                 "syntax error: line %d (just after volume "
-				 "'%s'): \"%s\"\n(%s)", 
-                                 yylineno, complete_tree->name, 
-				 yytext,
-                                 "allowed tokens are 'volume', 'type', "
-				 "'subvolumes', 'option', 'end-volume'");
+                } else {
                         gf_log ("parser", GF_LOG_ERROR,
                                 "syntax error: line %d (just after volume "
-				"'%s'): \"%s\"\n(%s)", 
-                                yylineno, complete_tree->name, 
+				"'%s'): \"%s\"\n(%s)",
+                                yylineno, curr->name,
 				yytext,
                                 "allowed tokens are 'volume', 'type', "
 				"'subvolumes', 'option', 'end-volume'");
                 }
-        }
-        else 
-        {
-                fprintf (stderr, 
-                         "syntax error in line %d: \"%s\" \n"
-                         "(allowed tokens are 'volume', 'type', "
-			 "'subvolumes', 'option', 'end-volume')\n", 
-                         yylineno, yytext);
+        } else {
                 gf_log ("parser", GF_LOG_ERROR,
                         "syntax error in line %d: \"%s\" \n"
                         "(allowed tokens are 'volume', 'type', "
-			"'subvolumes', 'option', 'end-volume')\n", 
+			"'subvolumes', 'option', 'end-volume')\n",
                         yylineno, yytext);
         }
-  
-        cut_tree (tree);
-        complete_tree = NULL;
-        return 0;
+
+        return -1;
 }
 
+
 static int
 execute_cmd (char *cmd, char **result, size_t size)
 {
-	FILE *fpp = NULL;
-	int i = 0, status = 0;
-	int character = 0;
-	char *buf = *result;
+	FILE       *fpp = NULL;
+	int         i = 0;
+        int         status = 0;
+	int         character = 0;
+	char       *buf = *result;
 
 	fpp = popen (cmd, "r");
 	if (!fpp) {
@@ -2023,8 +1992,8 @@ execute_cmd (char *cmd, char **result, size_t size)
 	while ((character = fgetc (fpp)) != EOF) {
 		if (i == size) {
 			size *= 2;
-			buf = *result = realloc (*result, size);
-	}
+			buf = *result = GF_REALLOC (*result, size);
+                }
 
 		buf[i++] = character;
 	}
@@ -2033,9 +2002,9 @@ execute_cmd (char *cmd, char **result, size_t size)
 		i--;
 		buf[i] = '\0';
 	}
-	
+
 	status = pclose (fpp);
-	if (status == -1 || !WIFEXITED (status) || 
+	if (status == -1 || !WIFEXITED (status) ||
 	    ((WEXITSTATUS (status)) != 0)) {
 		i = -1;
 		buf[0] = '\0';
@@ -2044,26 +2013,39 @@ execute_cmd (char *cmd, char **result, size_t size)
 	return i;
 }
 
+
 static int
-parse_backtick (FILE *srcfp, FILE *dstfp)
+preprocess (FILE *srcfp, FILE *dstfp)
 {
-	int ret = 0, i = 0;
-	char *cmd = NULL, *result = NULL;
-	size_t cmd_buf_size = GF_CMD_BUFFER_LEN;
-	char escaped = 0, in_backtick = 0;
-	int line = 1, column = 0, backtick_line = 0, backtick_column = 0;
-        int character = 0;
+	int     ret = 0;
+        int     i = 0;
+	char   *cmd = NULL;
+        char   *result = NULL;
+	size_t  cmd_buf_size = GF_CMD_BUFFER_LEN;
+	char    escaped = 0;
+        char    in_backtick = 0;
+	int     line = 1;
+        int     column = 0;
+        int     backtick_line = 0;
+        int     backtick_column = 0;
+        int     character = 0;
+
 
 	fseek (srcfp, 0L, SEEK_SET);
 	fseek (dstfp, 0L, SEEK_SET);
 
-	cmd = CALLOC (cmd_buf_size, 1);
+	cmd = GF_CALLOC (cmd_buf_size, 1,
+                         gf_common_mt_char);
         if (cmd == NULL) {
+                gf_log ("parser", GF_LOG_ERROR, "Out of memory");
                 return -1;
         }
 
-	result = CALLOC (cmd_buf_size * 2, 1);
+	result = GF_CALLOC (cmd_buf_size * 2, 1,
+                            gf_common_mt_char);
         if (result == NULL) {
+                GF_FREE (cmd);
+                gf_log ("parser", GF_LOG_ERROR, "Out of memory");
                 return -1;
         }
 
@@ -2083,41 +2065,41 @@ parse_backtick (FILE *srcfp, FILE *dstfp)
 			} else {
 				i = 0;
 				cmd[i] = '\0';
-				
+
 				backtick_column = column;
 				backtick_line = line;
 			}
-			
+
 			in_backtick = !in_backtick;
 		} else {
 			if (in_backtick) {
 				if (i == cmd_buf_size) {
 					cmd_buf_size *= 2;
-					cmd = realloc (cmd, cmd_buf_size);
+					cmd = GF_REALLOC (cmd, cmd_buf_size);
                                         if (cmd == NULL) {
                                                 return -1;
                                         }
 
-					result = realloc (result,
-                                                          2 * cmd_buf_size);
+					result = GF_REALLOC (result,
+                                                             2 * cmd_buf_size);
                                         if (result == NULL) {
-                                                free (cmd);
+                                                GF_FREE (cmd);
                                                 return -1;
                                         }
                                 }
-                                
+
 				cmd[i++] = character;
                         } else {
 				fputc (character, dstfp);
                         }
                 }
-                
+
 		if (character == '\\') {
 			escaped = !escaped;
 		} else {
 			escaped = 0;
                 }
-		
+
 		if (character == '\n') {
 			line++;
 			column = 0;
@@ -2125,67 +2107,95 @@ parse_backtick (FILE *srcfp, FILE *dstfp)
 			column++;
 		}
         }
-        
+
 	if (in_backtick) {
 		gf_log ("parser", GF_LOG_ERROR,
-			"Unterminated backtick in volume specfication file at line (%d), column (%d).", 
+			"Unterminated backtick in volume specfication file at line (%d), column (%d).",
 			line, column);
                 ret = -1;
-	} 
-        
+	}
+
 out:
 	fseek (srcfp, 0L, SEEK_SET);
 	fseek (dstfp, 0L, SEEK_SET);
-	free (cmd);
-	free (result);
-        
+	GF_FREE (cmd);
+	GF_FREE (result);
+
 	return ret;
 }
 
+
 extern FILE *yyin;
-xlator_t *
-file_to_xlator_tree (glusterfs_ctx_t *ctx,
-                     FILE *fp)
+
+glusterfs_graph_t *
+glusterfs_graph_new ()
 {
-        int32_t ret = 0;
-        xlator_t *tmp_tree = NULL;
-	FILE *tmp_file = NULL;
-	char *buffer = NULL;
+        glusterfs_graph_t *graph = NULL;
+
+        graph = GF_CALLOC (1, sizeof (*graph),
+                           gf_common_mt_glusterfs_graph_t);
+        if (!graph)
+                return NULL;
+
+        INIT_LIST_HEAD (&graph->list);
+
+        gettimeofday (&graph->dob, NULL);
+
+        return graph;
+}
+
+
+glusterfs_graph_t *
+glusterfs_graph_construct (FILE *fp)
+{
+        int                ret = 0;
+        glusterfs_graph_t *graph = NULL;
+	FILE              *tmp_file = NULL;
+
+        graph = glusterfs_graph_new ();
+        if (!graph)
+                return NULL;
 
 	tmp_file = tmpfile ();
-	if (NULL == tmp_file) {
+
+	if (tmp_file == NULL) {
 		gf_log ("parser", GF_LOG_ERROR,
 			"cannot create temparory file");
+
+                glusterfs_graph_destroy (graph);
 		return NULL;
 	}
 
-	ret = parse_backtick (fp, tmp_file);
+	ret = preprocess (fp, tmp_file);
 	if (ret < 0) {
 		gf_log ("parser", GF_LOG_ERROR,
 			"parsing of backticks failed");
+
+                glusterfs_graph_destroy (graph);
 		fclose (tmp_file);
-		FREE (buffer);
 		return NULL;
 	}
 
-        gctx = ctx;
         yyin = tmp_file;
-        ret = yyparse ();
-  
-	fclose (tmp_file);
-	FREE (buffer);
 
-        if (1 == ret) {
-                gf_log ("parser", GF_LOG_DEBUG, 
+        construct = graph;
+
+        ret = yyparse ();
+
+        construct = NULL;
+
+	fclose (tmp_file);
+
+        if (ret == 1) {
+                gf_log ("parser", GF_LOG_DEBUG,
 			"parsing of volfile failed, please review it "
 			"once more");
-                tree = complete_tree = NULL;
+
+                glusterfs_graph_destroy (graph);
                 return NULL;
         }
 
-        tmp_tree = complete_tree;
-        tree = complete_tree = NULL;
-
-        return tmp_tree;
+        return graph;
 }
 
+
