@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -54,29 +54,56 @@
 static void
 sh_full_private_cleanup (call_frame_t *frame, xlator_t *this)
 {
-        afr_private_t *             priv    = NULL;
         afr_local_t *               local   = NULL;
         afr_self_heal_t *           sh      = NULL;
         afr_sh_algo_full_private_t *sh_priv = NULL;
 
-        priv  = this->private;
         local = frame->local;
         sh    = &local->self_heal;
 
         sh_priv = sh->private;
 
         if (sh_priv)
-                FREE (sh_priv);
+                GF_FREE (sh_priv);
 }
 
 
 static int
-sh_full_loop_driver (call_frame_t *frame, xlator_t *this);
+sh_full_loop_driver (call_frame_t *frame, xlator_t *this, gf_boolean_t is_first_call);
+
+static int
+sh_full_loop_driver_done (call_frame_t *frame, xlator_t *this)
+{
+        afr_private_t * priv = NULL;
+        afr_local_t * local  = NULL;
+        afr_self_heal_t *sh  = NULL;
+        afr_sh_algo_full_private_t *sh_priv = NULL;
+
+        priv    = this->private;
+        local   = frame->local;
+        sh      = &local->self_heal;
+        sh_priv = sh->private;
+
+        sh_full_private_cleanup (frame, this);
+        if (sh->op_failed) {
+                gf_log (this->name, GF_LOG_TRACE,
+                        "full self-heal aborting on %s",
+                        local->loc.path);
+
+                local->self_heal.algo_abort_cbk (frame, this);
+        } else {
+                gf_log (this->name, GF_LOG_TRACE,
+                        "full self-heal completed on %s",
+                        local->loc.path);
+
+                local->self_heal.algo_completion_cbk (frame, this);
+        }
+        return 0;
+}
 
 static int
 sh_full_loop_return (call_frame_t *rw_frame, xlator_t *this, off_t offset)
 {
-        afr_private_t *             priv       = NULL;
         afr_local_t *               rw_local   = NULL;
         afr_self_heal_t *           rw_sh      = NULL;
 
@@ -85,8 +112,6 @@ sh_full_loop_return (call_frame_t *rw_frame, xlator_t *this, off_t offset)
 	afr_self_heal_t *sh                 = NULL;
         afr_sh_algo_full_private_t *sh_priv = NULL;
 
-        priv  = this->private;
-
         rw_local = rw_frame->local;
         rw_sh    = &rw_local->self_heal;
 
@@ -95,18 +120,9 @@ sh_full_loop_return (call_frame_t *rw_frame, xlator_t *this, off_t offset)
         sh       = &sh_local->self_heal;
         sh_priv  = sh->private;
 
-        LOCK (&sh_priv->lock);
-        {
-                sh_priv->loops_running--;
-        }
-        UNLOCK (&sh_priv->lock);
-
-        gf_log (this->name, GF_LOG_TRACE,
-                "loop for offset %"PRId64" returned", offset);
-
         AFR_STACK_DESTROY (rw_frame);
 
-        sh_full_loop_driver (sh_frame, this);
+        sh_full_loop_driver (sh_frame, this, _gf_false);
 
         return 0;
 }
@@ -114,8 +130,8 @@ sh_full_loop_return (call_frame_t *rw_frame, xlator_t *this, off_t offset)
 
 static int
 sh_full_write_cbk (call_frame_t *rw_frame, void *cookie, xlator_t *this,
-                   int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                   struct stat *postbuf)
+                   int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                   struct iatt *postbuf)
 {
 	afr_private_t * priv    = NULL;
 	afr_local_t * rw_local  = NULL;
@@ -169,7 +185,7 @@ sh_full_write_cbk (call_frame_t *rw_frame, void *cookie, xlator_t *this,
 static int
 sh_full_read_cbk (call_frame_t *rw_frame, void *cookie,
                   xlator_t *this, int32_t op_ret, int32_t op_errno,
-                  struct iovec *vector, int32_t count, struct stat *buf,
+                  struct iovec *vector, int32_t count, struct iatt *buf,
                   struct iobref *iobref)
 {
 	afr_private_t * priv    = NULL;
@@ -203,7 +219,6 @@ sh_full_read_cbk (call_frame_t *rw_frame, void *cookie,
 
 	if (op_ret <= 0) {
                 sh->op_failed = 1;
-
                 sh_full_loop_return (rw_frame, this, offset);
 		return 0;
 	}
@@ -270,7 +285,7 @@ sh_full_read_write (call_frame_t *frame, xlator_t *this, off_t offset)
         rw_frame->local = rw_local;
 	rw_sh           = &rw_local->self_heal;
 
-        rw_sh->offset       = sh->offset;
+        rw_sh->offset       = offset;
         rw_sh->sh_frame     = frame;
 
 	STACK_WIND_COOKIE (rw_frame, sh_full_read_cbk,
@@ -284,90 +299,72 @@ sh_full_read_write (call_frame_t *frame, xlator_t *this, off_t offset)
 out:
         sh->op_failed = 1;
 
-        sh_full_loop_driver (frame, this);
+        sh_full_loop_driver (frame, this, _gf_false);
 
 	return 0;
 }
 
 
 static int
-sh_full_loop_driver (call_frame_t *frame, xlator_t *this)
+sh_full_loop_driver (call_frame_t *frame, xlator_t *this, gf_boolean_t is_first_call)
 {
 	afr_private_t * priv = NULL;
 	afr_local_t * local  = NULL;
 	afr_self_heal_t *sh  = NULL;
         afr_sh_algo_full_private_t *sh_priv = NULL;
+        gf_boolean_t    is_driver_done = _gf_false;
+        blksize_t       block_size = 0;
+        off_t offset  = 0;
 
         int   loop    = 0;
-        int   recurse = 0;
-
-        off_t offset  = 0;
 
 	priv    = this->private;
 	local   = frame->local;
 	sh      = &local->self_heal;
         sh_priv = sh->private;
 
-	if (sh->op_failed) {
-                if (sh_priv->loops_running == 0) {
-                        gf_log (this->name, GF_LOG_TRACE,
-                                "full self-heal aborting on %s",
-                                local->loc.path);
-
-                        sh_full_private_cleanup (frame, this);
-                        local->self_heal.algo_abort_cbk (frame, this);
-                }
-
-		goto out;
-	}
-
-	if (sh_priv->offset >= sh->file_size) {
-                if (sh_priv->loops_running == 0) {
-
-                        gf_log (this->name, GF_LOG_TRACE,
-                                "full self-heal completed on %s",
-                                local->loc.path);
-
-                        sh_full_private_cleanup (frame, this);
-                        local->self_heal.algo_completion_cbk (frame, this);
-                }
-
-		goto out;
-	}
-
-spawn:
-        loop    = 0;
-        recurse = 0;
-
         LOCK (&sh_priv->lock);
         {
-                if ((sh_priv->loops_running < priv->data_self_heal_window_size)
+                if (_gf_false == is_first_call)
+                        sh_priv->loops_running--;
+                offset           = sh_priv->offset;
+                block_size       = sh->block_size;
+                while ((sh->op_failed == 0) &&
+                    (sh_priv->loops_running < priv->data_self_heal_window_size)
                     && (sh_priv->offset < sh->file_size)) {
 
+                        loop++;
                         gf_log (this->name, GF_LOG_TRACE,
                                 "spawning a loop for offset %"PRId64,
                                 sh_priv->offset);
 
-                        offset           = sh_priv->offset;
                         sh_priv->offset += sh->block_size;
-
                         sh_priv->loops_running++;
 
-                        loop = 1;
+                        if (_gf_false == is_first_call)
+                                break;
 
-                        if (sh_priv->offset < sh->file_size)
-                                recurse = 1;
+                }
+                if (0 == sh_priv->loops_running) {
+                        is_driver_done = _gf_true;
                 }
         }
         UNLOCK (&sh_priv->lock);
 
-        if (loop) {
-                sh_full_read_write (frame, this, offset);
-                if (recurse)
-                        goto spawn;
+        while (loop--) {
+                if (sh->op_failed) {
+                        // op failed in other loop, stop spawning more loops
+                        sh_full_loop_driver (frame, this, _gf_false);
+                } else {
+                        sh_full_read_write (frame, this, offset);
+                        offset += block_size;
+                }
+        }
+
+        if (is_driver_done) {
+                sh_full_loop_driver_done (frame, this);
         }
 
-out:
 	return 0;
 }
 
@@ -375,16 +372,15 @@ out:
 int
 afr_sh_algo_full (call_frame_t *frame, xlator_t *this)
 {
-        afr_private_t *             priv    = NULL;
         afr_local_t *               local   = NULL;
         afr_self_heal_t *           sh      = NULL;
         afr_sh_algo_full_private_t *sh_priv = NULL;
 
-        priv  = this->private;
         local = frame->local;
         sh    = &local->self_heal;
 
-        sh_priv = CALLOC (1, sizeof (*sh_priv));
+        sh_priv = GF_CALLOC (1, sizeof (*sh_priv),
+                             gf_afr_mt_afr_private_t);
 
         LOCK_INIT (&sh_priv->lock);
 
@@ -392,7 +388,7 @@ afr_sh_algo_full (call_frame_t *frame, xlator_t *this)
 
         local->call_count = 0;
 
-        sh_full_loop_driver (frame, this);
+        sh_full_loop_driver (frame, this, _gf_true);
         return 0;
 }
 
@@ -422,20 +418,20 @@ sh_diff_private_cleanup (call_frame_t *frame, xlator_t *this)
         for (i = 0; i < priv->data_self_heal_window_size; i++) {
                 if (sh_priv->loops[i]) {
                         if (sh_priv->loops[i]->write_needed)
-                                FREE (sh_priv->loops[i]->write_needed);
+                                GF_FREE (sh_priv->loops[i]->write_needed);
 
                         if (sh_priv->loops[i]->checksum)
-                                FREE (sh_priv->loops[i]->checksum);
+                                GF_FREE (sh_priv->loops[i]->checksum);
 
-                        FREE (sh_priv->loops[i]);
+                        GF_FREE (sh_priv->loops[i]);
                 }
         }
 
         if (sh_priv) {
                 if (sh_priv->loops)
-                        FREE (sh_priv->loops);
+                        GF_FREE (sh_priv->loops);
 
-                FREE (sh_priv);
+                GF_FREE (sh_priv);
         }
 
 
@@ -494,9 +490,52 @@ sh_diff_number_of_writes_needed (unsigned char *write_needed, int child_count)
 
 
 static int
-sh_diff_loop_driver (call_frame_t *frame, xlator_t *this);
+sh_diff_loop_driver_done (call_frame_t *frame, xlator_t *this)
+{
+        afr_private_t *   priv  = NULL;
+        afr_local_t *     local = NULL;
+        afr_self_heal_t * sh    = NULL;
+        afr_sh_algo_diff_private_t *sh_priv = NULL;
+        int32_t total_blocks = 0;
+        int32_t diff_blocks = 0;
 
 
+        priv         = this->private;
+        local        = frame->local;
+        sh           = &local->self_heal;
+        sh_priv      = sh->private;
+        total_blocks = sh_priv->total_blocks;
+        diff_blocks  = sh_priv->diff_blocks;
+
+        sh_diff_private_cleanup (frame, this);
+        if (sh->op_failed) {
+                gf_log (this->name, GF_LOG_TRACE,
+                        "diff self-heal aborting on %s",
+                        local->loc.path);
+
+                local->self_heal.algo_abort_cbk (frame, this);
+        } else {
+                gf_log (this->name, GF_LOG_TRACE,
+                        "diff self-heal completed on %s",
+                        local->loc.path);
+
+
+                gf_log (this->name, GF_LOG_NORMAL,
+                        "diff self-heal on %s: %d blocks of %d were different (%.2f%%)",
+                        local->loc.path, diff_blocks, total_blocks,
+                        ((diff_blocks * 1.0)/total_blocks) * 100);
+
+                local->self_heal.algo_completion_cbk (frame, this);
+        }
+
+        return 0;
+}
+
+static int
+sh_diff_loop_driver (call_frame_t *frame, xlator_t *this,
+                     gf_boolean_t is_first_call,
+                     struct sh_diff_loop_state *loop_state);
+
 static int
 sh_diff_loop_return (call_frame_t *rw_frame, xlator_t *this,
                      struct sh_diff_loop_state *loop_state)
@@ -523,16 +562,9 @@ sh_diff_loop_return (call_frame_t *rw_frame, xlator_t *this,
         gf_log (this->name, GF_LOG_TRACE,
                 "loop for offset %"PRId64" returned", loop_state->offset);
 
-        LOCK (&sh_priv->lock);
-        {
-                sh_priv->loops_running--;
-                sh_diff_loop_state_reset (loop_state, priv->child_count);
-        }
-        UNLOCK (&sh_priv->lock);
-
         AFR_STACK_DESTROY (rw_frame);
 
-        sh_diff_loop_driver (sh_frame, this);
+        sh_diff_loop_driver (sh_frame, this, _gf_false, loop_state);
 
         return 0;
 }
@@ -540,8 +572,8 @@ sh_diff_loop_return (call_frame_t *rw_frame, xlator_t *this,
 
 static int
 sh_diff_write_cbk (call_frame_t *rw_frame, void *cookie, xlator_t *this,
-                   int32_t op_ret, int32_t op_errno, struct stat *buf,
-                   struct stat *postbuf)
+                   int32_t op_ret, int32_t op_errno, struct iatt *buf,
+                   struct iatt *postbuf)
 {
 	afr_private_t *   priv     = NULL;
 	afr_local_t *     rw_local = NULL;
@@ -603,7 +635,7 @@ sh_diff_write_cbk (call_frame_t *rw_frame, void *cookie, xlator_t *this,
 static int
 sh_diff_read_cbk (call_frame_t *rw_frame, void *cookie,
                   xlator_t *this, int32_t op_ret, int32_t op_errno,
-                  struct iovec *vector, int32_t count, struct stat *buf,
+                  struct iovec *vector, int32_t count, struct iatt *buf,
                   struct iobref *iobref)
 {
 	afr_private_t *   priv     = NULL;
@@ -643,7 +675,7 @@ sh_diff_read_cbk (call_frame_t *rw_frame, void *cookie,
 
 	gf_log (this->name, GF_LOG_TRACE,
 		"read %d bytes of data from %s, offset %"PRId64"",
-		op_ret, sh_local->loc.path, sh->offset);
+		op_ret, sh_local->loc.path, loop_state->offset);
 
 	if ((op_ret <= 0) ||
             (call_count == 0)) {
@@ -928,96 +960,79 @@ sh_diff_checksum (call_frame_t *frame, xlator_t *this, off_t offset)
 out:
         sh->op_failed = 1;
 
-        sh_diff_loop_driver (frame, this);
+        sh_diff_loop_driver (frame, this, _gf_false, loop_state);
 
         return 0;
 }
 
 
 static int
-sh_diff_loop_driver (call_frame_t *frame, xlator_t *this)
+sh_diff_loop_driver (call_frame_t *frame, xlator_t *this,
+                     gf_boolean_t is_first_call,
+                     struct sh_diff_loop_state *loop_state)
 {
 	afr_private_t *   priv  = NULL;
 	afr_local_t *     local = NULL;
 	afr_self_heal_t * sh    = NULL;
         afr_sh_algo_diff_private_t *sh_priv = NULL;
+        gf_boolean_t      is_driver_done = _gf_false;
+        blksize_t         block_size = 0;
 
         int   loop    = 0;
-        int   recurse = 0;
 
         off_t offset = 0;
- 
+        char  sh_type_str[256] = {0,};
+
 	priv    = this->private;
 	local   = frame->local;
 	sh      = &local->self_heal;
         sh_priv = sh->private;
 
-	if (sh->op_failed) {
-                if (sh_priv->loops_running == 0) {
-                        gf_log (this->name, GF_LOG_TRACE,
-                                "diff self-heal aborting on %s",
-                                local->loc.path);
-
-                        sh_diff_private_cleanup (frame, this);
-                        local->self_heal.algo_abort_cbk (frame, this);
-                }
-
-		goto out;
-	}
-
-	if (sh_priv->offset >= sh->file_size) {
-                if (sh_priv->loops_running == 0) {
-                        gf_log (this->name, GF_LOG_TRACE,
-                                "diff self-heal completed on %s",
-                                local->loc.path);
-
-
-                        gf_log (this->name, GF_LOG_DEBUG,
-                                "diff self-heal on %s: %d blocks of %d were different (%.2f%%)",
-                                local->loc.path, sh_priv->diff_blocks,
-                                sh_priv->total_blocks,
-                                ((sh_priv->diff_blocks * 1.0)/sh_priv->total_blocks) * 100);
-
-                        sh_diff_private_cleanup (frame, this);
-                        local->self_heal.algo_completion_cbk (frame, this);
-                }
-
-		goto out;
-	}
-
-spawn:
-        loop    = 0;
-        recurse = 0;
+        afr_self_heal_type_str_get(sh, sh_type_str, sizeof(sh_type_str));
 
         LOCK (&sh_priv->lock);
         {
-                if ((sh_priv->loops_running < priv->data_self_heal_window_size)
+                if (loop_state)
+                        sh_diff_loop_state_reset (loop_state, priv->child_count);
+                if (_gf_false == is_first_call)
+                        sh_priv->loops_running--;
+                offset = sh_priv->offset;
+                block_size = sh_priv->block_size;
+                while ((0 == sh->op_failed) &&
+                    (sh_priv->loops_running < priv->data_self_heal_window_size)
                     && (sh_priv->offset < sh->file_size)) {
 
+                        loop++;
                         gf_log (this->name, GF_LOG_TRACE,
                                 "spawning a loop for offset %"PRId64,
                                 sh_priv->offset);
 
-                        offset           = sh_priv->offset;
                         sh_priv->offset += sh_priv->block_size;
-
                         sh_priv->loops_running++;
 
-                        loop = 1;
+                        if (_gf_false == is_first_call)
+                                break;
 
-                        if (sh_priv->offset < sh->file_size)
-                                recurse = 1;
+                }
+                if (0 == sh_priv->loops_running) {
+                        is_driver_done = _gf_true;
                 }
         }
         UNLOCK (&sh_priv->lock);
 
-        if (loop) {
-                sh_diff_checksum (frame, this, offset);
-                if (recurse)
-                    goto spawn;
+        while (loop--) {
+                if (sh->op_failed) {
+                        // op failed in other loop, stop spawning more loops
+                        sh_diff_loop_driver (frame, this, _gf_false, NULL);
+                } else {
+                        sh_diff_checksum (frame, this, offset);
+                        offset += block_size;
+                }
         }
 
-out:
+        if (is_driver_done) {
+                sh_diff_loop_driver_done (frame, this);
+        }
         return 0;
 }
 
@@ -1036,7 +1051,8 @@ afr_sh_algo_diff (call_frame_t *frame, xlator_t *this)
         local = frame->local;
         sh    = &local->self_heal;
 
-        sh_priv = CALLOC (1, sizeof (*sh_priv));
+        sh_priv = GF_CALLOC (1, sizeof (*sh_priv),
+                             gf_afr_mt_afr_private_t);
 
         sh_priv->block_size = this->ctx->page_size;
 
@@ -1046,19 +1062,22 @@ afr_sh_algo_diff (call_frame_t *frame, xlator_t *this)
 
         local->call_count = 0;
 
-        sh_priv->loops = CALLOC (priv->data_self_heal_window_size,
-                                 sizeof (*sh_priv->loops));
+        sh_priv->loops = GF_CALLOC (priv->data_self_heal_window_size,
+                                    sizeof (*sh_priv->loops),
+                                    gf_afr_mt_sh_diff_loop_state);
 
         for (i = 0; i < priv->data_self_heal_window_size; i++) {
-                sh_priv->loops[i]               = CALLOC (1, sizeof (*sh_priv->loops[i]));
+                sh_priv->loops[i]               = GF_CALLOC (1, sizeof (*sh_priv->loops[i]),
+                                                             gf_afr_mt_sh_diff_loop_state);
 
-                sh_priv->loops[i]->checksum     = CALLOC (priv->child_count,
-                                                          MD5_DIGEST_LEN);
-                sh_priv->loops[i]->write_needed = CALLOC (priv->child_count,
-                                                          sizeof (*sh_priv->loops[i]->write_needed));
+                sh_priv->loops[i]->checksum     = GF_CALLOC (priv->child_count,
+                                                             MD5_DIGEST_LEN, gf_afr_mt_uint8_t);
+                sh_priv->loops[i]->write_needed = GF_CALLOC (priv->child_count,
+                                                             sizeof (*sh_priv->loops[i]->write_needed),
+                                                             gf_afr_mt_char);
         }
 
-        sh_diff_loop_driver (frame, this);
+        sh_diff_loop_driver (frame, this, _gf_true, NULL);
 
         return 0;
 }
