@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -62,12 +62,12 @@ afr_sh_metadata_done (call_frame_t *frame, xlator_t *this)
 	priv = this->private;
 
 //	memset (sh->child_errno, 0, sizeof (int) * priv->child_count);
-	memset (sh->buf, 0, sizeof (struct stat) * priv->child_count);
+	memset (sh->buf, 0, sizeof (struct iatt) * priv->child_count);
 	memset (sh->success, 0, sizeof (int) * priv->child_count);
 
-        for (i = 0; i < priv->child_count; i++) {
-                sh->locked_nodes[i] = 1;
-        }
+/*         for (i = 0; i < priv->child_count; i++) { */
+/*                 sh->locked_nodes[i] = 1; */
+/*         } */
 
 	for (i = 0; i < priv->child_count; i++) {
 		if (sh->xattr[i])
@@ -81,7 +81,7 @@ afr_sh_metadata_done (call_frame_t *frame, xlator_t *this)
 			local->loc.path);
 		sh->completion_cbk (frame, this);
 	} else {
-		if (S_ISREG (sh->mode)) {
+		if (IA_ISREG (sh->type)) {
 			gf_log (this->name, GF_LOG_TRACE,
 				"proceeding to data check on %s",
 				local->loc.path);
@@ -89,7 +89,7 @@ afr_sh_metadata_done (call_frame_t *frame, xlator_t *this)
 			return 0;
 		}
 
-		if (S_ISDIR (sh->mode)) {
+		if (IA_ISDIR (sh->type)) {
 			gf_log (this->name, GF_LOG_TRACE,
 				"proceeding to entry check on %s",
 				local->loc.path);
@@ -111,12 +111,8 @@ int
 afr_sh_metadata_unlck_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			   int32_t op_ret, int32_t op_errno)
 {
-	afr_local_t      *local = NULL;
 	int               call_count = 0;
 
-
-	local = frame->local;
-
 	call_count = afr_frame_return (frame);
 
 	if (call_count == 0)
@@ -125,54 +121,25 @@ afr_sh_metadata_unlck_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	return 0;
 }
 
+int
+afr_sh_inode_unlock (call_frame_t *frame, xlator_t *this)
+{
+        afr_internal_lock_t *int_lock = NULL;
+        afr_local_t         *local    = NULL;
+
+        local    = frame->local;
+        int_lock = &local->internal_lock;
+
+        int_lock->lock_cbk = afr_sh_metadata_done;
+        afr_unlock (frame, this);
+
+        return 0;
+}
 
 int
 afr_sh_metadata_finish (call_frame_t *frame, xlator_t *this)
 {
-	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
-	afr_private_t   *priv = NULL;
-	int              i = 0;
-	int              call_count = 0;
-	struct flock     flock = {0, };
-
-
-	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
-
-        for (i = 0; i < priv->child_count; i++) {
-                if (sh->locked_nodes[i])
-                        call_count++;
-        }
-
-        if (call_count == 0) {
-                afr_sh_metadata_done (frame, this);
-                return 0;
-        }
-
-	local->call_count = call_count;
-
-	for (i = 0; i < priv->child_count; i++) {
-		flock.l_start   = 0;
-		flock.l_len     = 0;
-		flock.l_type    = F_UNLCK;
-
-		if (sh->locked_nodes[i]) {
-			gf_log (this->name, GF_LOG_TRACE,
-				"unlocking %s on subvolume %s",
-				local->loc.path, priv->children[i]->name);
-
-			STACK_WIND (frame, afr_sh_metadata_unlck_cbk,
-				    priv->children[i],
-				    priv->children[i]->fops->inodelk,
-                                    this->name,
-				    &local->loc, F_SETLK, &flock);
-
-			if (!--call_count)
-				break;
-		}
-	}
+        afr_sh_inode_unlock (frame, this);
 
 	return 0;
 }
@@ -184,13 +151,9 @@ afr_sh_metadata_erase_pending_cbk (call_frame_t *frame, void *cookie,
 				   int32_t op_errno, dict_t *xattr)
 {
 	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
-	afr_private_t   *priv = NULL;
 	int             call_count = 0;
 
 	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
 
 	LOCK (&frame->lock);
 	{
@@ -225,7 +188,8 @@ afr_sh_metadata_erase_pending (call_frame_t *frame, xlator_t *this)
                                  sh->success, priv->child_count,
                                  AFR_METADATA_TRANSACTION);
 
-	erase_xattr = CALLOC (sizeof (*erase_xattr), priv->child_count);
+	erase_xattr = GF_CALLOC (sizeof (*erase_xattr), priv->child_count,
+                                 gf_afr_mt_dict_t);
 
 	for (i = 0; i < priv->child_count; i++) {
 		if (sh->xattr[i]) {
@@ -272,7 +236,7 @@ afr_sh_metadata_erase_pending (call_frame_t *frame, xlator_t *this)
 			dict_unref (erase_xattr[i]);
 		}
 	}
-	FREE (erase_xattr);
+	GF_FREE (erase_xattr);
 
 	return 0;
 }
@@ -321,7 +285,7 @@ afr_sh_metadata_sync_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 int
 afr_sh_metadata_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                              int32_t op_ret, int32_t op_errno,
-                             struct stat *preop, struct stat *postop)
+                             struct iatt *preop, struct iatt *postop)
 {
 	afr_sh_metadata_sync_cbk (frame, cookie, this, op_ret, op_errno);
 
@@ -350,7 +314,7 @@ afr_sh_metadata_sync (call_frame_t *frame, xlator_t *this, dict_t *xattr)
 	int              call_count = 0;
 	int              i = 0;
 
-        struct stat      stbuf;
+        struct iatt      stbuf;
         int32_t          valid = 0;
 
 	local = frame->local;
@@ -370,22 +334,16 @@ afr_sh_metadata_sync (call_frame_t *frame, xlator_t *this, dict_t *xattr)
 
 	local->call_count = call_count;
 
-#ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
-	stbuf.st_atim = sh->buf[source].st_atim;
-	stbuf.st_mtim = sh->buf[source].st_mtim;
-        
-#elif HAVE_STRUCT_STAT_ST_ATIMESPEC_TV_NSEC
-	stbuf.st_atimespec = sh->buf[source].st_atimespec;
-	stbuf.st_mtimespec = sh->buf[source].st_mtimespec;
-#else
-	stbuf.st_atime = sh->buf[source].st_atime;
-	stbuf.st_mtime = sh->buf[source].st_mtime;
-#endif
+	stbuf.ia_atime = sh->buf[source].ia_atime;
+	stbuf.ia_atime_nsec = sh->buf[source].ia_atime_nsec;
+	stbuf.ia_mtime = sh->buf[source].ia_mtime;
+	stbuf.ia_mtime_nsec = sh->buf[source].ia_mtime_nsec;
 
-        stbuf.st_uid = sh->buf[source].st_uid;
-        stbuf.st_gid = sh->buf[source].st_gid;
+        stbuf.ia_uid = sh->buf[source].ia_uid;
+        stbuf.ia_gid = sh->buf[source].ia_gid;
 
-        stbuf.st_mode = sh->buf[source].st_mode;
+        stbuf.ia_type = sh->buf[source].ia_type;
+        stbuf.ia_prot = sh->buf[source].ia_prot;
         
         valid = GF_SET_ATTR_MODE  | 
                 GF_SET_ATTR_UID   | GF_SET_ATTR_GID |
@@ -604,8 +562,8 @@ afr_sh_metadata_fix (call_frame_t *frame, xlator_t *this)
 int
 afr_sh_metadata_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			    int32_t op_ret, int32_t op_errno,
-                            inode_t *inode, struct stat *buf, dict_t *xattr,
-                            struct stat *postparent)
+                            inode_t *inode, struct iatt *buf, dict_t *xattr,
+                            struct iatt *postparent)
 {
 	afr_local_t     *local = NULL;
 	afr_self_heal_t *sh = NULL;
@@ -627,7 +585,7 @@ afr_sh_metadata_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 				"path %s on subvolume %s is of mode 0%o",
 				local->loc.path,
 				priv->children[child_index]->name,
-				buf->st_mode);
+				buf->ia_type);
 
 			sh->buf[child_index] = *buf;
 			if (xattr)
@@ -657,7 +615,6 @@ int
 afr_sh_metadata_lookup (call_frame_t *frame, xlator_t *this)
 {
 	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
 	afr_private_t   *priv = NULL;
 	int              i = 0;
 	int              call_count = 0;
@@ -665,20 +622,22 @@ afr_sh_metadata_lookup (call_frame_t *frame, xlator_t *this)
 	int              ret = 0;
 
 	local = frame->local;
-	sh = &local->self_heal;
 	priv = this->private;
 
 	call_count = afr_up_children_count (priv->child_count,
                                             local->child_up);
 	local->call_count = call_count;
-	
+
 	xattr_req = dict_new();
-	
+
 	if (xattr_req) {
                 for (i = 0; i < priv->child_count; i++) {
-                        ret = dict_set_uint64 (xattr_req, 
+                        ret = dict_set_uint64 (xattr_req,
                                                priv->pending_key[i],
                                                3 * sizeof(int32_t));
+                        if (ret < 0)
+                                gf_log (this->name, GF_LOG_WARNING,
+                                        "Unable to set dict value.");
                 }
         }
 
@@ -704,97 +663,49 @@ afr_sh_metadata_lookup (call_frame_t *frame, xlator_t *this)
 	return 0;
 }
 
-
 int
-afr_sh_metadata_lk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-			int32_t op_ret, int32_t op_errno)
+afr_sh_post_nonblocking_inodelk_cbk (call_frame_t *frame, xlator_t *this)
 {
-	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
-	afr_private_t   *priv = NULL;
-	int              call_count = 0;
-	int              child_index = (long) cookie;
+        afr_internal_lock_t *int_lock = NULL;
+        afr_local_t         *local    = NULL;
 
-	/* TODO: what if lock fails? */
-	
-	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
+        local    = frame->local;
+        int_lock = &local->internal_lock;
 
-	LOCK (&frame->lock);
-	{
-		if (op_ret == -1) {
-			sh->op_failed = 1;
+        if (int_lock->lock_op_ret < 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Non Blocking inodelks failed.");
+                afr_sh_metadata_done (frame, this);
+        } else {
 
-                        sh->locked_nodes[child_index] = 0;
-			gf_log (this->name, GF_LOG_DEBUG,
-				"locking of %s on child %d failed: %s",
-				local->loc.path, child_index,
-				strerror (op_errno));
-		} else {
-                        sh->locked_nodes[child_index] = 1;
-			gf_log (this->name, GF_LOG_TRACE,
-				"inode of %s on child %d locked",
-				local->loc.path, child_index);
-		}
-	}
-	UNLOCK (&frame->lock);
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Non Blocking inodelks done. Proceeding to FOP");
+                afr_sh_metadata_lookup (frame, this);
+        }
 
-	call_count = afr_frame_return (frame);
-
-	if (call_count == 0) {
-		if (sh->op_failed) {
-			afr_sh_metadata_finish (frame, this);
-			return 0;
-		}
-
-		afr_sh_metadata_lookup (frame, this);
-	}
-
-	return 0;
+        return 0;
 }
 
-
 int
 afr_sh_metadata_lock (call_frame_t *frame, xlator_t *this)
 {
-	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
-	afr_private_t   *priv = NULL;
-	int              i = 0;
-	int              call_count = 0;
-	struct flock     flock = {0, };
+        afr_internal_lock_t *int_lock = NULL;
+        afr_local_t         *local    = NULL;
 
+        local    = frame->local;
+        int_lock = &local->internal_lock;
 
-	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
+        int_lock->transaction_lk_type = AFR_SELFHEAL_LK;
+        int_lock->selfheal_lk_type    = AFR_METADATA_SELF_HEAL_LK;
 
-	call_count = afr_up_children_count (priv->child_count,
-                                            local->child_up);
-	local->call_count = call_count;
+        afr_set_lock_number (frame, this);
 
-	for (i = 0; i < priv->child_count; i++) {
-		flock.l_start   = 0;
-		flock.l_len     = 0;
-		flock.l_type    = F_WRLCK;
+        int_lock->lk_flock.l_start = 0;
+        int_lock->lk_flock.l_len   = 0;
+        int_lock->lk_flock.l_type  = F_WRLCK;
+        int_lock->lock_cbk         = afr_sh_post_nonblocking_inodelk_cbk;
 
-		if (local->child_up[i]) {
-			gf_log (this->name, GF_LOG_TRACE,
-				"locking %s on subvolume %s",
-				local->loc.path, priv->children[i]->name);
-
-			STACK_WIND_COOKIE (frame, afr_sh_metadata_lk_cbk,
-					   (void *) (long) i,
-					   priv->children[i],
-					   priv->children[i]->fops->inodelk,
-                                           this->name,
-					   &local->loc, F_SETLK, &flock);
-
-			if (!--call_count)
-				break;
-		}
-	}
+        afr_nonblocking_inodelk (frame, this);
 
 	return 0;
 }
@@ -804,12 +715,10 @@ int
 afr_self_heal_metadata (call_frame_t *frame, xlator_t *this)
 {
 	afr_local_t   *local = NULL;
-	afr_self_heal_t *sh = NULL;
 	afr_private_t *priv = this->private;
 
 
 	local = frame->local;
-	sh = &local->self_heal;
 
 	if (local->self_heal.need_metadata_self_heal && priv->metadata_self_heal) {
 		afr_sh_metadata_lock (frame, this);
