@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -32,21 +32,18 @@
 
 #include <sys/time.h>
 
-static int gf_log_subvol_full;
 
 int 
 dht_du_info_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int op_ret, int op_errno, struct statvfs *statvfs)
 {
 	dht_conf_t    *conf         = NULL;
-	dht_local_t   *local = NULL;
         call_frame_t  *prev          = NULL;
 	int            this_call_cnt = 0;
         int            i = 0;
         double         percent = 0;
         uint64_t       bytes = 0;
 
-	local = frame->local;
         conf = this->private;
         prev = cookie;
 
@@ -55,7 +52,7 @@ dht_du_info_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         if (statvfs && statvfs->f_blocks) {
                 percent = (statvfs->f_bfree * 100) / statvfs->f_blocks;
-                bytes = (statvfs->f_bfree * statvfs->f_bsize);
+                bytes = (statvfs->f_bfree * statvfs->f_frsize);
         }
         
         LOCK (&conf->subvolume_lock);
@@ -212,12 +209,11 @@ dht_is_subvol_filled (xlator_t *this, xlator_t *subvol)
 
         if (subvol_filled && conf->subvolume_status[i]) {
                 if (!(conf->du_stats[i].log++ % (GF_UNIVERSAL_ANSWER * 10))) {
-		  GF_LOG_OCCASIONALLY (gf_log_subvol_full,
-				       this->name, GF_LOG_WARNING,
-				       "disk space on subvolume '%s' is getting "
-				       "full (%.2f %%), consider adding more nodes", 
-				       subvol->name, 
-				       (100 - conf->du_stats[i].avail_percent));
+                        gf_log (this->name, GF_LOG_WARNING,
+                                "disk space on subvolume '%s' is getting "
+                                "full (%.2f %%), consider adding more nodes", 
+                                subvol->name, 
+                                (100 - conf->du_stats[i].avail_percent));
                 }
         }
 
@@ -233,7 +229,6 @@ dht_free_disk_available_subvol (xlator_t *this, xlator_t *subvol)
 	dht_conf_t *conf = NULL;
 
         conf = this->private;
-        avail_subvol = subvol;
 
         LOCK (&conf->subvolume_lock);
         {
@@ -253,13 +248,16 @@ dht_free_disk_available_subvol (xlator_t *this, xlator_t *subvol)
         }
         UNLOCK (&conf->subvolume_lock);
 
-        if (max < conf->min_free_disk)
-                avail_subvol = subvol;
-
-        if (avail_subvol == subvol) {
+        if (!avail_subvol) {
                 gf_log (this->name, GF_LOG_DEBUG,
                         "no subvolume has enough free space to create");
         }
-                
+
+        if (max < conf->min_free_disk)
+                avail_subvol = subvol;
+
+        if (!avail_subvol)
+                avail_subvol = subvol;                
+
         return avail_subvol;
 }
