@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -64,6 +64,8 @@ dht_itransform (xlator_t *this, xlator_t *subvol, uint64_t x, uint64_t *y_p)
 	}
 
 	conf = this->private;
+        if (!conf)
+                goto out;
 
 	max = conf->subvolume_cnt;
 	cnt = dht_subvol_cnt (this, subvol);
@@ -77,6 +79,65 @@ out:
 	return 0;
 }
 
+int
+dht_filter_loc_subvol_key (xlator_t *this, loc_t *loc, loc_t *new_loc,
+                           xlator_t **subvol)
+{
+        char          *new_name  = NULL;
+        char          *new_path  = NULL;
+        xlator_list_t *trav      = NULL;
+        char           key[1024] = {0,};
+        int            ret       = 0; /* not found */
+
+        /* Why do other tasks if first required 'char' itself is not there */
+        if (loc->name && !strchr (loc->name, '@'))
+                goto out;
+
+        trav = this->children;
+        while (trav) {
+                snprintf (key, 1024, "*@%s:%s", this->name, trav->xlator->name);
+                if (fnmatch (key, loc->name, FNM_NOESCAPE) == 0) {
+                        new_name = GF_CALLOC(strlen (loc->name),
+                                             sizeof (char),
+                                             gf_common_mt_char);
+                        if (!new_name)
+                                goto out;
+                        if (fnmatch (key, loc->path, FNM_NOESCAPE) == 0) {
+                                new_path = GF_CALLOC(strlen (loc->path),
+                                                     sizeof (char),
+                                                     gf_common_mt_char);
+                                if (!new_path)
+                                        goto out;
+                                strncpy (new_path, loc->path, (strlen (loc->path) -
+                                                               strlen (key) + 1));
+                        }
+                        strncpy (new_name, loc->name, (strlen (loc->name) -
+                                                       strlen (key) + 1));
+
+                        if (new_loc) {
+                                new_loc->path   = ((new_path) ? new_path:
+                                                   gf_strdup (loc->path));
+                                new_loc->name   = new_name;
+                                new_loc->ino    = loc->ino;
+                                new_loc->inode  = inode_ref (loc->inode);
+                                new_loc->parent = inode_ref (loc->parent);
+                        }
+                        *subvol         = trav->xlator;
+                        ret = 1;  /* success */
+                        goto out;
+                }
+                trav = trav->next;
+        }
+out:
+        if (!ret) {
+                /* !success */
+                if (new_path)
+                        GF_FREE (new_path);
+                if (new_name)
+                        GF_FREE (new_name);
+        }
+        return ret;
+}
 
 int
 dht_deitransform (xlator_t *this, uint64_t y, xlator_t **subvol_p,
@@ -88,6 +149,8 @@ dht_deitransform (xlator_t *this, uint64_t y, xlator_t **subvol_p,
 	uint64_t    x = 0;
 	xlator_t   *subvol = 0;
 
+        if (!this->private)
+                goto out;
 
 	conf = this->private;
 	max = conf->subvolume_cnt;
@@ -103,6 +166,7 @@ dht_deitransform (xlator_t *this, uint64_t y, xlator_t **subvol_p,
 	if (x_p)
 		*x_p = x;
 
+out:
 	return 0;
 }
 
@@ -140,6 +204,11 @@ dht_local_wipe (xlator_t *this, dht_local_t *local)
 		local->fd = NULL;
 	}
 
+        if (local->params) {
+                dict_unref (local->params);
+                local->params = NULL;
+        }
+
 	if (local->xattr_req)
 		dict_unref (local->xattr_req);
 
@@ -148,7 +217,15 @@ dht_local_wipe (xlator_t *this, dht_local_t *local)
                 local->selfheal.layout = NULL;
         }
 
-	FREE (local);
+        if (local->newpath) {
+                GF_FREE (local->newpath);
+        }
+
+        if (local->key) {
+               GF_FREE (local->key);
+        }
+           
+	GF_FREE (local);
 }
 
 
@@ -158,7 +235,8 @@ dht_local_init (call_frame_t *frame)
 	dht_local_t *local = NULL;
 
 	/* TODO: use mem-pool */
-	local = CALLOC (1, sizeof (*local));
+	local = GF_CALLOC (1, sizeof (*local),
+                           gf_dht_mt_dht_local_t);
 
 	if (!local)
 		return NULL;
@@ -193,6 +271,8 @@ dht_first_up_subvol (xlator_t *this)
 	int         i = 0;
 
 	conf = this->private;
+        if (!conf)
+                goto out;
 
 	LOCK (&conf->subvolume_lock);
 	{
@@ -205,9 +285,35 @@ dht_first_up_subvol (xlator_t *this)
 	}
 	UNLOCK (&conf->subvolume_lock);
 
+out:
 	return child;
 }
 
+xlator_t *
+dht_last_up_subvol (xlator_t *this)
+{
+        dht_conf_t *conf = NULL;
+        xlator_t   *child = NULL;
+        int         i = 0;
+
+        conf = this->private;
+        if (!conf)
+                goto out;
+
+        LOCK (&conf->subvolume_lock);
+        {
+                for (i = conf->subvolume_cnt-1; i >= 0; i--) {
+                        if (conf->subvolume_status[i]) {
+                                child = conf->subvolumes[i];
+                                break;
+                        }
+                }
+        }
+        UNLOCK (&conf->subvolume_lock);
+
+out:
+        return child;
+}
 
 xlator_t *
 dht_subvol_get_hashed (xlator_t *this, loc_t *loc)
@@ -279,6 +385,8 @@ dht_subvol_next (xlator_t *this, xlator_t *prev)
 	xlator_t   *next = NULL;
 
 	conf = this->private;
+        if (!conf)
+                goto out;
 
 	for (i = 0; i < conf->subvolume_cnt; i++) {
 		if (conf->subvolumes[i] == prev) {
@@ -288,6 +396,7 @@ dht_subvol_next (xlator_t *this, xlator_t *prev)
 		}
 	}
 
+out:
 	return next;
 }
 
@@ -299,8 +408,9 @@ dht_subvol_cnt (xlator_t *this, xlator_t *subvol)
 	int ret = -1;
 	dht_conf_t *conf = NULL;
 
-
 	conf = this->private;
+        if (!conf)
+                goto out;
 
 	for (i = 0; i < conf->subvolume_cnt; i++) {
 		if (subvol == conf->subvolumes[i]) {
@@ -309,6 +419,7 @@ dht_subvol_cnt (xlator_t *this, xlator_t *subvol)
 		}
 	}
 
+out:
 	return ret;
 }
 
@@ -319,34 +430,36 @@ dht_subvol_cnt (xlator_t *this, xlator_t *subvol)
 	} while (0)
 
 int
-dht_stat_merge (xlator_t *this, struct stat *to,
-		struct stat *from, xlator_t *subvol)
+dht_iatt_merge (xlator_t *this, struct iatt *to,
+		struct iatt *from, xlator_t *subvol)
 {
         if (!from || !to)
                 return 0;
 
-	to->st_dev      = from->st_dev;
+	to->ia_dev      = from->ia_dev;
 
-	dht_itransform (this, subvol, from->st_ino, &to->st_ino);
+        uuid_copy (to->ia_gfid, from->ia_gfid);
 
-	to->st_mode     = from->st_mode;
-	to->st_nlink    = from->st_nlink;
-	to->st_rdev     = from->st_rdev;
-	to->st_size    += from->st_size;
-	to->st_blksize  = from->st_blksize;
-	to->st_blocks  += from->st_blocks;
+	dht_itransform (this, subvol, from->ia_ino, &to->ia_ino);
 
-	set_if_greater (to->st_uid, from->st_uid);
-	set_if_greater (to->st_gid, from->st_gid);
+	to->ia_prot     = from->ia_prot;
+	to->ia_type     = from->ia_type;
+	to->ia_nlink    = from->ia_nlink;
+	to->ia_rdev     = from->ia_rdev;
+	to->ia_size    += from->ia_size;
+	to->ia_blksize  = from->ia_blksize;
+	to->ia_blocks  += from->ia_blocks;
 
-	set_if_greater (to->st_atime, from->st_atime);
-	set_if_greater (to->st_mtime, from->st_mtime);
-	set_if_greater (to->st_ctime, from->st_ctime);
+	set_if_greater (to->ia_uid, from->ia_uid);
+	set_if_greater (to->ia_gid, from->ia_gid);
+
+	set_if_greater (to->ia_atime, from->ia_atime);
+	set_if_greater (to->ia_mtime, from->ia_mtime);
+	set_if_greater (to->ia_ctime, from->ia_ctime);
 
 	return 0;
 }
 
-
 int
 dht_build_child_loc (xlator_t *this, loc_t *child, loc_t *parent, char *name)
 {
@@ -355,9 +468,9 @@ dht_build_child_loc (xlator_t *this, loc_t *child, loc_t *parent, char *name)
         }
 
         if (strcmp (parent->path, "/") == 0)
-                asprintf ((char **)&child->path, "/%s", name);
+                gf_asprintf ((char **)&child->path, "/%s", name);
         else
-                asprintf ((char **)&child->path, "%s/%s", parent->path, name);
+                gf_asprintf ((char **)&child->path, "%s/%s", parent->path, name);
 
         if (!child->path) {
                 gf_log (this->name, GF_LOG_ERROR,
