@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -24,6 +24,7 @@
 #endif
 
 #include "dht-common.c"
+#include "dht-mem-types.h"
 
 #include <sys/time.h>
 #include <stdlib.h>
@@ -82,7 +83,7 @@ get_switch_matching_subvol (const char *path, dht_conf_t *conf,
                 return hashed_subvol;
 
         trav = cond;
-        pathname = strdup (path);
+        pathname = gf_strdup (path);
 	while (trav) {
 		if (fnmatch (trav->path_pattern,
 			     pathname, FNM_NOESCAPE) == 0) {
@@ -96,7 +97,7 @@ get_switch_matching_subvol (const char *path, dht_conf_t *conf,
 		}
 		trav = trav->next;
 	}
-	free (pathname);
+	GF_FREE (pathname);
 	return hashed_subvol;
 }
 
@@ -104,8 +105,8 @@ get_switch_matching_subvol (const char *path, dht_conf_t *conf,
 int
 switch_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                          int op_ret, int op_errno,
-                         inode_t *inode, struct stat *stbuf, dict_t *xattr,
-                         struct stat *postparent)
+                         inode_t *inode, struct iatt *stbuf, dict_t *xattr,
+                         struct iatt *postparent)
 {
         xlator_t     *subvol      = NULL;
         char          is_linkfile = 0;
@@ -141,8 +142,8 @@ switch_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!is_dir && !is_linkfile) {
                 /* non-directory and not a linkfile */
 
-		dht_itransform (this, prev->this, stbuf->st_ino,
-				&stbuf->st_ino);
+		dht_itransform (this, prev->this, stbuf->ia_ino,
+				&stbuf->ia_ino);
 
 		ret = dht_layout_preset (this, prev->this, inode);
 		if (ret < 0) {
@@ -207,8 +208,9 @@ out:
 		gf_log (this->name, GF_LOG_DEBUG,
 			"no subvolume in layout for path=%s",
 			local->loc.path);
-		op_errno = EINVAL;
-		goto err;
+                local->op_errno = ENOENT;
+                dht_lookup_everywhere (frame, this, loc);
+                return 0;
 	}
 
 	STACK_WIND (frame, dht_lookup_cbk,
@@ -296,7 +298,7 @@ switch_lookup (call_frame_t *frame, xlator_t *this,
 		}
 
 		local->inode    = inode_ref (loc->inode);
-		local->st_ino   = loc->inode->ino;
+		local->ia_ino   = loc->inode->ino;
 
 		local->call_cnt = layout->cnt;
 		call_cnt = local->call_cnt;
@@ -306,6 +308,10 @@ switch_lookup (call_frame_t *frame, xlator_t *this,
 		 */
 		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht", 4 * 4);
+                if (ret < 0)
+                        gf_log (this->name, GF_LOG_WARNING,
+                                "failed to set dict value for "
+                                "trusted.glusterfs.dht");
 
 		for (i = 0; i < layout->cnt; i++) {
 			subvol = layout->list[i].xlator;
@@ -321,9 +327,17 @@ switch_lookup (call_frame_t *frame, xlator_t *this,
         do_fresh_lookup:
 		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht", 4 * 4);
-
+                if (ret < 0)
+                        gf_log (this->name, GF_LOG_WARNING,
+                                "failed to set dict value for "
+                                "trusted.glusterfs.dht");
+                
 		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht.linkto", 256);
+                if (ret < 0)
+                        gf_log (this->name, GF_LOG_WARNING, 
+                                "failed to set dict value for " 
+                                "trusted.glusterfs.dht.linkto");
 
                 if (!hashed_subvol) {
 			gf_log (this->name, GF_LOG_DEBUG,
@@ -378,24 +392,21 @@ err:
 int
 switch_create_linkfile_create_cbk (call_frame_t *frame, void *cookie,
 				 xlator_t *this, int op_ret, int op_errno,
-                                 inode_t *inode, struct stat *stbuf,
-                                 struct stat *preparent,
-                                 struct stat *postparent)
+                                 inode_t *inode, struct iatt *stbuf,
+                                 struct iatt *preparent,
+                                 struct iatt *postparent)
 {
 	dht_local_t  *local = NULL;
-	call_frame_t *prev = NULL;
-	dht_conf_t   *conf  = NULL;
 
 	local = frame->local;
-	prev  = cookie;
-	conf  = this->private;
 
 	if (op_ret == -1)
 		goto err;
 
 	STACK_WIND (frame, dht_create_cbk,
 		    local->cached_subvol, local->cached_subvol->fops->create,
-		    &local->loc, local->flags, local->mode, local->fd);
+		    &local->loc, local->flags, local->mode, local->fd,
+                    local->params);
 
 	return 0;
 
@@ -407,7 +418,8 @@ switch_create_linkfile_create_cbk (call_frame_t *frame, void *cookie,
 
 int
 switch_create (call_frame_t *frame, xlator_t *this,
-	     loc_t *loc, int32_t flags, mode_t mode, fd_t *fd)
+               loc_t *loc, int32_t flags, mode_t mode,
+               fd_t *fd, dict_t *params)
 {
 	dht_local_t *local = NULL;
 	dht_conf_t  *conf  = NULL;
@@ -473,7 +485,7 @@ switch_create (call_frame_t *frame, xlator_t *this,
 
         STACK_WIND (frame, dht_create_cbk,
                     subvol, subvol->fops->create,
-                    loc, flags, mode, fd);
+                    loc, flags, mode, fd, params);
 
         return 0;
 
@@ -488,22 +500,19 @@ err:
 int
 switch_mknod_linkfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                          int op_ret, int op_errno, inode_t *inode,
-                         struct stat *stbuf, struct stat *preparent,
-                         struct stat *postparent)
+                         struct iatt *stbuf, struct iatt *preparent,
+                         struct iatt *postparent)
 {
 	dht_local_t  *local = NULL;
-	call_frame_t *prev = NULL;
-	dht_conf_t   *conf  = NULL;
 
 	local = frame->local;
-	prev  = cookie;
-	conf  = this->private;
 
 	if (op_ret >= 0) {
 		STACK_WIND (frame, dht_newfile_cbk,
 			    local->cached_subvol,
 			    local->cached_subvol->fops->mknod,
-			    &local->loc, local->mode, local->rdev);
+			    &local->loc, local->mode, local->rdev,
+                            local->params);
 
 		return 0;
 	}
@@ -516,7 +525,7 @@ switch_mknod_linkfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 switch_mknod (call_frame_t *frame, xlator_t *this,
-	    loc_t *loc, mode_t mode, dev_t rdev)
+              loc_t *loc, mode_t mode, dev_t rdev, dict_t *params)
 {
 	dht_local_t *local = NULL;
 	dht_conf_t  *conf  = NULL;
@@ -567,6 +576,7 @@ switch_mknod (call_frame_t *frame, xlator_t *this,
 			goto err;
 		}
 
+                local->params = dict_ref (params);
 		local->mode = mode;
 		local->rdev = rdev;
 		local->cached_subvol = avail_subvol;
@@ -581,7 +591,7 @@ switch_mknod (call_frame_t *frame, xlator_t *this,
 
 	STACK_WIND (frame, dht_newfile_cbk,
 		    subvol, subvol->fops->mknod,
-		    loc, mode, rdev);
+		    loc, mode, rdev, params);
 
 	return 0;
 
@@ -619,29 +629,29 @@ fini (xlator_t *this)
                 conf->private = NULL;
                 while (trav) {
                         if (trav->array)
-                                FREE (trav->array);
+                                GF_FREE (trav->array);
                         prev = trav;
                         trav = trav->next;
-                        FREE (prev);
+                        GF_FREE (prev);
                 }
 
                 if (conf->file_layouts) {
                         for (i = 0; i < conf->subvolume_cnt; i++) {
-                                FREE (conf->file_layouts[i]);
+                                GF_FREE (conf->file_layouts[i]);
                         }
-                        FREE (conf->file_layouts);
+                        GF_FREE (conf->file_layouts);
                 }
 
                 if (conf->default_dir_layout)
-                        FREE (conf->default_dir_layout);
+                        GF_FREE (conf->default_dir_layout);
 
                 if (conf->subvolumes)
-                        FREE (conf->subvolumes);
+                        GF_FREE (conf->subvolumes);
 
 		if (conf->subvolume_status)
-			FREE (conf->subvolume_status);
+			GF_FREE (conf->subvolume_status);
 
-                FREE (conf);
+                GF_FREE (conf);
         }
 
 	return;
@@ -678,8 +688,9 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
 		trav_xl = trav_xl->next;
 	}
 	child_count = index;
-	switch_buf_array = CALLOC ((index + 1),
-                                   sizeof (struct switch_sched_array));
+	switch_buf_array = GF_CALLOC ((index + 1),
+                                      sizeof (struct switch_sched_array),
+                                      gf_switch_mt_switch_sched_array);
         if (!switch_buf_array)
                 goto err;
 
@@ -697,11 +708,12 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
 
         /* Get the pattern for considering switch case.
            "option block-size *avi:10MB" etc */
-        option_string = strdup (pattern_str);
+        option_string = gf_strdup (pattern_str);
         switch_str = strtok_r (option_string, ";", &tmp_str);
         while (switch_str) {
-                dup_str = strdup (switch_str);
-                switch_opt = CALLOC (1, sizeof (struct switch_struct));
+                dup_str = gf_strdup (switch_str);
+                switch_opt = GF_CALLOC (1, sizeof (struct switch_struct),
+                                        gf_switch_mt_switch_struct);
                 if (!switch_opt)
                         goto err;
 
@@ -713,12 +725,12 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
                                 "for all the unconfigured child nodes,"
                                 " hence neglecting current option");
                         switch_str = strtok_r (NULL, ";", &tmp_str);
-                        free (dup_str);
+                        GF_FREE (dup_str);
                         continue;
                 }
                 memcpy (switch_opt->path_pattern, pattern, strlen (pattern));
                 if (childs) {
-                        dup_childs = strdup (childs);
+                        dup_childs = gf_strdup (childs);
                         child = strtok_r (dup_childs, ",", &tmp);
                         while (child) {
                                 if (gf_switch_valid_child (this, child)) {
@@ -733,11 +745,12 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
                                         goto err;
                                 }
                         }
-                        free (dup_childs);
+                        GF_FREE (dup_childs);
                         child = strtok_r (childs, ",", &tmp1);
                         switch_opt->num_child = idx;
-                        switch_opt->array = CALLOC (1, (idx *
-                                                        sizeof (struct switch_sched_array)));
+                        switch_opt->array = GF_CALLOC (1, (idx *
+                                                       sizeof (struct switch_sched_array)),
+                                                       gf_switch_mt_switch_sched_array);
                         if (!switch_opt->array)
                                 goto err;
                         idx = 0;
@@ -771,7 +784,7 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
                                 "option in unify volume. Exiting");
                         goto err;
                 }
-                free (dup_str);
+                GF_FREE (dup_str);
                 
                 /* Link it to the main structure */
                 if (switch_buf) {
@@ -802,7 +815,8 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
 				"No nodes left for pattern '*'. Exiting");
 			goto err;
 		}
-		switch_opt = CALLOC (1, sizeof (struct switch_struct));
+		switch_opt = GF_CALLOC (1, sizeof (struct switch_struct),
+                                        gf_switch_mt_switch_struct);
                 if (!switch_opt)
                         goto err;
 
@@ -810,7 +824,9 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
 		memcpy (switch_opt->path_pattern, "*", 2);
 		switch_opt->num_child = flag;
 		switch_opt->array =
-			CALLOC (1, flag * sizeof (struct switch_sched_array));
+			GF_CALLOC (1, 
+                                   flag * sizeof (struct switch_sched_array),
+                                   gf_switch_mt_switch_sched_array);
                 if (!switch_opt->array)
                         goto err;
 		flag = 0;
@@ -845,14 +861,14 @@ set_switch_pattern (xlator_t *this, dht_conf_t *conf,
 err:
         if (switch_buf) {
                 if (switch_buf_array)
-                        FREE (switch_buf_array);
+                        GF_FREE (switch_buf_array);
                 trav = switch_buf;
                 while (trav) {
                         if (trav->array)
-                                FREE (trav->array);
+                                GF_FREE (trav->array);
                         switch_opt = trav;
                         trav = trav->next;
-                        FREE (switch_opt);
+                        GF_FREE (switch_opt);
                 }
         }
         return -1;
@@ -880,18 +896,20 @@ init (xlator_t *this)
 			"dangling volume. check volfile");
 	}
 
-        conf = CALLOC (1, sizeof (*conf));
+        conf = GF_CALLOC (1, sizeof (*conf), gf_switch_mt_dht_conf_t);
         if (!conf) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory");
                 goto err;
         }
 
-	conf->search_unhashed =  GF_DHT_LOOKUP_UNHASHED_AUTO;
+        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_ON;
 	if (dict_get_str (this->options, "lookup-unhashed", &temp_str) == 0) {
                 /* If option is not "auto", other options _should_ be boolean */
                 if (strcasecmp (temp_str, "auto"))
                         gf_string2boolean (temp_str, &conf->search_unhashed);
+                else
+                        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_AUTO;
 	}
 
 	conf->unhashed_sticky_bit = 0;
@@ -946,7 +964,8 @@ init (xlator_t *this)
 
 	conf->gen = 1;
 
-        conf->du_stats = CALLOC (conf->subvolume_cnt, sizeof (dht_du_t));
+        conf->du_stats = GF_CALLOC (conf->subvolume_cnt, sizeof (dht_du_t),
+                                    gf_switch_mt_dht_du_t);
         if (!conf->du_stats) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory");
@@ -961,24 +980,24 @@ err:
         if (conf) {
                 if (conf->file_layouts) {
                         for (i = 0; i < conf->subvolume_cnt; i++) {
-                                FREE (conf->file_layouts[i]);
+                                GF_FREE (conf->file_layouts[i]);
                         }
-                        FREE (conf->file_layouts);
+                        GF_FREE (conf->file_layouts);
                 }
 
                 if (conf->default_dir_layout)
-                        FREE (conf->default_dir_layout);
+                        GF_FREE (conf->default_dir_layout);
 
                 if (conf->subvolumes)
-                        FREE (conf->subvolumes);
+                        GF_FREE (conf->subvolumes);
 
 		if (conf->subvolume_status)
-			FREE (conf->subvolume_status);
+			GF_FREE (conf->subvolume_status);
 
                 if (conf->du_stats)
-                        FREE (conf->du_stats);
+                        GF_FREE (conf->du_stats);
 
-                FREE (conf);
+                GF_FREE (conf);
         }
 
         return -1;
@@ -1023,15 +1042,6 @@ struct xlator_fops fops = {
 	.xattrop     = dht_xattrop,
 	.fxattrop    = dht_fxattrop,
         .setattr     = dht_setattr,
-#if 0
-	.setdents    = dht_setdents,
-	.getdents    = dht_getdents,
-	.checksum    = dht_checksum,
-#endif
-};
-
-
-struct xlator_mops mops = {
 };
 
 
