@@ -3,16 +3,16 @@
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -35,17 +35,18 @@
 
 static pl_entry_lock_t *
 new_entrylk_lock (pl_inode_t *pinode, const char *basename, entrylk_type type,
-		  transport_t *trans, pid_t client_pid, uint64_t owner, const char *volume)
+		  void *trans, pid_t client_pid, uint64_t owner, const char *volume)
 
 {
 	pl_entry_lock_t *newlock = NULL;
 
-	newlock = CALLOC (1, sizeof (pl_entry_lock_t));
+	newlock = GF_CALLOC (1, sizeof (pl_entry_lock_t),
+                             gf_locks_mt_pl_entry_lock_t);
 	if (!newlock) {
 		goto out;
 	}
 
-	newlock->basename       = basename ? strdup (basename) : NULL;
+	newlock->basename       = basename ? gf_strdup (basename) : NULL;
 	newlock->type           = type;
 	newlock->trans          = trans;
 	newlock->volume         = volume;
@@ -317,7 +318,7 @@ __lock_name (pl_inode_t *pinode, const char *basename, entrylk_type type,
 {
 	pl_entry_lock_t *lock       = NULL;
 	pl_entry_lock_t *conf       = NULL;
-	transport_t     *trans      = NULL;
+	void            *trans      = NULL;
         pid_t            client_pid = 0;
         uint64_t         owner      = 0;
 
@@ -333,14 +334,20 @@ __lock_name (pl_inode_t *pinode, const char *basename, entrylk_type type,
 		goto out;
 	}
 
+        lock->frame   = frame;
+        lock->this    = this;
+        lock->trans   = trans;
+
 	conf = __lock_grantable (dom, basename, type);
 	if (conf) {
 		ret = -EAGAIN;
-		if (nonblock)
-			goto out;
+		if (nonblock){
+			if (lock->basename)
+				GF_FREE ((char *)lock->basename);
+                        GF_FREE (lock);
+                        goto out;
 
-		lock->frame   = frame;
-		lock->this    = this;
+                }
 
 		list_add_tail (&lock->blocked_locks, &dom->blocked_entrylks);
 
@@ -353,8 +360,13 @@ __lock_name (pl_inode_t *pinode, const char *basename, entrylk_type type,
 
         if ( __blocked_lock_conflict (dom, basename, type) && !(__owner_has_lock (dom, lock))) {
                 ret = -EAGAIN;
-                if (nonblock)
+                if (nonblock) {
+			if (lock->basename)
+				GF_FREE ((char *) lock->basename);
+                        GF_FREE (lock);
                         goto out;
+
+                }
                 lock->frame     = frame;
                 lock->this      = this;
 
@@ -366,12 +378,13 @@ __lock_name (pl_inode_t *pinode, const char *basename, entrylk_type type,
 			"Blocking lock: {pinode=%p, basename=%s}",
 			pinode, basename);
 
+                ret = -EAGAIN;
 		goto out;
         }
         switch (type) {
 
 	case ENTRYLK_WRLCK:
-		list_add (&lock->domain_list, &dom->entrylk_list);
+		list_add_tail (&lock->domain_list, &dom->entrylk_list);
 		break;
 
         default:
@@ -413,7 +426,7 @@ __unlock_name (pl_dom_list_t *dom, const char *basename, entrylk_type type)
 	    && lock->type == type) {
 
 		if (type == ENTRYLK_WRLCK) {
-			list_del (&lock->domain_list);
+			list_del_init (&lock->domain_list);
 			ret_lock = lock;
 		}
 	} else {
@@ -456,9 +469,11 @@ __grant_blocked_entry_locks (xlator_t *this, pl_inode_t *pl_inode,
 		if (bl_ret == 0) {
 			list_add (&bl->blocked_locks, granted);
 		} else {
+                        gf_log (this->name, GF_LOG_DEBUG,
+                                "should never happen");
 			if (bl->basename)
-				FREE (bl->basename);
-			FREE (bl);
+				GF_FREE ((char *)bl->basename);
+			GF_FREE (bl);
 		}
 	}
 	return;
@@ -490,12 +505,10 @@ grant_blocked_entry_locks (xlator_t *this, pl_inode_t *pl_inode,
 
 		STACK_UNWIND_STRICT (entrylk, lock->frame, 0, 0);
 
-		FREE (lock->basename);
-		FREE (lock);
 	}
 
-	FREE (unlocked->basename);
-	FREE (unlocked);
+	GF_FREE ((char *)unlocked->basename);
+	GF_FREE (unlocked);
 
 	return;
 }
@@ -507,7 +520,7 @@ grant_blocked_entry_locks (xlator_t *this, pl_inode_t *pl_inode,
 
 static int
 release_entry_locks_for_transport (xlator_t *this, pl_inode_t *pinode,
-				   pl_dom_list_t *dom, transport_t *trans)
+				   pl_dom_list_t *dom, void *trans)
 {
 	pl_entry_lock_t  *lock = NULL;
 	pl_entry_lock_t  *tmp = NULL;
@@ -545,8 +558,8 @@ release_entry_locks_for_transport (xlator_t *this, pl_inode_t *pinode,
                                 "releasing lock on  held by "
                                 "{transport=%p}",trans);
 
-			FREE (lock->basename);
-			FREE (lock);
+			GF_FREE ((char *)lock->basename);
+			GF_FREE (lock);
 		}
 
 		__grant_blocked_entry_locks (this, pinode, dom, &granted);
@@ -561,8 +574,8 @@ release_entry_locks_for_transport (xlator_t *this, pl_inode_t *pinode,
                 STACK_UNWIND_STRICT (entrylk, lock->frame, -1, EAGAIN);
 
                 if (lock->basename)
-                        FREE (lock->basename);
-                FREE (lock);
+                        GF_FREE ((char *)lock->basename);
+                GF_FREE (lock);
 
         }
 
@@ -572,8 +585,8 @@ release_entry_locks_for_transport (xlator_t *this, pl_inode_t *pinode,
 		STACK_UNWIND_STRICT (entrylk, lock->frame, 0, 0);
 
 		if (lock->basename)
-			FREE (lock->basename);
-		FREE (lock);
+			GF_FREE ((char *)lock->basename);
+		GF_FREE (lock);
 	}
 
 	return 0;
@@ -585,12 +598,12 @@ pl_common_entrylk (call_frame_t *frame, xlator_t *this,
                    const char *volume, inode_t *inode, const char *basename,
                    entrylk_cmd cmd, entrylk_type type, loc_t *loc, fd_t *fd)
 {
-	int32_t op_ret   = -1;
-	int32_t op_errno = 0;
+        uint64_t owner    = 0;
+	int32_t  op_ret   = -1;
+	int32_t  op_errno = 0;
 
-	transport_t * transport = NULL;
+	void *        transport = NULL;
 	pid_t         pid       = -1;
-        uint64_t      owner     = -1;
 
 	pl_inode_t *     pinode   = NULL;
 	int              ret      = -1;
@@ -618,10 +631,10 @@ pl_common_entrylk (call_frame_t *frame, xlator_t *this,
         entrylk_trace_in (this, frame, volume, fd, loc, basename, cmd, type);
 
 	pid       = frame->root->pid;
-        owner     = (uint64_t)(long) frame->root;
+        owner     = frame->root->lk_owner;
 	transport = frame->root->trans;
 
-	if (pid == 0) {
+	if (owner == 0) {
 		/*
                   this is a special case that means release
                   all locks from this transport
@@ -645,12 +658,19 @@ pl_common_entrylk (call_frame_t *frame, xlator_t *this,
 		}
 		pthread_mutex_unlock (&pinode->mutex);
 
+                op_errno = -ret;
 		if (ret < 0) {
-			if (ret == -EAGAIN)
-				unwind = 0;
-			op_errno = -ret;
+                        if (ret == -EAGAIN)
+                                unwind = 0;
+                        else
+                                unwind = 1;
 			goto out;
-		}
+		} else {
+                        op_ret = 0;
+                        op_errno = 0;
+                        unwind = 1;
+                        goto out;
+                }
 
 		break;
 
@@ -662,11 +682,11 @@ pl_common_entrylk (call_frame_t *frame, xlator_t *this,
 		}
 		pthread_mutex_unlock (&pinode->mutex);
 
-		if (ret < 0) {
+		if (ret < 0)
 			op_errno = -ret;
-			goto out;
-		}
 
+                unwind = 1;
+                goto out;
 		break;
 
 	case ENTRYLK_UNLOCK:
