@@ -3,16 +3,16 @@
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -42,7 +42,7 @@ __delete_inode_lock (pl_inode_lock_t *lock)
 void
 __destroy_inode_lock (pl_inode_lock_t *lock)
 {
-	FREE (lock);
+	GF_FREE (lock);
 }
 
 /* Check if 2 inodelks are conflicting on type. Only 2 shared locks don't conflict */
@@ -56,7 +56,7 @@ inodelk_type_conflict (pl_inode_lock_t *l1, pl_inode_lock_t *l2)
 }
 
 void
-pl_print_inodelk (char *str, int size, int cmd, struct flock *flock, const char *domain)
+pl_print_inodelk (char *str, int size, int cmd, struct gf_flock *flock, const char *domain)
 {
         char *cmd_str = NULL;
         char *type_str = NULL;
@@ -130,13 +130,8 @@ static int same_inodelk_owner (pl_inode_lock_t *l1, pl_inode_lock_t *l2)
 static int
 inodelk_conflict (pl_inode_lock_t *l1, pl_inode_lock_t *l2)
 {
-	if (same_inodelk_owner (l1, l2))
-		return 0;
-
-	if (!inodelk_overlap (l1, l2))
-		return 0;
-
-	return (inodelk_type_conflict(l1, l2));
+        return (inodelk_overlap (l1, l2) &&
+                inodelk_type_conflict (l1, l2));
 }
 
 /* Determine if lock is grantable or not */
@@ -271,7 +266,8 @@ find_matching_inodelk (pl_inode_lock_t *lock, pl_dom_list_t *dom)
 {
 	pl_inode_lock_t *l = NULL;
 	list_for_each_entry (l, &dom->inodelk_list, list) {
-		if (inodelks_equal (l, lock))
+		if (inodelks_equal (l, lock) &&
+                    same_inodelk_owner (l, lock))
 			return l;
 	}
 	return NULL;
@@ -370,7 +366,7 @@ grant_blocked_inode_locks (xlator_t *this, pl_inode_t *pl_inode, pl_dom_list_t *
 /* Release all inodelks from this transport */
 static int
 release_inode_locks_of_transport (xlator_t *this, pl_dom_list_t *dom,
-                                  inode_t *inode, transport_t *trans)
+                                  inode_t *inode, void *trans)
 {
 	pl_inode_lock_t *tmp = NULL;
 	pl_inode_lock_t *l = NULL;
@@ -439,7 +435,7 @@ release_inode_locks_of_transport (xlator_t *this, pl_dom_list_t *dom,
         }
 unlock:
         if (path)
-                FREE (path);
+                GF_FREE (path);
 
         pthread_mutex_unlock (&pinode->mutex);
 
@@ -447,7 +443,7 @@ unlock:
                 list_del_init (&l->blocked_locks);
 
                 STACK_UNWIND_STRICT (inodelk, l->frame, -1, EAGAIN);
-                FREE (l);
+                GF_FREE (l);
         }
 
 	grant_blocked_inode_locks (this, pinode, dom);
@@ -509,13 +505,14 @@ out:
 
 /* Create a new inode_lock_t */
 pl_inode_lock_t *
-new_inode_lock (struct flock *flock, transport_t *transport, pid_t client_pid,
+new_inode_lock (struct gf_flock *flock, void *transport, pid_t client_pid,
                 uint64_t owner, const char *volume)
 
 {
 	pl_inode_lock_t *lock = NULL;
 
-	lock = CALLOC (1, sizeof (*lock));
+	lock = GF_CALLOC (1, sizeof (*lock),
+                          gf_locks_mt_pl_inode_lock_t);
 	if (!lock) {
 		return NULL;
 	}
@@ -543,13 +540,13 @@ new_inode_lock (struct flock *flock, transport_t *transport, pid_t client_pid,
 int
 pl_common_inodelk (call_frame_t *frame, xlator_t *this,
                    const char *volume, inode_t *inode, int32_t cmd,
-                   struct flock *flock, loc_t *loc, fd_t *fd)
+                   struct gf_flock *flock, loc_t *loc, fd_t *fd)
 {
 	int32_t op_ret   = -1;
 	int32_t op_errno = 0;
 	int     ret      = -1;
 	int     can_block = 0;
-	transport_t *           transport  = NULL;
+	void *                  transport  = NULL;
 	pid_t                   client_pid = -1;
         uint64_t                owner      = -1;
 	pl_inode_t *            pinode     = NULL;
@@ -581,7 +578,7 @@ pl_common_inodelk (call_frame_t *frame, xlator_t *this,
 
 	dom = get_domain (pinode, volume);
 
-	if (client_pid == 0) {
+	if (owner == 0) {
 		/*
                   special case: this means release all locks
                   from this transport
@@ -603,6 +600,9 @@ pl_common_inodelk (call_frame_t *frame, xlator_t *this,
 		goto unwind;
 	}
 
+        reqlock->frame = frame;
+        reqlock->this  = this;
+
 	switch (cmd) {
 	case F_SETLKW:
 		can_block = 1;
@@ -612,7 +612,7 @@ pl_common_inodelk (call_frame_t *frame, xlator_t *this,
 		/* fall through */
 
 	case F_SETLK:
-		memcpy (&reqlock->user_flock, flock, sizeof (struct flock));
+		memcpy (&reqlock->user_flock, flock, sizeof (struct gf_flock));
 		ret = pl_inode_setlk (this, pinode, reqlock,
                                       can_block, dom);
 
@@ -653,7 +653,7 @@ out:
 
 int
 pl_inodelk (call_frame_t *frame, xlator_t *this,
-            const char *volume, loc_t *loc, int32_t cmd, struct flock *flock)
+            const char *volume, loc_t *loc, int32_t cmd, struct gf_flock *flock)
 {
 
         pl_common_inodelk (frame, this, volume, loc->inode, cmd, flock, loc, NULL);
@@ -663,7 +663,7 @@ pl_inodelk (call_frame_t *frame, xlator_t *this,
 
 int
 pl_finodelk (call_frame_t *frame, xlator_t *this,
-             const char *volume, fd_t *fd, int32_t cmd, struct flock *flock)
+             const char *volume, fd_t *fd, int32_t cmd, struct gf_flock *flock)
 {
 
         pl_common_inodelk (frame, this, volume, fd->inode, cmd, flock, NULL, fd);
