@@ -3,16 +3,16 @@
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -26,9 +26,9 @@
 #endif
 
 #include "compat-errno.h"
-#include "transport.h"
 #include "stack.h"
 #include "call-stub.h"
+#include "locks-mem-types.h"
 
 struct __pl_fd;
 
@@ -40,16 +40,17 @@ struct __posix_lock {
         off_t              fl_end;
 
         short              blocked;    /* waiting to acquire */
-        struct flock       user_flock; /* the flock supplied by the user */
+        struct gf_flock       user_flock; /* the flock supplied by the user */
         xlator_t          *this;       /* required for blocked locks */
         unsigned long      fd_num;
 
+        fd_t              *fd;
         call_frame_t      *frame;
 
         /* These two together serve to uniquely identify each process
            across nodes */
 
-        transport_t       *transport;     /* to identify client node */
+        void              *transport;     /* to identify client node */
         pid_t              client_pid;    /* pid of client process */
         uint64_t           owner;         /* lock owner from fuse */
 };
@@ -65,7 +66,7 @@ struct __pl_inode_lock {
 
         const char        *volume;
 
-        struct flock       user_flock; /* the flock supplied by the user */
+        struct gf_flock       user_flock; /* the flock supplied by the user */
         xlator_t          *this;       /* required for blocked locks */
         fd_t              *fd;
 
@@ -74,7 +75,7 @@ struct __pl_inode_lock {
         /* These two together serve to uniquely identify each process
            across nodes */
 
-        transport_t       *transport;     /* to identify client node */
+        void              *transport;     /* to identify client node */
         pid_t              client_pid;    /* pid of client process */
         uint64_t           owner;
 };
@@ -109,7 +110,7 @@ struct __entry_lock {
         const char       *basename;
         entrylk_type      type;
 
-        transport_t      *trans;
+        void      *trans;
         pid_t             client_pid;    /* pid of client process */
         uint64_t          owner;
 };
@@ -125,6 +126,9 @@ struct __pl_inode {
         struct list_head dom_list;       /* list of domains */
         struct list_head ext_list;       /* list of fcntl locks */
         struct list_head rw_list;        /* list of waiting r/w requests */
+        struct list_head reservelk_list;        /* list of reservelks */
+        struct list_head blocked_reservelks;        /* list of blocked reservelks */
+        struct list_head blocked_calls;  /* List of blocked lock calls while a reserve is held*/
         int              mandatory;      /* if mandatory locking is enabled */
 
         inode_t          *refkeeper;     /* hold refs on an inode while locks are
@@ -150,4 +154,8 @@ typedef struct {
         gf_boolean_t   posixlk_count_req;
 } pl_local_t;
 
+typedef struct {
+        struct list_head locks_list;
+} pl_fdctx_t;
+
 #endif /* __POSIX_LOCKS_H__ */
