@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -23,6 +23,7 @@
 #endif
 
 #include "io-cache.h"
+#include "ioc-mem-types.h"
 
 extern int ioc_log2_page_size;
 
@@ -51,7 +52,7 @@ ptr_to_str (void *ptr)
 {
         int   ret = 0;
 	char *str = NULL;
-	ret = asprintf (&str, "%p", ptr);
+	ret = gf_asprintf (&str, "%p", ptr);
         if (-1 == ret) {
                 gf_log ("ioc", GF_LOG_ERROR, 
                         "asprintf failed while converting ptr to str");
@@ -62,7 +63,7 @@ ptr_to_str (void *ptr)
 
 void
 ioc_inode_wakeup (call_frame_t *frame, ioc_inode_t *ioc_inode, 
-                  struct stat *stbuf)
+                  struct iatt *stbuf)
 {
 	ioc_waitq_t *waiter = NULL, *waited = NULL;
 	ioc_waitq_t *page_waitq = NULL;
@@ -137,7 +138,7 @@ ioc_inode_wakeup (call_frame_t *frame, ioc_inode_t *ioc_inode,
 		waiter = waiter->next;
     
 		waited->data = NULL;
-		free (waited);
+		GF_FREE (waited);
 	}
 }
 
@@ -155,18 +156,14 @@ ioc_inode_t *
 ioc_inode_update (ioc_table_t *table, inode_t *inode, uint32_t weight)
 {
 	ioc_inode_t     *ioc_inode   = NULL;
-        unsigned long    no_of_pages = 0;
 
-        ioc_inode = CALLOC (1, sizeof (ioc_inode_t));
+        ioc_inode = GF_CALLOC (1, sizeof (ioc_inode_t),
+                               gf_ioc_mt_ioc_inode_t);
         if (ioc_inode == NULL) {
                 goto out;
         }
   
 	ioc_inode->table = table;
-
-        no_of_pages = (table->cache_size / table->page_size)
-                + ((table->cache_size % table->page_size) ? 1 : 0);
-
 	INIT_LIST_HEAD (&ioc_inode->cache.page_lru);
 
 	ioc_table_lock (table);
@@ -213,5 +210,5 @@ ioc_inode_destroy (ioc_inode_t *ioc_inode)
         rbthash_table_destroy (ioc_inode->cache.page_table);
 
 	pthread_mutex_destroy (&ioc_inode->inode_lock);
-	free (ioc_inode);
+	GF_FREE (ioc_inode);
 }
