@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -27,6 +27,7 @@
 #include "dict.h"
 #include "xlator.h"
 #include "io-cache.h"
+#include "ioc-mem-types.h"
 #include <assert.h>
 #include <sys/time.h>
 
@@ -86,7 +87,7 @@ ioc_page_destroy (ioc_page_t *page)
     
 		if (page->vector){
 			iobref_unref (page->iobref);
-			free (page->vector);
+			GF_FREE (page->vector);
 			page->vector = NULL;
 		}
     
@@ -95,7 +96,7 @@ ioc_page_destroy (ioc_page_t *page)
 
 	if (page_size != -1) {
 		pthread_mutex_destroy (&page->page_lock);
-		free (page);
+		GF_FREE (page);
 	}
 
 	return page_size;
@@ -177,9 +178,9 @@ ioc_prune (ioc_table_t *table)
 }
 
 /*
- * ioc_page_create - create a new page. 
+ * ioc_page_create - create a new page.
  *
- * @ioc_inode: 
+ * @ioc_inode:
  * @offset:
  *
  */
@@ -190,30 +191,29 @@ ioc_page_create (ioc_inode_t *ioc_inode, off_t offset)
 	ioc_page_t  *page           = NULL;
 	off_t        rounded_offset = 0;
 	ioc_page_t  *newpage        = NULL;
-  
+
         table = ioc_inode->table;
         rounded_offset = floor (offset, table->page_size);
 
-        newpage = CALLOC (1, sizeof (*newpage));
+        newpage = GF_CALLOC (1, sizeof (*newpage),
+                             gf_ioc_mt_ioc_newpage_t);
         if (newpage == NULL) {
                 goto out;
         }
 
-	if (ioc_inode) {
-		table = ioc_inode->table;
-        } else {
-                free (newpage);
+	if (!ioc_inode) {
+                GF_FREE (newpage);
                 newpage = NULL;
                 goto out;
 	}
-   
+
 	newpage->offset = rounded_offset;
 	newpage->inode = ioc_inode;
 	pthread_mutex_init (&newpage->page_lock, NULL);
 
         rbthash_insert (ioc_inode->cache.page_table, newpage, &rounded_offset,
                         sizeof (rounded_offset));
- 
+
 	list_add_tail (&newpage->page_lru, &ioc_inode->cache.page_lru);
 
 	page = newpage;
@@ -241,7 +241,7 @@ ioc_wait_on_page (ioc_page_t *page, call_frame_t *frame, off_t offset,
 	ioc_waitq_t *waitq = NULL;
 	ioc_local_t *local = frame->local;
 
-	waitq = CALLOC (1, sizeof (*waitq));
+	waitq = GF_CALLOC (1, sizeof (*waitq), gf_ioc_mt_ioc_waitq_t);
         if (waitq == NULL) {
                 local->op_ret = -1;
                 local->op_errno = ENOMEM;
@@ -282,18 +282,18 @@ out:
  * assumes ioc_inode is locked
  */
 int8_t
-ioc_cache_still_valid (ioc_inode_t *ioc_inode, struct stat *stbuf)
+ioc_cache_still_valid (ioc_inode_t *ioc_inode, struct iatt *stbuf)
 {
 	int8_t cache_still_valid = 1;
 
 #if 0
-	if (!stbuf || (stbuf->st_mtime != ioc_inode->cache.mtime) || 
+	if (!stbuf || (stbuf->ia_mtime != ioc_inode->cache.mtime) || 
 	    (stbuf->st_mtim.tv_nsec != ioc_inode->stbuf.st_mtim.tv_nsec))
 		cache_still_valid = 0;
 
 #else
-	if (!stbuf || (stbuf->st_mtime != ioc_inode->cache.mtime)
-            || (ST_MTIM_NSEC(stbuf) != ioc_inode->cache.mtime_nsec))
+	if (!stbuf || (stbuf->ia_mtime != ioc_inode->cache.mtime)
+            || (stbuf->ia_mtime_nsec != ioc_inode->cache.mtime_nsec))
 		cache_still_valid = 0;
 
 #endif
@@ -302,7 +302,7 @@ ioc_cache_still_valid (ioc_inode_t *ioc_inode, struct stat *stbuf)
 	/* talk with avati@gluster.com to enable this section */
 	if (!ioc_inode->mtime && stbuf) {
 		cache_still_valid = 1;
-		ioc_inode->mtime = stbuf->st_mtime;
+		ioc_inode->mtime = stbuf->ia_mtime;
 	}
 #endif
 
@@ -322,7 +322,7 @@ ioc_waitq_return (ioc_waitq_t *waitq)
 
 		frame = trav->data;
 		ioc_frame_return (frame);
-		free (trav);
+		GF_FREE (trav);
 	}
 }
 
@@ -330,15 +330,13 @@ ioc_waitq_return (ioc_waitq_t *waitq)
 int
 ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno, struct iovec *vector,
-	       int32_t count, struct stat *stbuf, struct iobref *iobref)
+	       int32_t count, struct iatt *stbuf, struct iobref *iobref)
 {
 	ioc_local_t *local = NULL;
 	off_t       offset = 0;
 	ioc_inode_t *ioc_inode = NULL;
 	ioc_table_t *table = NULL;
 	ioc_page_t  *page = NULL;
-	off_t       trav_offset = 0;
-	size_t      payload_size = 0;
 	int32_t     destroy_size = 0;
 	size_t      page_size = 0;
 	ioc_waitq_t *waitq = NULL;
@@ -350,11 +348,8 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         ioc_inode = local->inode;
         table = ioc_inode->table;
 
-	trav_offset = offset;
-	payload_size = op_ret;
-
         zero_filled = ((op_ret >=0)
-                       && (stbuf->st_mtime == 0));
+                       && (stbuf->ia_mtime == 0));
 
 	ioc_inode_lock (ioc_inode);
 	{
@@ -368,8 +363,8 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		}
 
 		if ((op_ret >= 0) && !zero_filled) {
-			ioc_inode->cache.mtime = stbuf->st_mtime;
-                        ST_MTIM_NSEC_SET(stbuf, ioc_inode->cache.mtime_nsec);
+			ioc_inode->cache.mtime = stbuf->ia_mtime;
+                        ioc_inode->cache.mtime_nsec = stbuf->ia_mtime_nsec;
                 }
 
 		gettimeofday (&ioc_inode->cache.tv, NULL);
@@ -394,10 +389,10 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			} else {
 				if (page->vector) {
 					iobref_unref (page->iobref);
-					free (page->vector);
+					GF_FREE (page->vector);
 					page->vector = NULL;
 				}
-	
+
 				/* keep a copy of the page for our cache */
 				page->vector = iov_dup (vector, count);
                                 if (page->vector == NULL) {
@@ -406,8 +401,6 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                                 waitq = ioc_page_error (page,
                                                                         -1, 
                                                                         ENOMEM);
-                                        op_ret = -1;
-                                        op_errno = ENOMEM;
                                         goto unlock;
                                 }
 
@@ -420,7 +413,7 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 					gf_log (this->name, GF_LOG_CRITICAL,
 						"frame>root>rsp_refs is null");
 				} /* if(frame->root->rsp_refs) */
-	
+
 				/* page->size should indicate exactly how 
 				 * much the readv call to the child
 				 * translator returned. earlier op_ret 
@@ -429,7 +422,6 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 				 * io-cached volume were resulting in 0 
 				 * byte replies */
 				page_size = iov_length(vector, count);
-	
 				page->size = page_size;
 
                                 iobref_page_size = iobref_size (page->iobref);
@@ -507,7 +499,8 @@ ioc_page_fault (ioc_inode_t *ioc_inode,	call_frame_t *frame, fd_t *fd,
                 goto err;
         }
 
-        fault_local = CALLOC (1, sizeof (ioc_local_t));
+        fault_local = GF_CALLOC (1, sizeof (ioc_local_t),
+                                gf_ioc_mt_ioc_local_t);
         if (fault_local == NULL) {
                 op_ret = -1;
                 op_errno = ENOMEM;
@@ -562,13 +555,13 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
         ioc_fill_t  *new = NULL;
         int8_t      found = 0;
         int32_t     ret = 0;
-  
+
         local = frame->local;
         ioc_inode = page->inode;
 
 	gf_log (frame->this->name, GF_LOG_TRACE,
 		"frame (%p) offset = %"PRId64" && size = %"GF_PRI_SIZET" "
-		"&& page->size = %"GF_PRI_SIZET" && wait_count = %d", 
+		"&& page->size = %"GF_PRI_SIZET" && wait_count = %d",
 		frame, offset, size, page->size, local->wait_count);
 
 	/* immediately move this page to the end of the page_lru list */
@@ -576,17 +569,17 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 	/* fill local->pending_size bytes from local->pending_offset */
 	if (local->op_ret != -1 && page->size) {
 		if (offset > page->offset)
-			/* offset is offset in file, convert it to offset in 
+			/* offset is offset in file, convert it to offset in
 			 * page */
 			src_offset = offset - page->offset;
-		/*FIXME: since offset is the offset within page is the 
+		/*FIXME: since offset is the offset within page is the
 		 * else case valid? */
 		else
 			/* local->pending_offset is in previous page. do not
 			 * fill until we have filled all previous pages */
 			dst_offset = page->offset - offset;
 
-		/* we have to copy from offset to either end of this page 
+		/* we have to copy from offset to either end of this page
 		 * or till the requested size */
 		copy_size = min (page->size - src_offset,
 				 size - dst_offset);
@@ -596,14 +589,15 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 			   is beyond the page size in the page */
 			copy_size = src_offset = 0;
 		}
-    
+
 		gf_log (page->inode->table->xl->name, GF_LOG_TRACE,
 			"copy_size = %"GF_PRI_SIZET" && src_offset = "
 			"%"PRId64" && dst_offset = %"PRId64"",
 			copy_size, src_offset, dst_offset);
 
 		{
-                        new = CALLOC (1, sizeof (*new));
+                        new = GF_CALLOC (1, sizeof (*new),
+                                         gf_ioc_mt_ioc_fill_t);
                         if (new == NULL) {
                                 local->op_ret = -1;
                                 local->op_errno = ENOMEM;
@@ -622,14 +616,15 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 						 src_offset + copy_size,
 						 NULL);
 
-			new->vector = CALLOC (new->count, 
-					      sizeof (struct iovec));
+			new->vector = GF_CALLOC (new->count,
+					         sizeof (struct iovec),
+                                                 gf_ioc_mt_iovec);
                         if (new->vector == NULL) {
                                 local->op_ret = -1;
                                 local->op_errno = ENOMEM;
 
                                 iobref_unref (new->iobref);
-                                FREE (new);
+                                GF_FREE (new);
 
                                 ret = -1;
                                 gf_log (page->inode->table->xl->name,
@@ -647,15 +642,15 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 
 			/* add the ioc_fill to fill_list for this frame */
 			if (list_empty (&local->fill_list)) {
-				/* if list is empty, then this is the first 
-				 * time we are filling frame, add the 
+				/* if list is empty, then this is the first
+				 * time we are filling frame, add the
 				 * ioc_fill_t to the end of list */
 				list_add_tail (&new->list, &local->fill_list);
 			} else {
                                 found = 0;
-				/* list is not empty, we need to look for 
+				/* list is not empty, we need to look for
 				 * where this offset fits in list */
-				list_for_each_entry (fill, &local->fill_list, 
+				list_for_each_entry (fill, &local->fill_list,
 						     list) {
 					if (fill->offset > new->offset) {
 						found = 1;
@@ -664,11 +659,10 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 				}
 
 				if (found) {
-					found = 0;
-					list_add_tail (&new->list, 
+					list_add_tail (&new->list,
 						       &fill->list);
 				} else {
-					list_add_tail (&new->list, 
+					list_add_tail (&new->list,
 						       &local->fill_list);
 				}
 			}
@@ -698,8 +692,8 @@ ioc_frame_unwind (call_frame_t *frame)
 	struct iovec  *vector = NULL;
 	int32_t       copied = 0;
 	struct iobref *iobref = NULL;
-	struct stat   stbuf = {0,};
-	int32_t       op_ret = 0, op_errno = 0;
+	struct iatt   stbuf = {0,};
+	int32_t       op_ret = 0;
 
         local = frame->local;
 	//  ioc_local_lock (local);
@@ -707,7 +701,6 @@ ioc_frame_unwind (call_frame_t *frame)
 	iobref = iobref_new ();
         if (iobref == NULL) {
                 op_ret = -1;
-                op_errno = ENOMEM;
                 gf_log (frame->this->name, GF_LOG_ERROR, "out of memory");
         }
 
@@ -722,20 +715,19 @@ ioc_frame_unwind (call_frame_t *frame)
 		count += fill->count;
 	}
 
-	vector = CALLOC (count, sizeof (*vector));
+	vector = GF_CALLOC (count, sizeof (*vector), gf_ioc_mt_iovec);
         if (vector == NULL) {
                 op_ret = -1;
-                op_errno = ENOMEM;
-                
+
                 gf_log (frame->this->name, GF_LOG_ERROR, "out of memory");
         }
-  
+
 	list_for_each_entry_safe (fill, next, &local->fill_list, list) {
-                if ((vector != NULL) &&  (iobref != NULL)) { 
+                if ((vector != NULL) &&  (iobref != NULL)) {
                         memcpy (((char *)vector) + copied,
                                 fill->vector,
                                 fill->count * sizeof (*vector));
-    
+
                         copied += (fill->count * sizeof (*vector));
 
                         iobref_merge (iobref, fill->iobref);
@@ -743,10 +735,10 @@ ioc_frame_unwind (call_frame_t *frame)
 
 		list_del (&fill->list);
 		iobref_unref (fill->iobref);
-		free (fill->vector);
-		free (fill);
+		GF_FREE (fill->vector);
+		GF_FREE (fill);
 	}
-  
+
         if (op_ret != -1) {
                 op_ret = iov_length (vector, count);
         }
@@ -762,14 +754,14 @@ ioc_frame_unwind (call_frame_t *frame)
         if (iobref != NULL) {
                 iobref_unref (iobref);
         }
-        
+
         if (vector != NULL) {
-                free (vector);
+                GF_FREE (vector);
                 vector = NULL;
         }
-    
+
 	pthread_mutex_destroy (&local->local_lock);
-	free (local);
+	GF_FREE (local);
 
         return;
 }
@@ -787,7 +779,7 @@ ioc_frame_return (call_frame_t *frame)
 	int32_t wait_count = 0;
 
         local = frame->local;
-        assert (local->wait_count > 0);
+        GF_ASSERT (local->wait_count > 0);
 
 	ioc_local_lock (local);
 	{
@@ -797,12 +789,12 @@ ioc_frame_return (call_frame_t *frame)
 
 	if (!wait_count) {
 		ioc_frame_unwind (frame);
-	} 
+	}
 
 	return;
 }
 
-/* 
+/*
  * ioc_page_wakeup -
  * @page:
  *
@@ -813,20 +805,19 @@ ioc_page_wakeup (ioc_page_t *page)
 {
 	ioc_waitq_t  *waitq = NULL, *trav = NULL;
 	call_frame_t *frame = NULL;
-        int32_t      ret = -1; 
+        int32_t      ret = -1;
 
 	waitq = page->waitq;
 	page->waitq = NULL;
 
-	trav = waitq;
 	page->ready = 1;
 
 	gf_log (page->inode->table->xl->name, GF_LOG_TRACE,
 		"page is %p && waitq = %p", page, waitq);
-  
+
 	for (trav = waitq; trav; trav = trav->next) {
-		frame = trav->data; 
-		ret = ioc_frame_fill (page, frame, trav->pending_offset, 
+		frame = trav->data;
+		ret = ioc_frame_fill (page, frame, trav->pending_offset,
                                       trav->pending_size);
                 if (ret == -1) {
                         break;
