@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -62,7 +62,8 @@ ra_page_create (ra_file_t *file, off_t offset)
 		page = page->next;
 
 	if (page == &file->pages || page->offset != rounded_offset) {
-		newpage = CALLOC (1, sizeof (*newpage));
+		newpage = GF_CALLOC (1, sizeof (*newpage),
+                                     gf_ra_mt_ra_page_t);
 		if (!newpage)
 			return NULL;
 
@@ -87,7 +88,8 @@ ra_wait_on_page (ra_page_t *page, call_frame_t *frame)
 	ra_local_t *local = NULL;
 
 	local = frame->local;
-	waitq = CALLOC (1, sizeof (*waitq));
+	waitq = GF_CALLOC (1, sizeof (*waitq),
+                           gf_ra_mt_ra_waitq_t);
 	if (!waitq) {
 		gf_log (frame->this->name, GF_LOG_ERROR,
 			"out of memory");
@@ -123,7 +125,7 @@ ra_waitq_return (ra_waitq_t *waitq)
 
 		frame = trav->data;
 		ra_frame_return (frame);
-		free (trav);
+		GF_FREE (trav);
 	}
 }
 
@@ -131,28 +133,23 @@ ra_waitq_return (ra_waitq_t *waitq)
 int
 ra_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	      int32_t op_ret, int32_t op_errno, struct iovec *vector,
-	      int32_t count, struct stat *stbuf, struct iobref *iobref)
+	      int32_t count, struct iatt *stbuf, struct iobref *iobref)
 {
 	ra_local_t   *local = NULL;
 	off_t        pending_offset = 0;
 	ra_file_t    *file = NULL;
 	ra_page_t    *page = NULL;
-	off_t        trav_offset = 0;
-	size_t       payload_size = 0;
 	ra_waitq_t   *waitq = NULL;
 	fd_t         *fd = NULL;
-	int          ret = 0;
 	uint64_t     tmp_file = 0;
 
 	local = frame->local;
 	fd  = local->fd;
 
-	ret = fd_ctx_get (fd, this, &tmp_file);
+	fd_ctx_get (fd, this, &tmp_file);
 
 	file = (ra_file_t *)(long)tmp_file;
 	pending_offset = local->pending_offset;
-	trav_offset    = pending_offset;  
-	payload_size   = op_ret;
 
 	ra_file_lock (file);
 	{
@@ -176,7 +173,7 @@ ra_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 		if (page->vector) {
 			iobref_unref (page->iobref);
-			free (page->vector);
+			GF_FREE (page->vector);
 		}
 
 		page->vector = iov_dup (vector, count);
@@ -200,7 +197,7 @@ unlock:
 
 	fd_unref (local->fd);
 
-	free (frame->local);
+	GF_FREE (frame->local);
 	frame->local = NULL;
 
 	STACK_DESTROY (frame->root);
@@ -212,12 +209,11 @@ void
 ra_page_fault (ra_file_t *file, call_frame_t *frame, off_t offset)
 {
 	call_frame_t *fault_frame = NULL;
-	ra_local_t   *fault_local = NULL, *local = NULL;
+	ra_local_t   *fault_local = NULL;
         ra_page_t    *page = NULL;
         ra_waitq_t   *waitq = NULL;
         int32_t      op_ret = -1, op_errno = -1; 
-    
-        local = frame->local;
+
 	fault_frame = copy_frame (frame);
         if (fault_frame == NULL) {
                 op_ret = -1;
@@ -225,7 +221,8 @@ ra_page_fault (ra_file_t *file, call_frame_t *frame, off_t offset)
                 goto err;
         }
 
-	fault_local = CALLOC (1, sizeof (ra_local_t));
+	fault_local = GF_CALLOC (1, sizeof (ra_local_t),
+                                 gf_ra_mt_ra_local_t);
         if (fault_local == NULL) {
                 STACK_DESTROY (fault_frame->root);
                 op_ret = -1;
@@ -297,7 +294,8 @@ ra_frame_fill (ra_page_t *page, call_frame_t *frame)
 			fill = fill->next;
 		}
 
-		new = CALLOC (1, sizeof (*new));
+		new = GF_CALLOC (1, sizeof (*new),
+                                gf_ra_mt_ra_fill_t);
                 if (new == NULL) {
                         local->op_ret = -1;
                         local->op_errno = ENOMEM;
@@ -310,11 +308,12 @@ ra_frame_fill (ra_page_t *page, call_frame_t *frame)
 		new->count = iov_subset (page->vector, page->count,
 					 src_offset, src_offset+copy_size,
 					 NULL);
-		new->vector = CALLOC (new->count, sizeof (struct iovec));
+		new->vector = GF_CALLOC (new->count, sizeof (struct iovec),
+                                         gf_ra_mt_iovec);
                 if (new->vector == NULL) {
                         local->op_ret = -1;
                         local->op_errno = ENOMEM;
-                        FREE (new);
+                        GF_FREE (new);
                         goto out;
                 }
 
@@ -347,7 +346,6 @@ ra_frame_unwind (call_frame_t *frame)
 	ra_fill_t     *next = NULL;
 	fd_t          *fd = NULL;
 	ra_file_t     *file = NULL;
-	int           ret = 0;
 	uint64_t      tmp_file = 0;
 
 	local = frame->local;
@@ -366,7 +364,8 @@ ra_frame_unwind (call_frame_t *frame)
 		fill = fill->next;
 	}
 
-	vector = CALLOC (count, sizeof (*vector));
+	vector = GF_CALLOC (count, sizeof (*vector),
+                            gf_ra_mt_iovec);
         if (vector == NULL) {
                 local->op_ret = -1;
                 local->op_errno = ENOMEM;
@@ -391,14 +390,14 @@ ra_frame_unwind (call_frame_t *frame)
 		fill->prev->next = fill->prev;
 
 		iobref_unref (fill->iobref);
-		free (fill->vector);
-		free (fill);
+		GF_FREE (fill->vector);
+		GF_FREE (fill);
 
 		fill = next;
 	}
 
 	fd = local->fd;
-	ret = fd_ctx_get (fd, frame->this, &tmp_file);
+	fd_ctx_get (fd, frame->this, &tmp_file);
 	file = (ra_file_t *)(long)tmp_file;
 
 	STACK_UNWIND_STRICT (readv, frame, local->op_ret, local->op_errno,
@@ -406,8 +405,8 @@ ra_frame_unwind (call_frame_t *frame)
 
 	iobref_unref (iobref);
 	pthread_mutex_destroy (&local->local_lock);
-	free (local);
-	free (vector);
+	GF_FREE (local);
+	GF_FREE (vector);
 
 	return;
 }
@@ -424,7 +423,7 @@ ra_frame_return (call_frame_t *frame)
 	int32_t     wait_count = 0;
 
 	local = frame->local;
-	assert (local->wait_count > 0);
+	GF_ASSERT (local->wait_count > 0);
 
 	ra_local_lock (local);
 	{
@@ -452,7 +451,6 @@ ra_page_wakeup (ra_page_t *page)
 	waitq = page->waitq;
 	page->waitq = NULL;
 
-	trav = waitq;
 	for (trav = waitq; trav; trav = trav->next) {
 		frame = trav->data;
 		ra_frame_fill (page, frame);
@@ -475,8 +473,8 @@ ra_page_purge (ra_page_t *page)
 	if (page->iobref) {
 		iobref_unref (page->iobref);
 	}
-	free (page->vector);
-	free (page);
+	GF_FREE (page->vector);
+	GF_FREE (page);
 }
 
 /*
@@ -498,7 +496,6 @@ ra_page_error (ra_page_t *page, int32_t op_ret, int32_t op_errno)
 	waitq = page->waitq;
 	page->waitq = NULL;
 
-	trav = waitq;
 	for (trav = waitq; trav; trav = trav->next) {
 		frame = trav->data;
 
@@ -541,5 +538,5 @@ ra_file_destroy (ra_file_t *file)
 	}
 
 	pthread_mutex_destroy (&file->file_lock);
-	free (file);
+	GF_FREE (file);
 }
