@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -59,7 +59,7 @@ ra_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         wbflags = (long)frame->local;
 
-	file = CALLOC (1, sizeof (*file));
+	file = GF_CALLOC (1, sizeof (*file), gf_ra_mt_ra_file_t);
 	if (!file) {
                 op_ret = -1;
                 op_errno = ENOMEM;
@@ -68,12 +68,6 @@ ra_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		goto unwind;
 	}
 
-	/* If mandatory locking has been enabled on this file,
-	   we disable caching on it */
-
-	if ((fd->inode->st_mode & S_ISGID) && !(fd->inode->st_mode & S_IXGRP))
-		file->disabled = 1;
-
 	/* If O_DIRECT open, we disable caching on it */
 
 	if ((fd->flags & O_DIRECT) || ((fd->flags & O_ACCMODE) == O_WRONLY))
@@ -114,7 +108,7 @@ ra_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 op_ret = -1;
                 op_errno = ENOMEM;
         }
-
+        
 unwind:
         frame->local = NULL;
 
@@ -127,8 +121,8 @@ unwind:
 int
 ra_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno, fd_t *fd, inode_t *inode,
-               struct stat *buf, struct stat *preparent,
-               struct stat *postparent)
+               struct iatt *buf, struct iatt *preparent,
+               struct iatt *postparent)
 {
 	ra_conf_t  *conf = NULL;
 	ra_file_t  *file = NULL;
@@ -140,7 +134,7 @@ ra_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		goto unwind;
 	}
 
-	file = CALLOC (1, sizeof (*file));
+	file = GF_CALLOC (1, sizeof (*file), gf_ra_mt_ra_file_t);
 	if (!file) {
                 op_ret = -1;
                 op_errno = ENOMEM;
@@ -149,19 +143,13 @@ ra_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		goto unwind;
 	}
 
-	/* If mandatory locking has been enabled on this file,
-	   we disable caching on it */
-
-	if ((fd->inode->st_mode & S_ISGID) && !(fd->inode->st_mode & S_IXGRP))
-		file->disabled = 1;
-
 	/* If O_DIRECT open, we disable caching on it */
 
 	if ((fd->flags & O_DIRECT) || ((fd->flags & O_ACCMODE) == O_WRONLY))
 			file->disabled = 1;
 
 	file->offset = (unsigned long long) 0;
-	//file->size = fd->inode->buf.st_size;
+	//file->size = fd->inode->buf.ia_size;
 	file->conf = conf;
 	file->pages.next = &file->pages;
 	file->pages.prev = &file->pages;
@@ -213,12 +201,12 @@ ra_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
 
 int
 ra_create (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
-           mode_t mode, fd_t *fd)
+           mode_t mode, fd_t *fd, dict_t *params)
 {
 	STACK_WIND (frame, ra_create_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->create,
-		    loc, flags, mode, fd);
+		    loc, flags, mode, fd, params);
 
 	return 0;
 }
@@ -303,7 +291,6 @@ read_ahead (call_frame_t *frame, ra_file_t *file)
 
 	trav_offset = ra_offset;
 
-	trav = file->pages.next;
 	cap  = file->size ? file->size : ra_offset + ra_size;
 
 	while (trav_offset < min(ra_offset + ra_size, cap)) {
@@ -340,7 +327,7 @@ read_ahead (call_frame_t *frame, ra_file_t *file)
 int
 ra_need_atime_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                    int32_t op_ret, int32_t op_errno, struct iovec *vector,
-                   int32_t count, struct stat *stbuf, struct iobref *iobref)
+                   int32_t count, struct iatt *stbuf, struct iobref *iobref)
 {
 	STACK_DESTROY (frame->root);
 	return 0;
@@ -367,7 +354,6 @@ dispatch_requests (call_frame_t *frame, ra_file_t *file)
 	rounded_end    = roof (local->offset + local->size, file->page_size);
 
 	trav_offset = rounded_offset;
-	trav        = file->pages.next;
 
 	while (trav_offset < rounded_end) {
 		fault = 0;
@@ -435,7 +421,7 @@ out:
 int
 ra_readv_disabled_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                        int32_t op_ret, int32_t op_errno, struct iovec *vector,
-                       int32_t count, struct stat *stbuf, struct iobref *iobref)
+                       int32_t count, struct iatt *stbuf, struct iobref *iobref)
 {
 	STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, vector, count,
                              stbuf, iobref);
@@ -452,7 +438,6 @@ ra_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
 	ra_local_t   *local = NULL;
 	ra_conf_t    *conf = NULL;
 	int          op_errno = 0;
-	int          ret = 0;
 	char         expected_offset = 1;
 	uint64_t     tmp_file = 0;
 
@@ -462,7 +447,7 @@ ra_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
 		"NEW REQ at offset=%"PRId64" for size=%"GF_PRI_SIZET"",
 		offset, size);
 
-	ret = fd_ctx_get (fd, this, &tmp_file);
+	fd_ctx_get (fd, this, &tmp_file);
 	file = (ra_file_t *)(long)tmp_file;
 
         if (file == NULL) {
@@ -502,7 +487,8 @@ ra_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
 		return 0;
 	}
 
-	local = (void *) CALLOC (1, sizeof (*local));
+	local = (void *) GF_CALLOC (1, sizeof (*local),
+                                    gf_ra_mt_ra_local_t);
 	if (!local) {
 		gf_log (this->name, GF_LOG_ERROR,
 			"out of memory");
@@ -553,7 +539,7 @@ ra_flush_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
 
 int
 ra_fsync_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
-              int32_t op_errno, struct stat *prebuf, struct stat *postbuf)
+              int32_t op_errno, struct iatt *prebuf, struct iatt *postbuf)
 {
 	STACK_UNWIND_STRICT (fsync, frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
@@ -564,11 +550,10 @@ int
 ra_flush (call_frame_t *frame, xlator_t *this, fd_t *fd)
 {
 	ra_file_t *file = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
         int32_t   op_errno = 0;
 
-	ret = fd_ctx_get (fd, this, &tmp_file);
+	fd_ctx_get (fd, this, &tmp_file);
 	file = (ra_file_t *)(long)tmp_file;
         if (file == NULL) {
                 op_errno = EBADF;
@@ -595,11 +580,10 @@ int
 ra_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t datasync)
 {
 	ra_file_t *file = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
         int32_t   op_errno = 0;
 
-	ret = fd_ctx_get (fd, this, &tmp_file);
+	fd_ctx_get (fd, this, &tmp_file);
 	file = (ra_file_t *)(long)tmp_file;
         if (file == NULL) {
                 op_errno = EBADF;
@@ -626,17 +610,16 @@ unwind:
 
 int
 ra_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-               int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-               struct stat *postbuf)
+               int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+               struct iatt *postbuf)
 {
 	fd_t      *fd = NULL;
 	ra_file_t *file = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
 
 	fd = frame->local;
 
-	ret = fd_ctx_get (fd, this, &tmp_file);
+	fd_ctx_get (fd, this, &tmp_file);
 	file = (ra_file_t *)(long)tmp_file;
 
         flush_region (frame, file, 0, file->pages.prev->offset+1);
@@ -652,11 +635,10 @@ ra_writev (call_frame_t *frame, xlator_t *this, fd_t *fd, struct iovec *vector,
            int32_t count, off_t offset, struct iobref *iobref)
 {
 	ra_file_t *file = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
         int32_t   op_errno = 0;
 
-	ret = fd_ctx_get (fd, this, &tmp_file);
+	fd_ctx_get (fd, this, &tmp_file);
 	file = (ra_file_t *)(long)tmp_file;
         if (file == NULL) {
                 op_errno = EBADF;
@@ -687,8 +669,8 @@ unwind:
 
 int
 ra_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                 int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                 struct stat *postbuf)
+                 int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                 struct iatt *postbuf)
 {
 	STACK_UNWIND_STRICT (truncate, frame, op_ret, op_errno, prebuf,
                              postbuf);
@@ -698,7 +680,7 @@ ra_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 ra_attr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-	     int32_t op_ret, int32_t op_errno, struct stat *buf)
+	     int32_t op_ret, int32_t op_errno, struct iatt *buf)
 {
 	STACK_UNWIND_STRICT (stat, frame, op_ret, op_errno, buf);
 	return 0;
@@ -711,7 +693,6 @@ ra_truncate (call_frame_t *frame, xlator_t *this, loc_t *loc, off_t offset)
 	ra_file_t *file = NULL;
 	fd_t      *iter_fd = NULL;
 	inode_t   *inode = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
 
 	inode = loc->inode;
@@ -719,7 +700,7 @@ ra_truncate (call_frame_t *frame, xlator_t *this, loc_t *loc, off_t offset)
 	LOCK (&inode->lock);
 	{
 		list_for_each_entry (iter_fd, &inode->fd_list, inode_list) {
-			ret = fd_ctx_get (iter_fd, this, &tmp_file);
+			fd_ctx_get (iter_fd, this, &tmp_file);
 			file = (ra_file_t *)(long)tmp_file;
 
 			if (!file)
@@ -744,7 +725,6 @@ ra_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd)
 	ra_file_t *file = NULL;
 	fd_t      *iter_fd = NULL;
 	inode_t   *inode = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
 
 	inode = fd->inode;
@@ -752,7 +732,7 @@ ra_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd)
 	LOCK (&inode->lock);
 	{
 		list_for_each_entry (iter_fd, &inode->fd_list, inode_list) {
-			ret = fd_ctx_get (iter_fd, this, &tmp_file);
+			fd_ctx_get (iter_fd, this, &tmp_file);
 			file = (ra_file_t *)(long)tmp_file;
 
 			if (!file)
@@ -777,7 +757,6 @@ ra_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset)
 	ra_file_t *file = NULL;
 	fd_t      *iter_fd = NULL;
 	inode_t   *inode = NULL;
-	int       ret = 0;
 	uint64_t  tmp_file = 0;
 
 	inode = fd->inode;
@@ -785,7 +764,7 @@ ra_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset)
 	LOCK (&inode->lock);
 	{
 		list_for_each_entry (iter_fd, &inode->fd_list, inode_list) {
-			ret = fd_ctx_get (iter_fd, this, &tmp_file);
+			fd_ctx_get (iter_fd, this, &tmp_file);
 			file = (ra_file_t *)(long)tmp_file;
 			if (!file)
 				continue;
@@ -845,6 +824,25 @@ ra_priv_dump (xlator_t *this)
         return 0;
 }
 
+int32_t
+mem_acct_init (xlator_t *this)
+{
+        int     ret = -1;
+
+        if (!this)
+                return ret;
+
+        ret = xlator_mem_acct_init (this, gf_ra_mt_end + 1);
+        
+        if (ret != 0) {
+                gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
+                                "failed");
+                return ret;
+        }
+
+        return ret;
+}
+
 int
 init (xlator_t *this)
 {
@@ -865,7 +863,8 @@ init (xlator_t *this)
 			"dangling volume. check volfile ");
 	}
  
-	conf = (void *) CALLOC (1, sizeof (*conf));
+	conf = (void *) GF_CALLOC (1, sizeof (*conf),
+                                   gf_ra_mt_ra_conf_t);
         if (conf == NULL) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "FATAL: Out of memory");
@@ -919,7 +918,7 @@ init (xlator_t *this)
 out:
         if (ret == -1) {
                 if (conf != NULL) {
-                        FREE (conf);
+                        GF_FREE (conf);
                 }
         }
 
@@ -935,7 +934,7 @@ fini (xlator_t *this)
                 return;
 
 	pthread_mutex_destroy (&conf->conf_lock);
-	FREE (conf);
+	GF_FREE (conf);
 
 	this->private = NULL;
 	return;
@@ -953,9 +952,6 @@ struct xlator_fops fops = {
 	.fstat       = ra_fstat,
 };
 
-struct xlator_mops mops = {
-};
-
 struct xlator_cbks cbks = {
 	.release       = ra_release,
 };
