@@ -3,16 +3,16 @@
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -33,6 +33,7 @@
 #include "rbthash.h"
 #include "hashfn.h"
 #include "call-stub.h"
+#include "stat-prefetch-mem-types.h"
 #include <libgen.h>
 
 struct sp_cache {
@@ -73,7 +74,7 @@ struct sp_inode_ctx {
         char             need_unwind;
         int32_t          op_ret;
         int32_t          op_errno;
-        struct stat      stbuf;  
+        struct iatt      stbuf;  
         gf_lock_t        lock;
         struct list_head waiting_ops;
 };
@@ -81,6 +82,7 @@ typedef struct sp_inode_ctx sp_inode_ctx_t;
 
 struct sp_private {
         struct mem_pool  *mem_pool;
+        uint32_t         entries;
         gf_lock_t        lock;
 };
 typedef struct sp_private sp_private_t;
